#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 48
#define LARGE_STATE_COUNT 25
#define SYMBOL_COUNT 305
#define ALIAS_COUNT 0
#define TOKEN_COUNT 286
#define EXTERNAL_TOKEN_COUNT 1
#define FIELD_COUNT 6
#define MAX_ALIAS_SEQUENCE_LENGTH 5
#define PRODUCTION_ID_COUNT 7

enum ts_symbol_identifiers {
  sym_identifier = 1,
  anon_sym_PERCENT = 2,
  aux_sym_macro_token1 = 3,
  anon_sym_LBRACE = 4,
  anon_sym_RBRACE = 5,
  anon_sym_TILDE = 6,
  aux_sym_include_token1 = 7,
  anon_sym_BRK = 8,
  anon_sym_INC = 9,
  anon_sym_POP = 10,
  anon_sym_NIP = 11,
  anon_sym_SWP = 12,
  anon_sym_ROT = 13,
  anon_sym_DUP = 14,
  anon_sym_OVR = 15,
  anon_sym_EQU = 16,
  anon_sym_NEQ = 17,
  anon_sym_GTH = 18,
  anon_sym_LTH = 19,
  anon_sym_JMP = 20,
  anon_sym_JCN = 21,
  anon_sym_JSR = 22,
  anon_sym_STH = 23,
  anon_sym_LDZ = 24,
  anon_sym_STZ = 25,
  anon_sym_LDR = 26,
  anon_sym_STR = 27,
  anon_sym_LDA = 28,
  anon_sym_STA = 29,
  anon_sym_DEI = 30,
  anon_sym_DEO = 31,
  anon_sym_ADD = 32,
  anon_sym_SUB = 33,
  anon_sym_MUL = 34,
  anon_sym_DIV = 35,
  anon_sym_AND = 36,
  anon_sym_ORA = 37,
  anon_sym_EOR = 38,
  anon_sym_SFT = 39,
  anon_sym_JCI = 40,
  anon_sym_INC2 = 41,
  anon_sym_POP2 = 42,
  anon_sym_NIP2 = 43,
  anon_sym_SWP2 = 44,
  anon_sym_ROT2 = 45,
  anon_sym_DUP2 = 46,
  anon_sym_OVR2 = 47,
  anon_sym_EQU2 = 48,
  anon_sym_NEQ2 = 49,
  anon_sym_GTH2 = 50,
  anon_sym_LTH2 = 51,
  anon_sym_JMP2 = 52,
  anon_sym_JCN2 = 53,
  anon_sym_JSR2 = 54,
  anon_sym_STH2 = 55,
  anon_sym_LDZ2 = 56,
  anon_sym_STZ2 = 57,
  anon_sym_LDR2 = 58,
  anon_sym_STR2 = 59,
  anon_sym_LDA2 = 60,
  anon_sym_STA2 = 61,
  anon_sym_DEI2 = 62,
  anon_sym_DEO2 = 63,
  anon_sym_ADD2 = 64,
  anon_sym_SUB2 = 65,
  anon_sym_MUL2 = 66,
  anon_sym_DIV2 = 67,
  anon_sym_AND2 = 68,
  anon_sym_ORA2 = 69,
  anon_sym_EOR2 = 70,
  anon_sym_SFT2 = 71,
  anon_sym_JMI = 72,
  anon_sym_INCr = 73,
  anon_sym_POPr = 74,
  anon_sym_NIPr = 75,
  anon_sym_SWPr = 76,
  anon_sym_ROTr = 77,
  anon_sym_DUPr = 78,
  anon_sym_OVRr = 79,
  anon_sym_EQUr = 80,
  anon_sym_NEQr = 81,
  anon_sym_GTHr = 82,
  anon_sym_LTHr = 83,
  anon_sym_JMPr = 84,
  anon_sym_JCNr = 85,
  anon_sym_JSRr = 86,
  anon_sym_STHr = 87,
  anon_sym_LDZr = 88,
  anon_sym_STZr = 89,
  anon_sym_LDRr = 90,
  anon_sym_STRr = 91,
  anon_sym_LDAr = 92,
  anon_sym_STAr = 93,
  anon_sym_DEIr = 94,
  anon_sym_DEOr = 95,
  anon_sym_ADDr = 96,
  anon_sym_SUBr = 97,
  anon_sym_MULr = 98,
  anon_sym_DIVr = 99,
  anon_sym_ANDr = 100,
  anon_sym_ORAr = 101,
  anon_sym_EORr = 102,
  anon_sym_SFTr = 103,
  anon_sym_JSI = 104,
  anon_sym_INC2r = 105,
  anon_sym_POP2r = 106,
  anon_sym_NIP2r = 107,
  anon_sym_SWP2r = 108,
  anon_sym_ROT2r = 109,
  anon_sym_DUP2r = 110,
  anon_sym_OVR2r = 111,
  anon_sym_EQU2r = 112,
  anon_sym_NEQ2r = 113,
  anon_sym_GTH2r = 114,
  anon_sym_LTH2r = 115,
  anon_sym_JMP2r = 116,
  anon_sym_JCN2r = 117,
  anon_sym_JSR2r = 118,
  anon_sym_STH2r = 119,
  anon_sym_LDZ2r = 120,
  anon_sym_STZ2r = 121,
  anon_sym_LDR2r = 122,
  anon_sym_STR2r = 123,
  anon_sym_LDA2r = 124,
  anon_sym_STA2r = 125,
  anon_sym_DEI2r = 126,
  anon_sym_DEO2r = 127,
  anon_sym_ADD2r = 128,
  anon_sym_SUB2r = 129,
  anon_sym_MUL2r = 130,
  anon_sym_DIV2r = 131,
  anon_sym_AND2r = 132,
  anon_sym_ORA2r = 133,
  anon_sym_EOR2r = 134,
  anon_sym_SFT2r = 135,
  anon_sym_LIT = 136,
  anon_sym_INCk = 137,
  anon_sym_POPk = 138,
  anon_sym_NIPk = 139,
  anon_sym_SWPk = 140,
  anon_sym_ROTk = 141,
  anon_sym_DUPk = 142,
  anon_sym_OVRk = 143,
  anon_sym_EQUk = 144,
  anon_sym_NEQk = 145,
  anon_sym_GTHk = 146,
  anon_sym_LTHk = 147,
  anon_sym_JMPk = 148,
  anon_sym_JCNk = 149,
  anon_sym_JSRk = 150,
  anon_sym_STHk = 151,
  anon_sym_LDZk = 152,
  anon_sym_STZk = 153,
  anon_sym_LDRk = 154,
  anon_sym_STRk = 155,
  anon_sym_LDAk = 156,
  anon_sym_STAk = 157,
  anon_sym_DEIk = 158,
  anon_sym_DEOk = 159,
  anon_sym_ADDk = 160,
  anon_sym_SUBk = 161,
  anon_sym_MULk = 162,
  anon_sym_DIVk = 163,
  anon_sym_ANDk = 164,
  anon_sym_ORAk = 165,
  anon_sym_EORk = 166,
  anon_sym_SFTk = 167,
  anon_sym_LIT2 = 168,
  anon_sym_INC2k = 169,
  anon_sym_POP2k = 170,
  anon_sym_NIP2k = 171,
  anon_sym_SWP2k = 172,
  anon_sym_ROT2k = 173,
  anon_sym_DUP2k = 174,
  anon_sym_OVR2k = 175,
  anon_sym_EQU2k = 176,
  anon_sym_NEQ2k = 177,
  anon_sym_GTH2k = 178,
  anon_sym_LTH2k = 179,
  anon_sym_JMP2k = 180,
  anon_sym_JCN2k = 181,
  anon_sym_JSR2k = 182,
  anon_sym_STH2k = 183,
  anon_sym_LDZ2k = 184,
  anon_sym_STZ2k = 185,
  anon_sym_LDR2k = 186,
  anon_sym_STR2k = 187,
  anon_sym_LDA2k = 188,
  anon_sym_STA2k = 189,
  anon_sym_DEI2k = 190,
  anon_sym_DEO2k = 191,
  anon_sym_ADD2k = 192,
  anon_sym_SUB2k = 193,
  anon_sym_MUL2k = 194,
  anon_sym_DIV2k = 195,
  anon_sym_AND2k = 196,
  anon_sym_ORA2k = 197,
  anon_sym_EOR2k = 198,
  anon_sym_SFT2k = 199,
  anon_sym_LITr = 200,
  anon_sym_INCkr = 201,
  anon_sym_POPkr = 202,
  anon_sym_NIPkr = 203,
  anon_sym_SWPkr = 204,
  anon_sym_ROTkr = 205,
  anon_sym_DUPkr = 206,
  anon_sym_OVRkr = 207,
  anon_sym_EQUkr = 208,
  anon_sym_NEQkr = 209,
  anon_sym_GTHkr = 210,
  anon_sym_LTHkr = 211,
  anon_sym_JMPkr = 212,
  anon_sym_JCNkr = 213,
  anon_sym_JSRkr = 214,
  anon_sym_STHkr = 215,
  anon_sym_LDZkr = 216,
  anon_sym_STZkr = 217,
  anon_sym_LDRkr = 218,
  anon_sym_STRkr = 219,
  anon_sym_LDAkr = 220,
  anon_sym_STAkr = 221,
  anon_sym_DEIkr = 222,
  anon_sym_DEOkr = 223,
  anon_sym_ADDkr = 224,
  anon_sym_SUBkr = 225,
  anon_sym_MULkr = 226,
  anon_sym_DIVkr = 227,
  anon_sym_ANDkr = 228,
  anon_sym_ORAkr = 229,
  anon_sym_EORkr = 230,
  anon_sym_SFTkr = 231,
  anon_sym_LIT2r = 232,
  anon_sym_INC2kr = 233,
  anon_sym_POP2kr = 234,
  anon_sym_NIP2kr = 235,
  anon_sym_SWP2kr = 236,
  anon_sym_ROT2kr = 237,
  anon_sym_DUP2kr = 238,
  anon_sym_OVR2kr = 239,
  anon_sym_EQU2kr = 240,
  anon_sym_NEQ2kr = 241,
  anon_sym_GTH2kr = 242,
  anon_sym_LTH2kr = 243,
  anon_sym_JMP2kr = 244,
  anon_sym_JCN2kr = 245,
  anon_sym_JSR2kr = 246,
  anon_sym_STH2kr = 247,
  anon_sym_LDZ2kr = 248,
  anon_sym_STZ2kr = 249,
  anon_sym_LDR2kr = 250,
  anon_sym_STR2kr = 251,
  anon_sym_LDA2kr = 252,
  anon_sym_STA2kr = 253,
  anon_sym_DEI2kr = 254,
  anon_sym_DEO2kr = 255,
  anon_sym_ADD2kr = 256,
  anon_sym_SUB2kr = 257,
  anon_sym_MUL2kr = 258,
  anon_sym_DIV2kr = 259,
  anon_sym_AND2kr = 260,
  anon_sym_ORA2kr = 261,
  anon_sym_EOR2kr = 262,
  anon_sym_SFT2kr = 263,
  anon_sym_PIPE = 264,
  anon_sym_DOLLAR = 265,
  anon_sym_POUND = 266,
  aux_sym_hex_literal_token1 = 267,
  anon_sym_AT = 268,
  anon_sym_SLASH = 269,
  aux_sym_rune_token1 = 270,
  anon_sym_COMMA = 271,
  anon_sym__ = 272,
  anon_sym_DOT = 273,
  anon_sym_DASH = 274,
  anon_sym_SEMI = 275,
  anon_sym_EQ = 276,
  anon_sym_BANG = 277,
  anon_sym_QMARK = 278,
  anon_sym_AMP = 279,
  anon_sym_LBRACK = 280,
  anon_sym_RBRACK = 281,
  anon_sym_DQUOTE = 282,
  aux_sym_raw_ascii_token1 = 283,
  sym_number = 284,
  sym_comment = 285,
  sym_program = 286,
  sym_memory_execution = 287,
  sym_subroutine = 288,
  sym__non_toplevel_statement = 289,
  sym_macro = 290,
  sym_include = 291,
  sym_opcode = 292,
  sym_absolute_pad_operation = 293,
  sym_relative_pad_operation = 294,
  sym_hex_literal = 295,
  sym_label = 296,
  sym_sublabel_reference = 297,
  sym_rune = 298,
  sym_rune_char = 299,
  sym_brackets = 300,
  sym_raw_ascii = 301,
  aux_sym_program_repeat1 = 302,
  aux_sym_memory_execution_repeat1 = 303,
  aux_sym_rune_repeat1 = 304,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [anon_sym_PERCENT] = "%",
  [aux_sym_macro_token1] = "identifier",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_TILDE] = "~",
  [aux_sym_include_token1] = "include_token1",
  [anon_sym_BRK] = "BRK",
  [anon_sym_INC] = "INC",
  [anon_sym_POP] = "POP",
  [anon_sym_NIP] = "NIP",
  [anon_sym_SWP] = "SWP",
  [anon_sym_ROT] = "ROT",
  [anon_sym_DUP] = "DUP",
  [anon_sym_OVR] = "OVR",
  [anon_sym_EQU] = "EQU",
  [anon_sym_NEQ] = "NEQ",
  [anon_sym_GTH] = "GTH",
  [anon_sym_LTH] = "LTH",
  [anon_sym_JMP] = "JMP",
  [anon_sym_JCN] = "JCN",
  [anon_sym_JSR] = "JSR",
  [anon_sym_STH] = "STH",
  [anon_sym_LDZ] = "LDZ",
  [anon_sym_STZ] = "STZ",
  [anon_sym_LDR] = "LDR",
  [anon_sym_STR] = "STR",
  [anon_sym_LDA] = "LDA",
  [anon_sym_STA] = "STA",
  [anon_sym_DEI] = "DEI",
  [anon_sym_DEO] = "DEO",
  [anon_sym_ADD] = "ADD",
  [anon_sym_SUB] = "SUB",
  [anon_sym_MUL] = "MUL",
  [anon_sym_DIV] = "DIV",
  [anon_sym_AND] = "AND",
  [anon_sym_ORA] = "ORA",
  [anon_sym_EOR] = "EOR",
  [anon_sym_SFT] = "SFT",
  [anon_sym_JCI] = "JCI",
  [anon_sym_INC2] = "INC2",
  [anon_sym_POP2] = "POP2",
  [anon_sym_NIP2] = "NIP2",
  [anon_sym_SWP2] = "SWP2",
  [anon_sym_ROT2] = "ROT2",
  [anon_sym_DUP2] = "DUP2",
  [anon_sym_OVR2] = "OVR2",
  [anon_sym_EQU2] = "EQU2",
  [anon_sym_NEQ2] = "NEQ2",
  [anon_sym_GTH2] = "GTH2",
  [anon_sym_LTH2] = "LTH2",
  [anon_sym_JMP2] = "JMP2",
  [anon_sym_JCN2] = "JCN2",
  [anon_sym_JSR2] = "JSR2",
  [anon_sym_STH2] = "STH2",
  [anon_sym_LDZ2] = "LDZ2",
  [anon_sym_STZ2] = "STZ2",
  [anon_sym_LDR2] = "LDR2",
  [anon_sym_STR2] = "STR2",
  [anon_sym_LDA2] = "LDA2",
  [anon_sym_STA2] = "STA2",
  [anon_sym_DEI2] = "DEI2",
  [anon_sym_DEO2] = "DEO2",
  [anon_sym_ADD2] = "ADD2",
  [anon_sym_SUB2] = "SUB2",
  [anon_sym_MUL2] = "MUL2",
  [anon_sym_DIV2] = "DIV2",
  [anon_sym_AND2] = "AND2",
  [anon_sym_ORA2] = "ORA2",
  [anon_sym_EOR2] = "EOR2",
  [anon_sym_SFT2] = "SFT2",
  [anon_sym_JMI] = "JMI",
  [anon_sym_INCr] = "INCr",
  [anon_sym_POPr] = "POPr",
  [anon_sym_NIPr] = "NIPr",
  [anon_sym_SWPr] = "SWPr",
  [anon_sym_ROTr] = "ROTr",
  [anon_sym_DUPr] = "DUPr",
  [anon_sym_OVRr] = "OVRr",
  [anon_sym_EQUr] = "EQUr",
  [anon_sym_NEQr] = "NEQr",
  [anon_sym_GTHr] = "GTHr",
  [anon_sym_LTHr] = "LTHr",
  [anon_sym_JMPr] = "JMPr",
  [anon_sym_JCNr] = "JCNr",
  [anon_sym_JSRr] = "JSRr",
  [anon_sym_STHr] = "STHr",
  [anon_sym_LDZr] = "LDZr",
  [anon_sym_STZr] = "STZr",
  [anon_sym_LDRr] = "LDRr",
  [anon_sym_STRr] = "STRr",
  [anon_sym_LDAr] = "LDAr",
  [anon_sym_STAr] = "STAr",
  [anon_sym_DEIr] = "DEIr",
  [anon_sym_DEOr] = "DEOr",
  [anon_sym_ADDr] = "ADDr",
  [anon_sym_SUBr] = "SUBr",
  [anon_sym_MULr] = "MULr",
  [anon_sym_DIVr] = "DIVr",
  [anon_sym_ANDr] = "ANDr",
  [anon_sym_ORAr] = "ORAr",
  [anon_sym_EORr] = "EORr",
  [anon_sym_SFTr] = "SFTr",
  [anon_sym_JSI] = "JSI",
  [anon_sym_INC2r] = "INC2r",
  [anon_sym_POP2r] = "POP2r",
  [anon_sym_NIP2r] = "NIP2r",
  [anon_sym_SWP2r] = "SWP2r",
  [anon_sym_ROT2r] = "ROT2r",
  [anon_sym_DUP2r] = "DUP2r",
  [anon_sym_OVR2r] = "OVR2r",
  [anon_sym_EQU2r] = "EQU2r",
  [anon_sym_NEQ2r] = "NEQ2r",
  [anon_sym_GTH2r] = "GTH2r",
  [anon_sym_LTH2r] = "LTH2r",
  [anon_sym_JMP2r] = "JMP2r",
  [anon_sym_JCN2r] = "JCN2r",
  [anon_sym_JSR2r] = "JSR2r",
  [anon_sym_STH2r] = "STH2r",
  [anon_sym_LDZ2r] = "LDZ2r",
  [anon_sym_STZ2r] = "STZ2r",
  [anon_sym_LDR2r] = "LDR2r",
  [anon_sym_STR2r] = "STR2r",
  [anon_sym_LDA2r] = "LDA2r",
  [anon_sym_STA2r] = "STA2r",
  [anon_sym_DEI2r] = "DEI2r",
  [anon_sym_DEO2r] = "DEO2r",
  [anon_sym_ADD2r] = "ADD2r",
  [anon_sym_SUB2r] = "SUB2r",
  [anon_sym_MUL2r] = "MUL2r",
  [anon_sym_DIV2r] = "DIV2r",
  [anon_sym_AND2r] = "AND2r",
  [anon_sym_ORA2r] = "ORA2r",
  [anon_sym_EOR2r] = "EOR2r",
  [anon_sym_SFT2r] = "SFT2r",
  [anon_sym_LIT] = "LIT",
  [anon_sym_INCk] = "INCk",
  [anon_sym_POPk] = "POPk",
  [anon_sym_NIPk] = "NIPk",
  [anon_sym_SWPk] = "SWPk",
  [anon_sym_ROTk] = "ROTk",
  [anon_sym_DUPk] = "DUPk",
  [anon_sym_OVRk] = "OVRk",
  [anon_sym_EQUk] = "EQUk",
  [anon_sym_NEQk] = "NEQk",
  [anon_sym_GTHk] = "GTHk",
  [anon_sym_LTHk] = "LTHk",
  [anon_sym_JMPk] = "JMPk",
  [anon_sym_JCNk] = "JCNk",
  [anon_sym_JSRk] = "JSRk",
  [anon_sym_STHk] = "STHk",
  [anon_sym_LDZk] = "LDZk",
  [anon_sym_STZk] = "STZk",
  [anon_sym_LDRk] = "LDRk",
  [anon_sym_STRk] = "STRk",
  [anon_sym_LDAk] = "LDAk",
  [anon_sym_STAk] = "STAk",
  [anon_sym_DEIk] = "DEIk",
  [anon_sym_DEOk] = "DEOk",
  [anon_sym_ADDk] = "ADDk",
  [anon_sym_SUBk] = "SUBk",
  [anon_sym_MULk] = "MULk",
  [anon_sym_DIVk] = "DIVk",
  [anon_sym_ANDk] = "ANDk",
  [anon_sym_ORAk] = "ORAk",
  [anon_sym_EORk] = "EORk",
  [anon_sym_SFTk] = "SFTk",
  [anon_sym_LIT2] = "LIT2",
  [anon_sym_INC2k] = "INC2k",
  [anon_sym_POP2k] = "POP2k",
  [anon_sym_NIP2k] = "NIP2k",
  [anon_sym_SWP2k] = "SWP2k",
  [anon_sym_ROT2k] = "ROT2k",
  [anon_sym_DUP2k] = "DUP2k",
  [anon_sym_OVR2k] = "OVR2k",
  [anon_sym_EQU2k] = "EQU2k",
  [anon_sym_NEQ2k] = "NEQ2k",
  [anon_sym_GTH2k] = "GTH2k",
  [anon_sym_LTH2k] = "LTH2k",
  [anon_sym_JMP2k] = "JMP2k",
  [anon_sym_JCN2k] = "JCN2k",
  [anon_sym_JSR2k] = "JSR2k",
  [anon_sym_STH2k] = "STH2k",
  [anon_sym_LDZ2k] = "LDZ2k",
  [anon_sym_STZ2k] = "STZ2k",
  [anon_sym_LDR2k] = "LDR2k",
  [anon_sym_STR2k] = "STR2k",
  [anon_sym_LDA2k] = "LDA2k",
  [anon_sym_STA2k] = "STA2k",
  [anon_sym_DEI2k] = "DEI2k",
  [anon_sym_DEO2k] = "DEO2k",
  [anon_sym_ADD2k] = "ADD2k",
  [anon_sym_SUB2k] = "SUB2k",
  [anon_sym_MUL2k] = "MUL2k",
  [anon_sym_DIV2k] = "DIV2k",
  [anon_sym_AND2k] = "AND2k",
  [anon_sym_ORA2k] = "ORA2k",
  [anon_sym_EOR2k] = "EOR2k",
  [anon_sym_SFT2k] = "SFT2k",
  [anon_sym_LITr] = "LITr",
  [anon_sym_INCkr] = "INCkr",
  [anon_sym_POPkr] = "POPkr",
  [anon_sym_NIPkr] = "NIPkr",
  [anon_sym_SWPkr] = "SWPkr",
  [anon_sym_ROTkr] = "ROTkr",
  [anon_sym_DUPkr] = "DUPkr",
  [anon_sym_OVRkr] = "OVRkr",
  [anon_sym_EQUkr] = "EQUkr",
  [anon_sym_NEQkr] = "NEQkr",
  [anon_sym_GTHkr] = "GTHkr",
  [anon_sym_LTHkr] = "LTHkr",
  [anon_sym_JMPkr] = "JMPkr",
  [anon_sym_JCNkr] = "JCNkr",
  [anon_sym_JSRkr] = "JSRkr",
  [anon_sym_STHkr] = "STHkr",
  [anon_sym_LDZkr] = "LDZkr",
  [anon_sym_STZkr] = "STZkr",
  [anon_sym_LDRkr] = "LDRkr",
  [anon_sym_STRkr] = "STRkr",
  [anon_sym_LDAkr] = "LDAkr",
  [anon_sym_STAkr] = "STAkr",
  [anon_sym_DEIkr] = "DEIkr",
  [anon_sym_DEOkr] = "DEOkr",
  [anon_sym_ADDkr] = "ADDkr",
  [anon_sym_SUBkr] = "SUBkr",
  [anon_sym_MULkr] = "MULkr",
  [anon_sym_DIVkr] = "DIVkr",
  [anon_sym_ANDkr] = "ANDkr",
  [anon_sym_ORAkr] = "ORAkr",
  [anon_sym_EORkr] = "EORkr",
  [anon_sym_SFTkr] = "SFTkr",
  [anon_sym_LIT2r] = "LIT2r",
  [anon_sym_INC2kr] = "INC2kr",
  [anon_sym_POP2kr] = "POP2kr",
  [anon_sym_NIP2kr] = "NIP2kr",
  [anon_sym_SWP2kr] = "SWP2kr",
  [anon_sym_ROT2kr] = "ROT2kr",
  [anon_sym_DUP2kr] = "DUP2kr",
  [anon_sym_OVR2kr] = "OVR2kr",
  [anon_sym_EQU2kr] = "EQU2kr",
  [anon_sym_NEQ2kr] = "NEQ2kr",
  [anon_sym_GTH2kr] = "GTH2kr",
  [anon_sym_LTH2kr] = "LTH2kr",
  [anon_sym_JMP2kr] = "JMP2kr",
  [anon_sym_JCN2kr] = "JCN2kr",
  [anon_sym_JSR2kr] = "JSR2kr",
  [anon_sym_STH2kr] = "STH2kr",
  [anon_sym_LDZ2kr] = "LDZ2kr",
  [anon_sym_STZ2kr] = "STZ2kr",
  [anon_sym_LDR2kr] = "LDR2kr",
  [anon_sym_STR2kr] = "STR2kr",
  [anon_sym_LDA2kr] = "LDA2kr",
  [anon_sym_STA2kr] = "STA2kr",
  [anon_sym_DEI2kr] = "DEI2kr",
  [anon_sym_DEO2kr] = "DEO2kr",
  [anon_sym_ADD2kr] = "ADD2kr",
  [anon_sym_SUB2kr] = "SUB2kr",
  [anon_sym_MUL2kr] = "MUL2kr",
  [anon_sym_DIV2kr] = "DIV2kr",
  [anon_sym_AND2kr] = "AND2kr",
  [anon_sym_ORA2kr] = "ORA2kr",
  [anon_sym_EOR2kr] = "EOR2kr",
  [anon_sym_SFT2kr] = "SFT2kr",
  [anon_sym_PIPE] = "|",
  [anon_sym_DOLLAR] = "$",
  [anon_sym_POUND] = "#",
  [aux_sym_hex_literal_token1] = "hex_lit_value",
  [anon_sym_AT] = "@",
  [anon_sym_SLASH] = "/",
  [aux_sym_rune_token1] = "identifier",
  [anon_sym_COMMA] = ",",
  [anon_sym__] = "_",
  [anon_sym_DOT] = ".",
  [anon_sym_DASH] = "-",
  [anon_sym_SEMI] = ";",
  [anon_sym_EQ] = "=",
  [anon_sym_BANG] = "!",
  [anon_sym_QMARK] = "\?",
  [anon_sym_AMP] = "&",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_DQUOTE] = "\"",
  [aux_sym_raw_ascii_token1] = "raw_ascii_token1",
  [sym_number] = "number",
  [sym_comment] = "comment",
  [sym_program] = "program",
  [sym_memory_execution] = "memory_execution",
  [sym_subroutine] = "subroutine",
  [sym__non_toplevel_statement] = "_non_toplevel_statement",
  [sym_macro] = "macro",
  [sym_include] = "include",
  [sym_opcode] = "opcode",
  [sym_absolute_pad_operation] = "absolute_pad_operation",
  [sym_relative_pad_operation] = "relative_pad_operation",
  [sym_hex_literal] = "hex_literal",
  [sym_label] = "label",
  [sym_sublabel_reference] = "sublabel_reference",
  [sym_rune] = "rune",
  [sym_rune_char] = "rune_char",
  [sym_brackets] = "brackets",
  [sym_raw_ascii] = "raw_ascii",
  [aux_sym_program_repeat1] = "program_repeat1",
  [aux_sym_memory_execution_repeat1] = "memory_execution_repeat1",
  [aux_sym_rune_repeat1] = "rune_repeat1",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [aux_sym_macro_token1] = sym_identifier,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [aux_sym_include_token1] = aux_sym_include_token1,
  [anon_sym_BRK] = anon_sym_BRK,
  [anon_sym_INC] = anon_sym_INC,
  [anon_sym_POP] = anon_sym_POP,
  [anon_sym_NIP] = anon_sym_NIP,
  [anon_sym_SWP] = anon_sym_SWP,
  [anon_sym_ROT] = anon_sym_ROT,
  [anon_sym_DUP] = anon_sym_DUP,
  [anon_sym_OVR] = anon_sym_OVR,
  [anon_sym_EQU] = anon_sym_EQU,
  [anon_sym_NEQ] = anon_sym_NEQ,
  [anon_sym_GTH] = anon_sym_GTH,
  [anon_sym_LTH] = anon_sym_LTH,
  [anon_sym_JMP] = anon_sym_JMP,
  [anon_sym_JCN] = anon_sym_JCN,
  [anon_sym_JSR] = anon_sym_JSR,
  [anon_sym_STH] = anon_sym_STH,
  [anon_sym_LDZ] = anon_sym_LDZ,
  [anon_sym_STZ] = anon_sym_STZ,
  [anon_sym_LDR] = anon_sym_LDR,
  [anon_sym_STR] = anon_sym_STR,
  [anon_sym_LDA] = anon_sym_LDA,
  [anon_sym_STA] = anon_sym_STA,
  [anon_sym_DEI] = anon_sym_DEI,
  [anon_sym_DEO] = anon_sym_DEO,
  [anon_sym_ADD] = anon_sym_ADD,
  [anon_sym_SUB] = anon_sym_SUB,
  [anon_sym_MUL] = anon_sym_MUL,
  [anon_sym_DIV] = anon_sym_DIV,
  [anon_sym_AND] = anon_sym_AND,
  [anon_sym_ORA] = anon_sym_ORA,
  [anon_sym_EOR] = anon_sym_EOR,
  [anon_sym_SFT] = anon_sym_SFT,
  [anon_sym_JCI] = anon_sym_JCI,
  [anon_sym_INC2] = anon_sym_INC2,
  [anon_sym_POP2] = anon_sym_POP2,
  [anon_sym_NIP2] = anon_sym_NIP2,
  [anon_sym_SWP2] = anon_sym_SWP2,
  [anon_sym_ROT2] = anon_sym_ROT2,
  [anon_sym_DUP2] = anon_sym_DUP2,
  [anon_sym_OVR2] = anon_sym_OVR2,
  [anon_sym_EQU2] = anon_sym_EQU2,
  [anon_sym_NEQ2] = anon_sym_NEQ2,
  [anon_sym_GTH2] = anon_sym_GTH2,
  [anon_sym_LTH2] = anon_sym_LTH2,
  [anon_sym_JMP2] = anon_sym_JMP2,
  [anon_sym_JCN2] = anon_sym_JCN2,
  [anon_sym_JSR2] = anon_sym_JSR2,
  [anon_sym_STH2] = anon_sym_STH2,
  [anon_sym_LDZ2] = anon_sym_LDZ2,
  [anon_sym_STZ2] = anon_sym_STZ2,
  [anon_sym_LDR2] = anon_sym_LDR2,
  [anon_sym_STR2] = anon_sym_STR2,
  [anon_sym_LDA2] = anon_sym_LDA2,
  [anon_sym_STA2] = anon_sym_STA2,
  [anon_sym_DEI2] = anon_sym_DEI2,
  [anon_sym_DEO2] = anon_sym_DEO2,
  [anon_sym_ADD2] = anon_sym_ADD2,
  [anon_sym_SUB2] = anon_sym_SUB2,
  [anon_sym_MUL2] = anon_sym_MUL2,
  [anon_sym_DIV2] = anon_sym_DIV2,
  [anon_sym_AND2] = anon_sym_AND2,
  [anon_sym_ORA2] = anon_sym_ORA2,
  [anon_sym_EOR2] = anon_sym_EOR2,
  [anon_sym_SFT2] = anon_sym_SFT2,
  [anon_sym_JMI] = anon_sym_JMI,
  [anon_sym_INCr] = anon_sym_INCr,
  [anon_sym_POPr] = anon_sym_POPr,
  [anon_sym_NIPr] = anon_sym_NIPr,
  [anon_sym_SWPr] = anon_sym_SWPr,
  [anon_sym_ROTr] = anon_sym_ROTr,
  [anon_sym_DUPr] = anon_sym_DUPr,
  [anon_sym_OVRr] = anon_sym_OVRr,
  [anon_sym_EQUr] = anon_sym_EQUr,
  [anon_sym_NEQr] = anon_sym_NEQr,
  [anon_sym_GTHr] = anon_sym_GTHr,
  [anon_sym_LTHr] = anon_sym_LTHr,
  [anon_sym_JMPr] = anon_sym_JMPr,
  [anon_sym_JCNr] = anon_sym_JCNr,
  [anon_sym_JSRr] = anon_sym_JSRr,
  [anon_sym_STHr] = anon_sym_STHr,
  [anon_sym_LDZr] = anon_sym_LDZr,
  [anon_sym_STZr] = anon_sym_STZr,
  [anon_sym_LDRr] = anon_sym_LDRr,
  [anon_sym_STRr] = anon_sym_STRr,
  [anon_sym_LDAr] = anon_sym_LDAr,
  [anon_sym_STAr] = anon_sym_STAr,
  [anon_sym_DEIr] = anon_sym_DEIr,
  [anon_sym_DEOr] = anon_sym_DEOr,
  [anon_sym_ADDr] = anon_sym_ADDr,
  [anon_sym_SUBr] = anon_sym_SUBr,
  [anon_sym_MULr] = anon_sym_MULr,
  [anon_sym_DIVr] = anon_sym_DIVr,
  [anon_sym_ANDr] = anon_sym_ANDr,
  [anon_sym_ORAr] = anon_sym_ORAr,
  [anon_sym_EORr] = anon_sym_EORr,
  [anon_sym_SFTr] = anon_sym_SFTr,
  [anon_sym_JSI] = anon_sym_JSI,
  [anon_sym_INC2r] = anon_sym_INC2r,
  [anon_sym_POP2r] = anon_sym_POP2r,
  [anon_sym_NIP2r] = anon_sym_NIP2r,
  [anon_sym_SWP2r] = anon_sym_SWP2r,
  [anon_sym_ROT2r] = anon_sym_ROT2r,
  [anon_sym_DUP2r] = anon_sym_DUP2r,
  [anon_sym_OVR2r] = anon_sym_OVR2r,
  [anon_sym_EQU2r] = anon_sym_EQU2r,
  [anon_sym_NEQ2r] = anon_sym_NEQ2r,
  [anon_sym_GTH2r] = anon_sym_GTH2r,
  [anon_sym_LTH2r] = anon_sym_LTH2r,
  [anon_sym_JMP2r] = anon_sym_JMP2r,
  [anon_sym_JCN2r] = anon_sym_JCN2r,
  [anon_sym_JSR2r] = anon_sym_JSR2r,
  [anon_sym_STH2r] = anon_sym_STH2r,
  [anon_sym_LDZ2r] = anon_sym_LDZ2r,
  [anon_sym_STZ2r] = anon_sym_STZ2r,
  [anon_sym_LDR2r] = anon_sym_LDR2r,
  [anon_sym_STR2r] = anon_sym_STR2r,
  [anon_sym_LDA2r] = anon_sym_LDA2r,
  [anon_sym_STA2r] = anon_sym_STA2r,
  [anon_sym_DEI2r] = anon_sym_DEI2r,
  [anon_sym_DEO2r] = anon_sym_DEO2r,
  [anon_sym_ADD2r] = anon_sym_ADD2r,
  [anon_sym_SUB2r] = anon_sym_SUB2r,
  [anon_sym_MUL2r] = anon_sym_MUL2r,
  [anon_sym_DIV2r] = anon_sym_DIV2r,
  [anon_sym_AND2r] = anon_sym_AND2r,
  [anon_sym_ORA2r] = anon_sym_ORA2r,
  [anon_sym_EOR2r] = anon_sym_EOR2r,
  [anon_sym_SFT2r] = anon_sym_SFT2r,
  [anon_sym_LIT] = anon_sym_LIT,
  [anon_sym_INCk] = anon_sym_INCk,
  [anon_sym_POPk] = anon_sym_POPk,
  [anon_sym_NIPk] = anon_sym_NIPk,
  [anon_sym_SWPk] = anon_sym_SWPk,
  [anon_sym_ROTk] = anon_sym_ROTk,
  [anon_sym_DUPk] = anon_sym_DUPk,
  [anon_sym_OVRk] = anon_sym_OVRk,
  [anon_sym_EQUk] = anon_sym_EQUk,
  [anon_sym_NEQk] = anon_sym_NEQk,
  [anon_sym_GTHk] = anon_sym_GTHk,
  [anon_sym_LTHk] = anon_sym_LTHk,
  [anon_sym_JMPk] = anon_sym_JMPk,
  [anon_sym_JCNk] = anon_sym_JCNk,
  [anon_sym_JSRk] = anon_sym_JSRk,
  [anon_sym_STHk] = anon_sym_STHk,
  [anon_sym_LDZk] = anon_sym_LDZk,
  [anon_sym_STZk] = anon_sym_STZk,
  [anon_sym_LDRk] = anon_sym_LDRk,
  [anon_sym_STRk] = anon_sym_STRk,
  [anon_sym_LDAk] = anon_sym_LDAk,
  [anon_sym_STAk] = anon_sym_STAk,
  [anon_sym_DEIk] = anon_sym_DEIk,
  [anon_sym_DEOk] = anon_sym_DEOk,
  [anon_sym_ADDk] = anon_sym_ADDk,
  [anon_sym_SUBk] = anon_sym_SUBk,
  [anon_sym_MULk] = anon_sym_MULk,
  [anon_sym_DIVk] = anon_sym_DIVk,
  [anon_sym_ANDk] = anon_sym_ANDk,
  [anon_sym_ORAk] = anon_sym_ORAk,
  [anon_sym_EORk] = anon_sym_EORk,
  [anon_sym_SFTk] = anon_sym_SFTk,
  [anon_sym_LIT2] = anon_sym_LIT2,
  [anon_sym_INC2k] = anon_sym_INC2k,
  [anon_sym_POP2k] = anon_sym_POP2k,
  [anon_sym_NIP2k] = anon_sym_NIP2k,
  [anon_sym_SWP2k] = anon_sym_SWP2k,
  [anon_sym_ROT2k] = anon_sym_ROT2k,
  [anon_sym_DUP2k] = anon_sym_DUP2k,
  [anon_sym_OVR2k] = anon_sym_OVR2k,
  [anon_sym_EQU2k] = anon_sym_EQU2k,
  [anon_sym_NEQ2k] = anon_sym_NEQ2k,
  [anon_sym_GTH2k] = anon_sym_GTH2k,
  [anon_sym_LTH2k] = anon_sym_LTH2k,
  [anon_sym_JMP2k] = anon_sym_JMP2k,
  [anon_sym_JCN2k] = anon_sym_JCN2k,
  [anon_sym_JSR2k] = anon_sym_JSR2k,
  [anon_sym_STH2k] = anon_sym_STH2k,
  [anon_sym_LDZ2k] = anon_sym_LDZ2k,
  [anon_sym_STZ2k] = anon_sym_STZ2k,
  [anon_sym_LDR2k] = anon_sym_LDR2k,
  [anon_sym_STR2k] = anon_sym_STR2k,
  [anon_sym_LDA2k] = anon_sym_LDA2k,
  [anon_sym_STA2k] = anon_sym_STA2k,
  [anon_sym_DEI2k] = anon_sym_DEI2k,
  [anon_sym_DEO2k] = anon_sym_DEO2k,
  [anon_sym_ADD2k] = anon_sym_ADD2k,
  [anon_sym_SUB2k] = anon_sym_SUB2k,
  [anon_sym_MUL2k] = anon_sym_MUL2k,
  [anon_sym_DIV2k] = anon_sym_DIV2k,
  [anon_sym_AND2k] = anon_sym_AND2k,
  [anon_sym_ORA2k] = anon_sym_ORA2k,
  [anon_sym_EOR2k] = anon_sym_EOR2k,
  [anon_sym_SFT2k] = anon_sym_SFT2k,
  [anon_sym_LITr] = anon_sym_LITr,
  [anon_sym_INCkr] = anon_sym_INCkr,
  [anon_sym_POPkr] = anon_sym_POPkr,
  [anon_sym_NIPkr] = anon_sym_NIPkr,
  [anon_sym_SWPkr] = anon_sym_SWPkr,
  [anon_sym_ROTkr] = anon_sym_ROTkr,
  [anon_sym_DUPkr] = anon_sym_DUPkr,
  [anon_sym_OVRkr] = anon_sym_OVRkr,
  [anon_sym_EQUkr] = anon_sym_EQUkr,
  [anon_sym_NEQkr] = anon_sym_NEQkr,
  [anon_sym_GTHkr] = anon_sym_GTHkr,
  [anon_sym_LTHkr] = anon_sym_LTHkr,
  [anon_sym_JMPkr] = anon_sym_JMPkr,
  [anon_sym_JCNkr] = anon_sym_JCNkr,
  [anon_sym_JSRkr] = anon_sym_JSRkr,
  [anon_sym_STHkr] = anon_sym_STHkr,
  [anon_sym_LDZkr] = anon_sym_LDZkr,
  [anon_sym_STZkr] = anon_sym_STZkr,
  [anon_sym_LDRkr] = anon_sym_LDRkr,
  [anon_sym_STRkr] = anon_sym_STRkr,
  [anon_sym_LDAkr] = anon_sym_LDAkr,
  [anon_sym_STAkr] = anon_sym_STAkr,
  [anon_sym_DEIkr] = anon_sym_DEIkr,
  [anon_sym_DEOkr] = anon_sym_DEOkr,
  [anon_sym_ADDkr] = anon_sym_ADDkr,
  [anon_sym_SUBkr] = anon_sym_SUBkr,
  [anon_sym_MULkr] = anon_sym_MULkr,
  [anon_sym_DIVkr] = anon_sym_DIVkr,
  [anon_sym_ANDkr] = anon_sym_ANDkr,
  [anon_sym_ORAkr] = anon_sym_ORAkr,
  [anon_sym_EORkr] = anon_sym_EORkr,
  [anon_sym_SFTkr] = anon_sym_SFTkr,
  [anon_sym_LIT2r] = anon_sym_LIT2r,
  [anon_sym_INC2kr] = anon_sym_INC2kr,
  [anon_sym_POP2kr] = anon_sym_POP2kr,
  [anon_sym_NIP2kr] = anon_sym_NIP2kr,
  [anon_sym_SWP2kr] = anon_sym_SWP2kr,
  [anon_sym_ROT2kr] = anon_sym_ROT2kr,
  [anon_sym_DUP2kr] = anon_sym_DUP2kr,
  [anon_sym_OVR2kr] = anon_sym_OVR2kr,
  [anon_sym_EQU2kr] = anon_sym_EQU2kr,
  [anon_sym_NEQ2kr] = anon_sym_NEQ2kr,
  [anon_sym_GTH2kr] = anon_sym_GTH2kr,
  [anon_sym_LTH2kr] = anon_sym_LTH2kr,
  [anon_sym_JMP2kr] = anon_sym_JMP2kr,
  [anon_sym_JCN2kr] = anon_sym_JCN2kr,
  [anon_sym_JSR2kr] = anon_sym_JSR2kr,
  [anon_sym_STH2kr] = anon_sym_STH2kr,
  [anon_sym_LDZ2kr] = anon_sym_LDZ2kr,
  [anon_sym_STZ2kr] = anon_sym_STZ2kr,
  [anon_sym_LDR2kr] = anon_sym_LDR2kr,
  [anon_sym_STR2kr] = anon_sym_STR2kr,
  [anon_sym_LDA2kr] = anon_sym_LDA2kr,
  [anon_sym_STA2kr] = anon_sym_STA2kr,
  [anon_sym_DEI2kr] = anon_sym_DEI2kr,
  [anon_sym_DEO2kr] = anon_sym_DEO2kr,
  [anon_sym_ADD2kr] = anon_sym_ADD2kr,
  [anon_sym_SUB2kr] = anon_sym_SUB2kr,
  [anon_sym_MUL2kr] = anon_sym_MUL2kr,
  [anon_sym_DIV2kr] = anon_sym_DIV2kr,
  [anon_sym_AND2kr] = anon_sym_AND2kr,
  [anon_sym_ORA2kr] = anon_sym_ORA2kr,
  [anon_sym_EOR2kr] = anon_sym_EOR2kr,
  [anon_sym_SFT2kr] = anon_sym_SFT2kr,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_DOLLAR] = anon_sym_DOLLAR,
  [anon_sym_POUND] = anon_sym_POUND,
  [aux_sym_hex_literal_token1] = aux_sym_hex_literal_token1,
  [anon_sym_AT] = anon_sym_AT,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [aux_sym_rune_token1] = sym_identifier,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym__] = anon_sym__,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_QMARK] = anon_sym_QMARK,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_DQUOTE] = anon_sym_DQUOTE,
  [aux_sym_raw_ascii_token1] = aux_sym_raw_ascii_token1,
  [sym_number] = sym_number,
  [sym_comment] = sym_comment,
  [sym_program] = sym_program,
  [sym_memory_execution] = sym_memory_execution,
  [sym_subroutine] = sym_subroutine,
  [sym__non_toplevel_statement] = sym__non_toplevel_statement,
  [sym_macro] = sym_macro,
  [sym_include] = sym_include,
  [sym_opcode] = sym_opcode,
  [sym_absolute_pad_operation] = sym_absolute_pad_operation,
  [sym_relative_pad_operation] = sym_relative_pad_operation,
  [sym_hex_literal] = sym_hex_literal,
  [sym_label] = sym_label,
  [sym_sublabel_reference] = sym_sublabel_reference,
  [sym_rune] = sym_rune,
  [sym_rune_char] = sym_rune_char,
  [sym_brackets] = sym_brackets,
  [sym_raw_ascii] = sym_raw_ascii,
  [aux_sym_program_repeat1] = aux_sym_program_repeat1,
  [aux_sym_memory_execution_repeat1] = aux_sym_memory_execution_repeat1,
  [aux_sym_rune_repeat1] = aux_sym_rune_repeat1,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_macro_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_include_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_BRK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INC] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQU] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEO] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADD] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUB] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MUL] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIV] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AND] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EOR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INC2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POP2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIP2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWP2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROT2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUP2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVR2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQU2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQ2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTH2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTH2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMP2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCN2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSR2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STH2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZ2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZ2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDR2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STR2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDA2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STA2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEI2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEO2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADD2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUB2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MUL2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIV2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AND2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORA2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EOR2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFT2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INCr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POPr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIPr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWPr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROTr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUPr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVRr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQUr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTHr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTHr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMPr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCNr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSRr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STHr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDRr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STRr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDAr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEIr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEOr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADDr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUBr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MULr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIVr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ANDr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORAr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EORr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFTr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INC2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POP2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIP2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWP2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROT2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUP2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVR2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQU2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQ2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTH2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTH2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMP2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCN2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSR2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STH2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZ2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZ2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDR2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STR2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDA2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STA2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEI2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEO2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADD2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUB2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MUL2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIV2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AND2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORA2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EOR2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFT2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LIT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INCk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POPk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIPk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWPk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROTk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUPk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVRk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQUk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTHk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTHk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMPk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCNk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSRk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STHk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDRk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STRk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDAk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEIk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEOk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADDk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUBk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MULk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIVk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ANDk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORAk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EORk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFTk] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LIT2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INC2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POP2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIP2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWP2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROT2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUP2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVR2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQU2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQ2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTH2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTH2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMP2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCN2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSR2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STH2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZ2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZ2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDR2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STR2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDA2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STA2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEI2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEO2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADD2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUB2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MUL2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIV2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AND2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORA2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EOR2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFT2k] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LITr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INCkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POPkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIPkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWPkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROTkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUPkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVRkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQUkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTHkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTHkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMPkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCNkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSRkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STHkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDRkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STRkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDAkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEIkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEOkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADDkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUBkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MULkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIVkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ANDkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORAkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EORkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFTkr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LIT2r] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_INC2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POP2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NIP2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SWP2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ROT2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DUP2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_OVR2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQU2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NEQ2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GTH2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LTH2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JMP2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JCN2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_JSR2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STH2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDZ2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STZ2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDR2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STR2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LDA2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STA2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEI2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DEO2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ADD2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SUB2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_MUL2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DIV2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AND2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ORA2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EOR2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SFT2kr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOLLAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUND] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_hex_literal_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_rune_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_raw_ascii_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_number] = {
    .visible = true,
    .named = true,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_program] = {
    .visible = true,
    .named = true,
  },
  [sym_memory_execution] = {
    .visible = true,
    .named = true,
  },
  [sym_subroutine] = {
    .visible = true,
    .named = true,
  },
  [sym__non_toplevel_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_macro] = {
    .visible = true,
    .named = true,
  },
  [sym_include] = {
    .visible = true,
    .named = true,
  },
  [sym_opcode] = {
    .visible = true,
    .named = true,
  },
  [sym_absolute_pad_operation] = {
    .visible = true,
    .named = true,
  },
  [sym_relative_pad_operation] = {
    .visible = true,
    .named = true,
  },
  [sym_hex_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_label] = {
    .visible = true,
    .named = true,
  },
  [sym_sublabel_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_rune] = {
    .visible = true,
    .named = true,
  },
  [sym_rune_char] = {
    .visible = true,
    .named = true,
  },
  [sym_brackets] = {
    .visible = true,
    .named = true,
  },
  [sym_raw_ascii] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_program_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_memory_execution_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_rune_repeat1] = {
    .visible = false,
    .named = false,
  },
};

enum ts_field_identifiers {
  field_absolute = 1,
  field_immediate = 2,
  field_relative = 3,
  field_rune_start = 4,
  field_sublabel = 5,
  field_zero_page = 6,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_absolute] = "absolute",
  [field_immediate] = "immediate",
  [field_relative] = "relative",
  [field_rune_start] = "rune_start",
  [field_sublabel] = "sublabel",
  [field_zero_page] = "zero_page",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [1] = {.index = 0, .length = 1},
  [2] = {.index = 1, .length = 1},
  [3] = {.index = 2, .length = 1},
  [4] = {.index = 3, .length = 1},
  [5] = {.index = 4, .length = 1},
  [6] = {.index = 5, .length = 1},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_relative, 0},
  [1] =
    {field_zero_page, 0},
  [2] =
    {field_absolute, 0},
  [3] =
    {field_immediate, 0},
  [4] =
    {field_sublabel, 0},
  [5] =
    {field_rune_start, 0},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
};

static const uint16_t ts_non_terminal_alias_map[] = {
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 6,
  [7] = 7,
  [8] = 8,
  [9] = 9,
  [10] = 10,
  [11] = 11,
  [12] = 12,
  [13] = 13,
  [14] = 14,
  [15] = 15,
  [16] = 16,
  [17] = 17,
  [18] = 18,
  [19] = 19,
  [20] = 20,
  [21] = 21,
  [22] = 22,
  [23] = 23,
  [24] = 24,
  [25] = 25,
  [26] = 26,
  [27] = 27,
  [28] = 28,
  [29] = 29,
  [30] = 30,
  [31] = 31,
  [32] = 32,
  [33] = 33,
  [34] = 34,
  [35] = 35,
  [36] = 36,
  [37] = 37,
  [38] = 38,
  [39] = 39,
  [40] = 40,
  [41] = 41,
  [42] = 42,
  [43] = 43,
  [44] = 44,
  [45] = 45,
  [46] = 46,
  [47] = 47,
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(15);
      ADVANCE_MAP(
        '!', 45,
        '"', 50,
        '#', 27,
        '$', 26,
        '%', 16,
        '&', 47,
        ',', 40,
        '-', 42,
        '.', 41,
        '/', 31,
        ';', 43,
        '=', 44,
        '?', 46,
        '@', 30,
        '[', 48,
        ']', 49,
        '{', 21,
        '|', 25,
        '}', 22,
        '~', 23,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      if (('a' <= lookahead && lookahead <= 'f')) ADVANCE(53);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(3);
      if (lookahead == '*' ||
          lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(55);
      END_STATE();
    case 1:
      ADVANCE_MAP(
        '!', 45,
        '&', 47,
        ',', 40,
        '-', 42,
        '.', 41,
        ';', 43,
        '=', 44,
        '?', 46,
        '*', 36,
        '/', 36,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(1);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(37);
      if (lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(55);
      END_STATE();
    case 2:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      END_STATE();
    case 3:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (('a' <= lookahead && lookahead <= 'f')) ADVANCE(54);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(5);
      if (lookahead == '*' ||
          ('/' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(55);
      END_STATE();
    case 4:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(2);
      END_STATE();
    case 5:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(4);
      END_STATE();
    case 6:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(6);
      if (lookahead == '*' ||
          lookahead == '/' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(17);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(19);
      if (lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(55);
      END_STATE();
    case 7:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(7);
      if (lookahead != 0) ADVANCE(51);
      END_STATE();
    case 8:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(8);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(13);
      if (lookahead == '*' ||
          ('/' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(55);
      END_STATE();
    case 9:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(9);
      if (lookahead == '.' ||
          lookahead == '/' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(24);
      END_STATE();
    case 10:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(10);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(11);
      END_STATE();
    case 11:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(29);
      END_STATE();
    case 12:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(28);
      END_STATE();
    case 13:
      if (lookahead == '*' ||
          lookahead == '/' ||
          lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(55);
      END_STATE();
    case 14:
      if (eof) ADVANCE(15);
      ADVANCE_MAP(
        '!', 45,
        '"', 50,
        '#', 27,
        '$', 26,
        '%', 16,
        '&', 47,
        ',', 40,
        '-', 42,
        '.', 41,
        ';', 43,
        '=', 44,
        '?', 46,
        '@', 30,
        '[', 48,
        ']', 49,
        '|', 25,
        '}', 22,
        '~', 23,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(14);
      if (('a' <= lookahead && lookahead <= 'f')) ADVANCE(53);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(3);
      if (lookahead == '*' ||
          ('/' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(55);
      END_STATE();
    case 15:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 16:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 17:
      ACCEPT_TOKEN(aux_sym_macro_token1);
      if (lookahead == '/') ADVANCE(17);
      if (lookahead == '*' ||
          lookahead == '-' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(18);
      END_STATE();
    case 18:
      ACCEPT_TOKEN(aux_sym_macro_token1);
      if (lookahead == '/') ADVANCE(20);
      if (lookahead == '*' ||
          lookahead == '-' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(18);
      END_STATE();
    case 19:
      ACCEPT_TOKEN(aux_sym_macro_token1);
      if (lookahead == '-' ||
          ('0' <= lookahead && lookahead <= '9')) ADVANCE(20);
      if (lookahead == '*' ||
          lookahead == '/' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(17);
      END_STATE();
    case 20:
      ACCEPT_TOKEN(aux_sym_macro_token1);
      if (lookahead == '*' ||
          lookahead == '-' ||
          ('/' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(20);
      END_STATE();
    case 21:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 22:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 23:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 24:
      ACCEPT_TOKEN(aux_sym_include_token1);
      if (('-' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(24);
      END_STATE();
    case 25:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 26:
      ACCEPT_TOKEN(anon_sym_DOLLAR);
      END_STATE();
    case 27:
      ACCEPT_TOKEN(anon_sym_POUND);
      END_STATE();
    case 28:
      ACCEPT_TOKEN(aux_sym_hex_literal_token1);
      END_STATE();
    case 29:
      ACCEPT_TOKEN(aux_sym_hex_literal_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(12);
      END_STATE();
    case 30:
      ACCEPT_TOKEN(anon_sym_AT);
      END_STATE();
    case 31:
      ACCEPT_TOKEN(anon_sym_SLASH);
      END_STATE();
    case 32:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      END_STATE();
    case 33:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      if (lookahead == '/') ADVANCE(32);
      if (lookahead == '*' ||
          ('0' <= lookahead && lookahead <= '9')) ADVANCE(32);
      END_STATE();
    case 34:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      if (lookahead == '/') ADVANCE(38);
      if (lookahead == '*' ||
          ('0' <= lookahead && lookahead <= '9')) ADVANCE(33);
      END_STATE();
    case 35:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      if (lookahead == '/') ADVANCE(33);
      if (lookahead == '*' ||
          ('0' <= lookahead && lookahead <= '9')) ADVANCE(33);
      END_STATE();
    case 36:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      if (lookahead == '/') ADVANCE(35);
      if (lookahead == '*' ||
          ('0' <= lookahead && lookahead <= '9')) ADVANCE(34);
      END_STATE();
    case 37:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      if (lookahead == '*' ||
          lookahead == '/') ADVANCE(35);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(39);
      END_STATE();
    case 38:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      if (lookahead == '*' ||
          ('/' <= lookahead && lookahead <= '9')) ADVANCE(32);
      END_STATE();
    case 39:
      ACCEPT_TOKEN(aux_sym_rune_token1);
      if (lookahead == '*' ||
          ('/' <= lookahead && lookahead <= '9')) ADVANCE(38);
      END_STATE();
    case 40:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 41:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 45:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_QMARK);
      END_STATE();
    case 47:
      ACCEPT_TOKEN(anon_sym_AMP);
      END_STATE();
    case 48:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 49:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 50:
      ACCEPT_TOKEN(anon_sym_DQUOTE);
      END_STATE();
    case 51:
      ACCEPT_TOKEN(aux_sym_raw_ascii_token1);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != ' ') ADVANCE(51);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(sym_number);
      END_STATE();
    case 53:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '/') ADVANCE(55);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(57);
      if (lookahead == '#' ||
          lookahead == '*' ||
          lookahead == '-' ||
          lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(59);
      END_STATE();
    case 54:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '/') ADVANCE(55);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(56);
      if (lookahead == '#' ||
          lookahead == '*' ||
          lookahead == '-' ||
          lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(59);
      END_STATE();
    case 55:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '/') ADVANCE(55);
      if (lookahead == '#' ||
          lookahead == '*' ||
          lookahead == '-' ||
          ('0' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(59);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(sym_identifier);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(58);
      if (lookahead == '#' ||
          lookahead == '*' ||
          lookahead == '-' ||
          lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(59);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(sym_identifier);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(56);
      if (lookahead == '#' ||
          lookahead == '*' ||
          lookahead == '-' ||
          lookahead == ':' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(59);
      END_STATE();
    case 58:
      ACCEPT_TOKEN(sym_identifier);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(52);
      if (lookahead == '#' ||
          lookahead == '*' ||
          lookahead == '-' ||
          ('0' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(59);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '#' ||
          lookahead == '*' ||
          lookahead == '-' ||
          ('0' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(59);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      ADVANCE_MAP(
        'A', 1,
        'B', 2,
        'D', 3,
        'E', 4,
        'G', 5,
        'I', 6,
        'J', 7,
        'L', 8,
        'M', 9,
        'N', 10,
        'O', 11,
        'P', 12,
        'R', 13,
        'S', 14,
        '_', 15,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      END_STATE();
    case 1:
      if (lookahead == 'D') ADVANCE(16);
      if (lookahead == 'N') ADVANCE(17);
      END_STATE();
    case 2:
      if (lookahead == 'R') ADVANCE(18);
      END_STATE();
    case 3:
      if (lookahead == 'E') ADVANCE(19);
      if (lookahead == 'I') ADVANCE(20);
      if (lookahead == 'U') ADVANCE(21);
      END_STATE();
    case 4:
      if (lookahead == 'O') ADVANCE(22);
      if (lookahead == 'Q') ADVANCE(23);
      END_STATE();
    case 5:
      if (lookahead == 'T') ADVANCE(24);
      END_STATE();
    case 6:
      if (lookahead == 'N') ADVANCE(25);
      END_STATE();
    case 7:
      if (lookahead == 'C') ADVANCE(26);
      if (lookahead == 'M') ADVANCE(27);
      if (lookahead == 'S') ADVANCE(28);
      END_STATE();
    case 8:
      if (lookahead == 'D') ADVANCE(29);
      if (lookahead == 'I') ADVANCE(30);
      if (lookahead == 'T') ADVANCE(31);
      END_STATE();
    case 9:
      if (lookahead == 'U') ADVANCE(32);
      END_STATE();
    case 10:
      if (lookahead == 'E') ADVANCE(33);
      if (lookahead == 'I') ADVANCE(34);
      END_STATE();
    case 11:
      if (lookahead == 'R') ADVANCE(35);
      if (lookahead == 'V') ADVANCE(36);
      END_STATE();
    case 12:
      if (lookahead == 'O') ADVANCE(37);
      END_STATE();
    case 13:
      if (lookahead == 'O') ADVANCE(38);
      END_STATE();
    case 14:
      if (lookahead == 'F') ADVANCE(39);
      if (lookahead == 'T') ADVANCE(40);
      if (lookahead == 'U') ADVANCE(41);
      if (lookahead == 'W') ADVANCE(42);
      END_STATE();
    case 15:
      ACCEPT_TOKEN(anon_sym__);
      END_STATE();
    case 16:
      if (lookahead == 'D') ADVANCE(43);
      END_STATE();
    case 17:
      if (lookahead == 'D') ADVANCE(44);
      END_STATE();
    case 18:
      if (lookahead == 'K') ADVANCE(45);
      END_STATE();
    case 19:
      if (lookahead == 'I') ADVANCE(46);
      if (lookahead == 'O') ADVANCE(47);
      END_STATE();
    case 20:
      if (lookahead == 'V') ADVANCE(48);
      END_STATE();
    case 21:
      if (lookahead == 'P') ADVANCE(49);
      END_STATE();
    case 22:
      if (lookahead == 'R') ADVANCE(50);
      END_STATE();
    case 23:
      if (lookahead == 'U') ADVANCE(51);
      END_STATE();
    case 24:
      if (lookahead == 'H') ADVANCE(52);
      END_STATE();
    case 25:
      if (lookahead == 'C') ADVANCE(53);
      END_STATE();
    case 26:
      if (lookahead == 'I') ADVANCE(54);
      if (lookahead == 'N') ADVANCE(55);
      END_STATE();
    case 27:
      if (lookahead == 'I') ADVANCE(56);
      if (lookahead == 'P') ADVANCE(57);
      END_STATE();
    case 28:
      if (lookahead == 'I') ADVANCE(58);
      if (lookahead == 'R') ADVANCE(59);
      END_STATE();
    case 29:
      if (lookahead == 'A') ADVANCE(60);
      if (lookahead == 'R') ADVANCE(61);
      if (lookahead == 'Z') ADVANCE(62);
      END_STATE();
    case 30:
      if (lookahead == 'T') ADVANCE(63);
      END_STATE();
    case 31:
      if (lookahead == 'H') ADVANCE(64);
      END_STATE();
    case 32:
      if (lookahead == 'L') ADVANCE(65);
      END_STATE();
    case 33:
      if (lookahead == 'Q') ADVANCE(66);
      END_STATE();
    case 34:
      if (lookahead == 'P') ADVANCE(67);
      END_STATE();
    case 35:
      if (lookahead == 'A') ADVANCE(68);
      END_STATE();
    case 36:
      if (lookahead == 'R') ADVANCE(69);
      END_STATE();
    case 37:
      if (lookahead == 'P') ADVANCE(70);
      END_STATE();
    case 38:
      if (lookahead == 'T') ADVANCE(71);
      END_STATE();
    case 39:
      if (lookahead == 'T') ADVANCE(72);
      END_STATE();
    case 40:
      if (lookahead == 'A') ADVANCE(73);
      if (lookahead == 'H') ADVANCE(74);
      if (lookahead == 'R') ADVANCE(75);
      if (lookahead == 'Z') ADVANCE(76);
      END_STATE();
    case 41:
      if (lookahead == 'B') ADVANCE(77);
      END_STATE();
    case 42:
      if (lookahead == 'P') ADVANCE(78);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_ADD);
      if (lookahead == '2') ADVANCE(79);
      if (lookahead == 'k') ADVANCE(80);
      if (lookahead == 'r') ADVANCE(81);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_AND);
      if (lookahead == '2') ADVANCE(82);
      if (lookahead == 'k') ADVANCE(83);
      if (lookahead == 'r') ADVANCE(84);
      END_STATE();
    case 45:
      ACCEPT_TOKEN(anon_sym_BRK);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_DEI);
      if (lookahead == '2') ADVANCE(85);
      if (lookahead == 'k') ADVANCE(86);
      if (lookahead == 'r') ADVANCE(87);
      END_STATE();
    case 47:
      ACCEPT_TOKEN(anon_sym_DEO);
      if (lookahead == '2') ADVANCE(88);
      if (lookahead == 'k') ADVANCE(89);
      if (lookahead == 'r') ADVANCE(90);
      END_STATE();
    case 48:
      ACCEPT_TOKEN(anon_sym_DIV);
      if (lookahead == '2') ADVANCE(91);
      if (lookahead == 'k') ADVANCE(92);
      if (lookahead == 'r') ADVANCE(93);
      END_STATE();
    case 49:
      ACCEPT_TOKEN(anon_sym_DUP);
      if (lookahead == '2') ADVANCE(94);
      if (lookahead == 'k') ADVANCE(95);
      if (lookahead == 'r') ADVANCE(96);
      END_STATE();
    case 50:
      ACCEPT_TOKEN(anon_sym_EOR);
      if (lookahead == '2') ADVANCE(97);
      if (lookahead == 'k') ADVANCE(98);
      if (lookahead == 'r') ADVANCE(99);
      END_STATE();
    case 51:
      ACCEPT_TOKEN(anon_sym_EQU);
      if (lookahead == '2') ADVANCE(100);
      if (lookahead == 'k') ADVANCE(101);
      if (lookahead == 'r') ADVANCE(102);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(anon_sym_GTH);
      if (lookahead == '2') ADVANCE(103);
      if (lookahead == 'k') ADVANCE(104);
      if (lookahead == 'r') ADVANCE(105);
      END_STATE();
    case 53:
      ACCEPT_TOKEN(anon_sym_INC);
      if (lookahead == '2') ADVANCE(106);
      if (lookahead == 'k') ADVANCE(107);
      if (lookahead == 'r') ADVANCE(108);
      END_STATE();
    case 54:
      ACCEPT_TOKEN(anon_sym_JCI);
      END_STATE();
    case 55:
      ACCEPT_TOKEN(anon_sym_JCN);
      if (lookahead == '2') ADVANCE(109);
      if (lookahead == 'k') ADVANCE(110);
      if (lookahead == 'r') ADVANCE(111);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_JMI);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(anon_sym_JMP);
      if (lookahead == '2') ADVANCE(112);
      if (lookahead == 'k') ADVANCE(113);
      if (lookahead == 'r') ADVANCE(114);
      END_STATE();
    case 58:
      ACCEPT_TOKEN(anon_sym_JSI);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(anon_sym_JSR);
      if (lookahead == '2') ADVANCE(115);
      if (lookahead == 'k') ADVANCE(116);
      if (lookahead == 'r') ADVANCE(117);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(anon_sym_LDA);
      if (lookahead == '2') ADVANCE(118);
      if (lookahead == 'k') ADVANCE(119);
      if (lookahead == 'r') ADVANCE(120);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_LDR);
      if (lookahead == '2') ADVANCE(121);
      if (lookahead == 'k') ADVANCE(122);
      if (lookahead == 'r') ADVANCE(123);
      END_STATE();
    case 62:
      ACCEPT_TOKEN(anon_sym_LDZ);
      if (lookahead == '2') ADVANCE(124);
      if (lookahead == 'k') ADVANCE(125);
      if (lookahead == 'r') ADVANCE(126);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_LIT);
      if (lookahead == '2') ADVANCE(127);
      if (lookahead == 'r') ADVANCE(128);
      END_STATE();
    case 64:
      ACCEPT_TOKEN(anon_sym_LTH);
      if (lookahead == '2') ADVANCE(129);
      if (lookahead == 'k') ADVANCE(130);
      if (lookahead == 'r') ADVANCE(131);
      END_STATE();
    case 65:
      ACCEPT_TOKEN(anon_sym_MUL);
      if (lookahead == '2') ADVANCE(132);
      if (lookahead == 'k') ADVANCE(133);
      if (lookahead == 'r') ADVANCE(134);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_NEQ);
      if (lookahead == '2') ADVANCE(135);
      if (lookahead == 'k') ADVANCE(136);
      if (lookahead == 'r') ADVANCE(137);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_NIP);
      if (lookahead == '2') ADVANCE(138);
      if (lookahead == 'k') ADVANCE(139);
      if (lookahead == 'r') ADVANCE(140);
      END_STATE();
    case 68:
      ACCEPT_TOKEN(anon_sym_ORA);
      if (lookahead == '2') ADVANCE(141);
      if (lookahead == 'k') ADVANCE(142);
      if (lookahead == 'r') ADVANCE(143);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_OVR);
      if (lookahead == '2') ADVANCE(144);
      if (lookahead == 'k') ADVANCE(145);
      if (lookahead == 'r') ADVANCE(146);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_POP);
      if (lookahead == '2') ADVANCE(147);
      if (lookahead == 'k') ADVANCE(148);
      if (lookahead == 'r') ADVANCE(149);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(anon_sym_ROT);
      if (lookahead == '2') ADVANCE(150);
      if (lookahead == 'k') ADVANCE(151);
      if (lookahead == 'r') ADVANCE(152);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(anon_sym_SFT);
      if (lookahead == '2') ADVANCE(153);
      if (lookahead == 'k') ADVANCE(154);
      if (lookahead == 'r') ADVANCE(155);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_STA);
      if (lookahead == '2') ADVANCE(156);
      if (lookahead == 'k') ADVANCE(157);
      if (lookahead == 'r') ADVANCE(158);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(anon_sym_STH);
      if (lookahead == '2') ADVANCE(159);
      if (lookahead == 'k') ADVANCE(160);
      if (lookahead == 'r') ADVANCE(161);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_STR);
      if (lookahead == '2') ADVANCE(162);
      if (lookahead == 'k') ADVANCE(163);
      if (lookahead == 'r') ADVANCE(164);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(anon_sym_STZ);
      if (lookahead == '2') ADVANCE(165);
      if (lookahead == 'k') ADVANCE(166);
      if (lookahead == 'r') ADVANCE(167);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_SUB);
      if (lookahead == '2') ADVANCE(168);
      if (lookahead == 'k') ADVANCE(169);
      if (lookahead == 'r') ADVANCE(170);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_SWP);
      if (lookahead == '2') ADVANCE(171);
      if (lookahead == 'k') ADVANCE(172);
      if (lookahead == 'r') ADVANCE(173);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_ADD2);
      if (lookahead == 'k') ADVANCE(174);
      if (lookahead == 'r') ADVANCE(175);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(anon_sym_ADDk);
      if (lookahead == 'r') ADVANCE(176);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(anon_sym_ADDr);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_AND2);
      if (lookahead == 'k') ADVANCE(177);
      if (lookahead == 'r') ADVANCE(178);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_ANDk);
      if (lookahead == 'r') ADVANCE(179);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_ANDr);
      END_STATE();
    case 85:
      ACCEPT_TOKEN(anon_sym_DEI2);
      if (lookahead == 'k') ADVANCE(180);
      if (lookahead == 'r') ADVANCE(181);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(anon_sym_DEIk);
      if (lookahead == 'r') ADVANCE(182);
      END_STATE();
    case 87:
      ACCEPT_TOKEN(anon_sym_DEIr);
      END_STATE();
    case 88:
      ACCEPT_TOKEN(anon_sym_DEO2);
      if (lookahead == 'k') ADVANCE(183);
      if (lookahead == 'r') ADVANCE(184);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_DEOk);
      if (lookahead == 'r') ADVANCE(185);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(anon_sym_DEOr);
      END_STATE();
    case 91:
      ACCEPT_TOKEN(anon_sym_DIV2);
      if (lookahead == 'k') ADVANCE(186);
      if (lookahead == 'r') ADVANCE(187);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(anon_sym_DIVk);
      if (lookahead == 'r') ADVANCE(188);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_DIVr);
      END_STATE();
    case 94:
      ACCEPT_TOKEN(anon_sym_DUP2);
      if (lookahead == 'k') ADVANCE(189);
      if (lookahead == 'r') ADVANCE(190);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(anon_sym_DUPk);
      if (lookahead == 'r') ADVANCE(191);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_DUPr);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(anon_sym_EOR2);
      if (lookahead == 'k') ADVANCE(192);
      if (lookahead == 'r') ADVANCE(193);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_EORk);
      if (lookahead == 'r') ADVANCE(194);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_EORr);
      END_STATE();
    case 100:
      ACCEPT_TOKEN(anon_sym_EQU2);
      if (lookahead == 'k') ADVANCE(195);
      if (lookahead == 'r') ADVANCE(196);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(anon_sym_EQUk);
      if (lookahead == 'r') ADVANCE(197);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_EQUr);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(anon_sym_GTH2);
      if (lookahead == 'k') ADVANCE(198);
      if (lookahead == 'r') ADVANCE(199);
      END_STATE();
    case 104:
      ACCEPT_TOKEN(anon_sym_GTHk);
      if (lookahead == 'r') ADVANCE(200);
      END_STATE();
    case 105:
      ACCEPT_TOKEN(anon_sym_GTHr);
      END_STATE();
    case 106:
      ACCEPT_TOKEN(anon_sym_INC2);
      if (lookahead == 'k') ADVANCE(201);
      if (lookahead == 'r') ADVANCE(202);
      END_STATE();
    case 107:
      ACCEPT_TOKEN(anon_sym_INCk);
      if (lookahead == 'r') ADVANCE(203);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_INCr);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(anon_sym_JCN2);
      if (lookahead == 'k') ADVANCE(204);
      if (lookahead == 'r') ADVANCE(205);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(anon_sym_JCNk);
      if (lookahead == 'r') ADVANCE(206);
      END_STATE();
    case 111:
      ACCEPT_TOKEN(anon_sym_JCNr);
      END_STATE();
    case 112:
      ACCEPT_TOKEN(anon_sym_JMP2);
      if (lookahead == 'k') ADVANCE(207);
      if (lookahead == 'r') ADVANCE(208);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_JMPk);
      if (lookahead == 'r') ADVANCE(209);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_JMPr);
      END_STATE();
    case 115:
      ACCEPT_TOKEN(anon_sym_JSR2);
      if (lookahead == 'k') ADVANCE(210);
      if (lookahead == 'r') ADVANCE(211);
      END_STATE();
    case 116:
      ACCEPT_TOKEN(anon_sym_JSRk);
      if (lookahead == 'r') ADVANCE(212);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(anon_sym_JSRr);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_LDA2);
      if (lookahead == 'k') ADVANCE(213);
      if (lookahead == 'r') ADVANCE(214);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_LDAk);
      if (lookahead == 'r') ADVANCE(215);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_LDAr);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_LDR2);
      if (lookahead == 'k') ADVANCE(216);
      if (lookahead == 'r') ADVANCE(217);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(anon_sym_LDRk);
      if (lookahead == 'r') ADVANCE(218);
      END_STATE();
    case 123:
      ACCEPT_TOKEN(anon_sym_LDRr);
      END_STATE();
    case 124:
      ACCEPT_TOKEN(anon_sym_LDZ2);
      if (lookahead == 'k') ADVANCE(219);
      if (lookahead == 'r') ADVANCE(220);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_LDZk);
      if (lookahead == 'r') ADVANCE(221);
      END_STATE();
    case 126:
      ACCEPT_TOKEN(anon_sym_LDZr);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_LIT2);
      if (lookahead == 'r') ADVANCE(222);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_LITr);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_LTH2);
      if (lookahead == 'k') ADVANCE(223);
      if (lookahead == 'r') ADVANCE(224);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_LTHk);
      if (lookahead == 'r') ADVANCE(225);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_LTHr);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(anon_sym_MUL2);
      if (lookahead == 'k') ADVANCE(226);
      if (lookahead == 'r') ADVANCE(227);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_MULk);
      if (lookahead == 'r') ADVANCE(228);
      END_STATE();
    case 134:
      ACCEPT_TOKEN(anon_sym_MULr);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(anon_sym_NEQ2);
      if (lookahead == 'k') ADVANCE(229);
      if (lookahead == 'r') ADVANCE(230);
      END_STATE();
    case 136:
      ACCEPT_TOKEN(anon_sym_NEQk);
      if (lookahead == 'r') ADVANCE(231);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(anon_sym_NEQr);
      END_STATE();
    case 138:
      ACCEPT_TOKEN(anon_sym_NIP2);
      if (lookahead == 'k') ADVANCE(232);
      if (lookahead == 'r') ADVANCE(233);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(anon_sym_NIPk);
      if (lookahead == 'r') ADVANCE(234);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(anon_sym_NIPr);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(anon_sym_ORA2);
      if (lookahead == 'k') ADVANCE(235);
      if (lookahead == 'r') ADVANCE(236);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(anon_sym_ORAk);
      if (lookahead == 'r') ADVANCE(237);
      END_STATE();
    case 143:
      ACCEPT_TOKEN(anon_sym_ORAr);
      END_STATE();
    case 144:
      ACCEPT_TOKEN(anon_sym_OVR2);
      if (lookahead == 'k') ADVANCE(238);
      if (lookahead == 'r') ADVANCE(239);
      END_STATE();
    case 145:
      ACCEPT_TOKEN(anon_sym_OVRk);
      if (lookahead == 'r') ADVANCE(240);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(anon_sym_OVRr);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(anon_sym_POP2);
      if (lookahead == 'k') ADVANCE(241);
      if (lookahead == 'r') ADVANCE(242);
      END_STATE();
    case 148:
      ACCEPT_TOKEN(anon_sym_POPk);
      if (lookahead == 'r') ADVANCE(243);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(anon_sym_POPr);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(anon_sym_ROT2);
      if (lookahead == 'k') ADVANCE(244);
      if (lookahead == 'r') ADVANCE(245);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(anon_sym_ROTk);
      if (lookahead == 'r') ADVANCE(246);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(anon_sym_ROTr);
      END_STATE();
    case 153:
      ACCEPT_TOKEN(anon_sym_SFT2);
      if (lookahead == 'k') ADVANCE(247);
      if (lookahead == 'r') ADVANCE(248);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(anon_sym_SFTk);
      if (lookahead == 'r') ADVANCE(249);
      END_STATE();
    case 155:
      ACCEPT_TOKEN(anon_sym_SFTr);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(anon_sym_STA2);
      if (lookahead == 'k') ADVANCE(250);
      if (lookahead == 'r') ADVANCE(251);
      END_STATE();
    case 157:
      ACCEPT_TOKEN(anon_sym_STAk);
      if (lookahead == 'r') ADVANCE(252);
      END_STATE();
    case 158:
      ACCEPT_TOKEN(anon_sym_STAr);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(anon_sym_STH2);
      if (lookahead == 'k') ADVANCE(253);
      if (lookahead == 'r') ADVANCE(254);
      END_STATE();
    case 160:
      ACCEPT_TOKEN(anon_sym_STHk);
      if (lookahead == 'r') ADVANCE(255);
      END_STATE();
    case 161:
      ACCEPT_TOKEN(anon_sym_STHr);
      END_STATE();
    case 162:
      ACCEPT_TOKEN(anon_sym_STR2);
      if (lookahead == 'k') ADVANCE(256);
      if (lookahead == 'r') ADVANCE(257);
      END_STATE();
    case 163:
      ACCEPT_TOKEN(anon_sym_STRk);
      if (lookahead == 'r') ADVANCE(258);
      END_STATE();
    case 164:
      ACCEPT_TOKEN(anon_sym_STRr);
      END_STATE();
    case 165:
      ACCEPT_TOKEN(anon_sym_STZ2);
      if (lookahead == 'k') ADVANCE(259);
      if (lookahead == 'r') ADVANCE(260);
      END_STATE();
    case 166:
      ACCEPT_TOKEN(anon_sym_STZk);
      if (lookahead == 'r') ADVANCE(261);
      END_STATE();
    case 167:
      ACCEPT_TOKEN(anon_sym_STZr);
      END_STATE();
    case 168:
      ACCEPT_TOKEN(anon_sym_SUB2);
      if (lookahead == 'k') ADVANCE(262);
      if (lookahead == 'r') ADVANCE(263);
      END_STATE();
    case 169:
      ACCEPT_TOKEN(anon_sym_SUBk);
      if (lookahead == 'r') ADVANCE(264);
      END_STATE();
    case 170:
      ACCEPT_TOKEN(anon_sym_SUBr);
      END_STATE();
    case 171:
      ACCEPT_TOKEN(anon_sym_SWP2);
      if (lookahead == 'k') ADVANCE(265);
      if (lookahead == 'r') ADVANCE(266);
      END_STATE();
    case 172:
      ACCEPT_TOKEN(anon_sym_SWPk);
      if (lookahead == 'r') ADVANCE(267);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(anon_sym_SWPr);
      END_STATE();
    case 174:
      ACCEPT_TOKEN(anon_sym_ADD2k);
      if (lookahead == 'r') ADVANCE(268);
      END_STATE();
    case 175:
      ACCEPT_TOKEN(anon_sym_ADD2r);
      END_STATE();
    case 176:
      ACCEPT_TOKEN(anon_sym_ADDkr);
      END_STATE();
    case 177:
      ACCEPT_TOKEN(anon_sym_AND2k);
      if (lookahead == 'r') ADVANCE(269);
      END_STATE();
    case 178:
      ACCEPT_TOKEN(anon_sym_AND2r);
      END_STATE();
    case 179:
      ACCEPT_TOKEN(anon_sym_ANDkr);
      END_STATE();
    case 180:
      ACCEPT_TOKEN(anon_sym_DEI2k);
      if (lookahead == 'r') ADVANCE(270);
      END_STATE();
    case 181:
      ACCEPT_TOKEN(anon_sym_DEI2r);
      END_STATE();
    case 182:
      ACCEPT_TOKEN(anon_sym_DEIkr);
      END_STATE();
    case 183:
      ACCEPT_TOKEN(anon_sym_DEO2k);
      if (lookahead == 'r') ADVANCE(271);
      END_STATE();
    case 184:
      ACCEPT_TOKEN(anon_sym_DEO2r);
      END_STATE();
    case 185:
      ACCEPT_TOKEN(anon_sym_DEOkr);
      END_STATE();
    case 186:
      ACCEPT_TOKEN(anon_sym_DIV2k);
      if (lookahead == 'r') ADVANCE(272);
      END_STATE();
    case 187:
      ACCEPT_TOKEN(anon_sym_DIV2r);
      END_STATE();
    case 188:
      ACCEPT_TOKEN(anon_sym_DIVkr);
      END_STATE();
    case 189:
      ACCEPT_TOKEN(anon_sym_DUP2k);
      if (lookahead == 'r') ADVANCE(273);
      END_STATE();
    case 190:
      ACCEPT_TOKEN(anon_sym_DUP2r);
      END_STATE();
    case 191:
      ACCEPT_TOKEN(anon_sym_DUPkr);
      END_STATE();
    case 192:
      ACCEPT_TOKEN(anon_sym_EOR2k);
      if (lookahead == 'r') ADVANCE(274);
      END_STATE();
    case 193:
      ACCEPT_TOKEN(anon_sym_EOR2r);
      END_STATE();
    case 194:
      ACCEPT_TOKEN(anon_sym_EORkr);
      END_STATE();
    case 195:
      ACCEPT_TOKEN(anon_sym_EQU2k);
      if (lookahead == 'r') ADVANCE(275);
      END_STATE();
    case 196:
      ACCEPT_TOKEN(anon_sym_EQU2r);
      END_STATE();
    case 197:
      ACCEPT_TOKEN(anon_sym_EQUkr);
      END_STATE();
    case 198:
      ACCEPT_TOKEN(anon_sym_GTH2k);
      if (lookahead == 'r') ADVANCE(276);
      END_STATE();
    case 199:
      ACCEPT_TOKEN(anon_sym_GTH2r);
      END_STATE();
    case 200:
      ACCEPT_TOKEN(anon_sym_GTHkr);
      END_STATE();
    case 201:
      ACCEPT_TOKEN(anon_sym_INC2k);
      if (lookahead == 'r') ADVANCE(277);
      END_STATE();
    case 202:
      ACCEPT_TOKEN(anon_sym_INC2r);
      END_STATE();
    case 203:
      ACCEPT_TOKEN(anon_sym_INCkr);
      END_STATE();
    case 204:
      ACCEPT_TOKEN(anon_sym_JCN2k);
      if (lookahead == 'r') ADVANCE(278);
      END_STATE();
    case 205:
      ACCEPT_TOKEN(anon_sym_JCN2r);
      END_STATE();
    case 206:
      ACCEPT_TOKEN(anon_sym_JCNkr);
      END_STATE();
    case 207:
      ACCEPT_TOKEN(anon_sym_JMP2k);
      if (lookahead == 'r') ADVANCE(279);
      END_STATE();
    case 208:
      ACCEPT_TOKEN(anon_sym_JMP2r);
      END_STATE();
    case 209:
      ACCEPT_TOKEN(anon_sym_JMPkr);
      END_STATE();
    case 210:
      ACCEPT_TOKEN(anon_sym_JSR2k);
      if (lookahead == 'r') ADVANCE(280);
      END_STATE();
    case 211:
      ACCEPT_TOKEN(anon_sym_JSR2r);
      END_STATE();
    case 212:
      ACCEPT_TOKEN(anon_sym_JSRkr);
      END_STATE();
    case 213:
      ACCEPT_TOKEN(anon_sym_LDA2k);
      if (lookahead == 'r') ADVANCE(281);
      END_STATE();
    case 214:
      ACCEPT_TOKEN(anon_sym_LDA2r);
      END_STATE();
    case 215:
      ACCEPT_TOKEN(anon_sym_LDAkr);
      END_STATE();
    case 216:
      ACCEPT_TOKEN(anon_sym_LDR2k);
      if (lookahead == 'r') ADVANCE(282);
      END_STATE();
    case 217:
      ACCEPT_TOKEN(anon_sym_LDR2r);
      END_STATE();
    case 218:
      ACCEPT_TOKEN(anon_sym_LDRkr);
      END_STATE();
    case 219:
      ACCEPT_TOKEN(anon_sym_LDZ2k);
      if (lookahead == 'r') ADVANCE(283);
      END_STATE();
    case 220:
      ACCEPT_TOKEN(anon_sym_LDZ2r);
      END_STATE();
    case 221:
      ACCEPT_TOKEN(anon_sym_LDZkr);
      END_STATE();
    case 222:
      ACCEPT_TOKEN(anon_sym_LIT2r);
      END_STATE();
    case 223:
      ACCEPT_TOKEN(anon_sym_LTH2k);
      if (lookahead == 'r') ADVANCE(284);
      END_STATE();
    case 224:
      ACCEPT_TOKEN(anon_sym_LTH2r);
      END_STATE();
    case 225:
      ACCEPT_TOKEN(anon_sym_LTHkr);
      END_STATE();
    case 226:
      ACCEPT_TOKEN(anon_sym_MUL2k);
      if (lookahead == 'r') ADVANCE(285);
      END_STATE();
    case 227:
      ACCEPT_TOKEN(anon_sym_MUL2r);
      END_STATE();
    case 228:
      ACCEPT_TOKEN(anon_sym_MULkr);
      END_STATE();
    case 229:
      ACCEPT_TOKEN(anon_sym_NEQ2k);
      if (lookahead == 'r') ADVANCE(286);
      END_STATE();
    case 230:
      ACCEPT_TOKEN(anon_sym_NEQ2r);
      END_STATE();
    case 231:
      ACCEPT_TOKEN(anon_sym_NEQkr);
      END_STATE();
    case 232:
      ACCEPT_TOKEN(anon_sym_NIP2k);
      if (lookahead == 'r') ADVANCE(287);
      END_STATE();
    case 233:
      ACCEPT_TOKEN(anon_sym_NIP2r);
      END_STATE();
    case 234:
      ACCEPT_TOKEN(anon_sym_NIPkr);
      END_STATE();
    case 235:
      ACCEPT_TOKEN(anon_sym_ORA2k);
      if (lookahead == 'r') ADVANCE(288);
      END_STATE();
    case 236:
      ACCEPT_TOKEN(anon_sym_ORA2r);
      END_STATE();
    case 237:
      ACCEPT_TOKEN(anon_sym_ORAkr);
      END_STATE();
    case 238:
      ACCEPT_TOKEN(anon_sym_OVR2k);
      if (lookahead == 'r') ADVANCE(289);
      END_STATE();
    case 239:
      ACCEPT_TOKEN(anon_sym_OVR2r);
      END_STATE();
    case 240:
      ACCEPT_TOKEN(anon_sym_OVRkr);
      END_STATE();
    case 241:
      ACCEPT_TOKEN(anon_sym_POP2k);
      if (lookahead == 'r') ADVANCE(290);
      END_STATE();
    case 242:
      ACCEPT_TOKEN(anon_sym_POP2r);
      END_STATE();
    case 243:
      ACCEPT_TOKEN(anon_sym_POPkr);
      END_STATE();
    case 244:
      ACCEPT_TOKEN(anon_sym_ROT2k);
      if (lookahead == 'r') ADVANCE(291);
      END_STATE();
    case 245:
      ACCEPT_TOKEN(anon_sym_ROT2r);
      END_STATE();
    case 246:
      ACCEPT_TOKEN(anon_sym_ROTkr);
      END_STATE();
    case 247:
      ACCEPT_TOKEN(anon_sym_SFT2k);
      if (lookahead == 'r') ADVANCE(292);
      END_STATE();
    case 248:
      ACCEPT_TOKEN(anon_sym_SFT2r);
      END_STATE();
    case 249:
      ACCEPT_TOKEN(anon_sym_SFTkr);
      END_STATE();
    case 250:
      ACCEPT_TOKEN(anon_sym_STA2k);
      if (lookahead == 'r') ADVANCE(293);
      END_STATE();
    case 251:
      ACCEPT_TOKEN(anon_sym_STA2r);
      END_STATE();
    case 252:
      ACCEPT_TOKEN(anon_sym_STAkr);
      END_STATE();
    case 253:
      ACCEPT_TOKEN(anon_sym_STH2k);
      if (lookahead == 'r') ADVANCE(294);
      END_STATE();
    case 254:
      ACCEPT_TOKEN(anon_sym_STH2r);
      END_STATE();
    case 255:
      ACCEPT_TOKEN(anon_sym_STHkr);
      END_STATE();
    case 256:
      ACCEPT_TOKEN(anon_sym_STR2k);
      if (lookahead == 'r') ADVANCE(295);
      END_STATE();
    case 257:
      ACCEPT_TOKEN(anon_sym_STR2r);
      END_STATE();
    case 258:
      ACCEPT_TOKEN(anon_sym_STRkr);
      END_STATE();
    case 259:
      ACCEPT_TOKEN(anon_sym_STZ2k);
      if (lookahead == 'r') ADVANCE(296);
      END_STATE();
    case 260:
      ACCEPT_TOKEN(anon_sym_STZ2r);
      END_STATE();
    case 261:
      ACCEPT_TOKEN(anon_sym_STZkr);
      END_STATE();
    case 262:
      ACCEPT_TOKEN(anon_sym_SUB2k);
      if (lookahead == 'r') ADVANCE(297);
      END_STATE();
    case 263:
      ACCEPT_TOKEN(anon_sym_SUB2r);
      END_STATE();
    case 264:
      ACCEPT_TOKEN(anon_sym_SUBkr);
      END_STATE();
    case 265:
      ACCEPT_TOKEN(anon_sym_SWP2k);
      if (lookahead == 'r') ADVANCE(298);
      END_STATE();
    case 266:
      ACCEPT_TOKEN(anon_sym_SWP2r);
      END_STATE();
    case 267:
      ACCEPT_TOKEN(anon_sym_SWPkr);
      END_STATE();
    case 268:
      ACCEPT_TOKEN(anon_sym_ADD2kr);
      END_STATE();
    case 269:
      ACCEPT_TOKEN(anon_sym_AND2kr);
      END_STATE();
    case 270:
      ACCEPT_TOKEN(anon_sym_DEI2kr);
      END_STATE();
    case 271:
      ACCEPT_TOKEN(anon_sym_DEO2kr);
      END_STATE();
    case 272:
      ACCEPT_TOKEN(anon_sym_DIV2kr);
      END_STATE();
    case 273:
      ACCEPT_TOKEN(anon_sym_DUP2kr);
      END_STATE();
    case 274:
      ACCEPT_TOKEN(anon_sym_EOR2kr);
      END_STATE();
    case 275:
      ACCEPT_TOKEN(anon_sym_EQU2kr);
      END_STATE();
    case 276:
      ACCEPT_TOKEN(anon_sym_GTH2kr);
      END_STATE();
    case 277:
      ACCEPT_TOKEN(anon_sym_INC2kr);
      END_STATE();
    case 278:
      ACCEPT_TOKEN(anon_sym_JCN2kr);
      END_STATE();
    case 279:
      ACCEPT_TOKEN(anon_sym_JMP2kr);
      END_STATE();
    case 280:
      ACCEPT_TOKEN(anon_sym_JSR2kr);
      END_STATE();
    case 281:
      ACCEPT_TOKEN(anon_sym_LDA2kr);
      END_STATE();
    case 282:
      ACCEPT_TOKEN(anon_sym_LDR2kr);
      END_STATE();
    case 283:
      ACCEPT_TOKEN(anon_sym_LDZ2kr);
      END_STATE();
    case 284:
      ACCEPT_TOKEN(anon_sym_LTH2kr);
      END_STATE();
    case 285:
      ACCEPT_TOKEN(anon_sym_MUL2kr);
      END_STATE();
    case 286:
      ACCEPT_TOKEN(anon_sym_NEQ2kr);
      END_STATE();
    case 287:
      ACCEPT_TOKEN(anon_sym_NIP2kr);
      END_STATE();
    case 288:
      ACCEPT_TOKEN(anon_sym_ORA2kr);
      END_STATE();
    case 289:
      ACCEPT_TOKEN(anon_sym_OVR2kr);
      END_STATE();
    case 290:
      ACCEPT_TOKEN(anon_sym_POP2kr);
      END_STATE();
    case 291:
      ACCEPT_TOKEN(anon_sym_ROT2kr);
      END_STATE();
    case 292:
      ACCEPT_TOKEN(anon_sym_SFT2kr);
      END_STATE();
    case 293:
      ACCEPT_TOKEN(anon_sym_STA2kr);
      END_STATE();
    case 294:
      ACCEPT_TOKEN(anon_sym_STH2kr);
      END_STATE();
    case 295:
      ACCEPT_TOKEN(anon_sym_STR2kr);
      END_STATE();
    case 296:
      ACCEPT_TOKEN(anon_sym_STZ2kr);
      END_STATE();
    case 297:
      ACCEPT_TOKEN(anon_sym_SUB2kr);
      END_STATE();
    case 298:
      ACCEPT_TOKEN(anon_sym_SWP2kr);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 0, .external_lex_state = 1},
  [2] = {.lex_state = 14, .external_lex_state = 1},
  [3] = {.lex_state = 14, .external_lex_state = 1},
  [4] = {.lex_state = 14, .external_lex_state = 1},
  [5] = {.lex_state = 14, .external_lex_state = 1},
  [6] = {.lex_state = 14, .external_lex_state = 1},
  [7] = {.lex_state = 14, .external_lex_state = 1},
  [8] = {.lex_state = 14, .external_lex_state = 1},
  [9] = {.lex_state = 14, .external_lex_state = 1},
  [10] = {.lex_state = 14, .external_lex_state = 1},
  [11] = {.lex_state = 0, .external_lex_state = 1},
  [12] = {.lex_state = 0, .external_lex_state = 1},
  [13] = {.lex_state = 0, .external_lex_state = 1},
  [14] = {.lex_state = 14, .external_lex_state = 1},
  [15] = {.lex_state = 14, .external_lex_state = 1},
  [16] = {.lex_state = 14, .external_lex_state = 1},
  [17] = {.lex_state = 14, .external_lex_state = 1},
  [18] = {.lex_state = 14, .external_lex_state = 1},
  [19] = {.lex_state = 14, .external_lex_state = 1},
  [20] = {.lex_state = 14, .external_lex_state = 1},
  [21] = {.lex_state = 14, .external_lex_state = 1},
  [22] = {.lex_state = 14, .external_lex_state = 1},
  [23] = {.lex_state = 14, .external_lex_state = 1},
  [24] = {.lex_state = 14, .external_lex_state = 1},
  [25] = {.lex_state = 1, .external_lex_state = 1},
  [26] = {.lex_state = 1, .external_lex_state = 1},
  [27] = {.lex_state = 1, .external_lex_state = 1},
  [28] = {.lex_state = 0, .external_lex_state = 1},
  [29] = {.lex_state = 0, .external_lex_state = 1},
  [30] = {.lex_state = 1, .external_lex_state = 1},
  [31] = {.lex_state = 1, .external_lex_state = 1},
  [32] = {.lex_state = 1, .external_lex_state = 1},
  [33] = {.lex_state = 1, .external_lex_state = 1},
  [34] = {.lex_state = 1, .external_lex_state = 1},
  [35] = {.lex_state = 0, .external_lex_state = 1},
  [36] = {.lex_state = 0, .external_lex_state = 1},
  [37] = {.lex_state = 0, .external_lex_state = 1},
  [38] = {.lex_state = 6, .external_lex_state = 1},
  [39] = {.lex_state = 7, .external_lex_state = 1},
  [40] = {.lex_state = 0, .external_lex_state = 1},
  [41] = {.lex_state = 0, .external_lex_state = 1},
  [42] = {.lex_state = 0, .external_lex_state = 1},
  [43] = {.lex_state = 8, .external_lex_state = 1},
  [44] = {.lex_state = 8, .external_lex_state = 1},
  [45] = {.lex_state = 9, .external_lex_state = 1},
  [46] = {.lex_state = 0, .external_lex_state = 1},
  [47] = {.lex_state = 10, .external_lex_state = 1},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_BRK] = ACTIONS(1),
    [anon_sym_INC] = ACTIONS(1),
    [anon_sym_POP] = ACTIONS(1),
    [anon_sym_NIP] = ACTIONS(1),
    [anon_sym_SWP] = ACTIONS(1),
    [anon_sym_ROT] = ACTIONS(1),
    [anon_sym_DUP] = ACTIONS(1),
    [anon_sym_OVR] = ACTIONS(1),
    [anon_sym_EQU] = ACTIONS(1),
    [anon_sym_NEQ] = ACTIONS(1),
    [anon_sym_GTH] = ACTIONS(1),
    [anon_sym_LTH] = ACTIONS(1),
    [anon_sym_JMP] = ACTIONS(1),
    [anon_sym_JCN] = ACTIONS(1),
    [anon_sym_JSR] = ACTIONS(1),
    [anon_sym_STH] = ACTIONS(1),
    [anon_sym_LDZ] = ACTIONS(1),
    [anon_sym_STZ] = ACTIONS(1),
    [anon_sym_LDR] = ACTIONS(1),
    [anon_sym_STR] = ACTIONS(1),
    [anon_sym_LDA] = ACTIONS(1),
    [anon_sym_STA] = ACTIONS(1),
    [anon_sym_DEI] = ACTIONS(1),
    [anon_sym_DEO] = ACTIONS(1),
    [anon_sym_ADD] = ACTIONS(1),
    [anon_sym_SUB] = ACTIONS(1),
    [anon_sym_MUL] = ACTIONS(1),
    [anon_sym_DIV] = ACTIONS(1),
    [anon_sym_AND] = ACTIONS(1),
    [anon_sym_ORA] = ACTIONS(1),
    [anon_sym_EOR] = ACTIONS(1),
    [anon_sym_SFT] = ACTIONS(1),
    [anon_sym_JCI] = ACTIONS(1),
    [anon_sym_INC2] = ACTIONS(1),
    [anon_sym_POP2] = ACTIONS(1),
    [anon_sym_NIP2] = ACTIONS(1),
    [anon_sym_SWP2] = ACTIONS(1),
    [anon_sym_ROT2] = ACTIONS(1),
    [anon_sym_DUP2] = ACTIONS(1),
    [anon_sym_OVR2] = ACTIONS(1),
    [anon_sym_EQU2] = ACTIONS(1),
    [anon_sym_NEQ2] = ACTIONS(1),
    [anon_sym_GTH2] = ACTIONS(1),
    [anon_sym_LTH2] = ACTIONS(1),
    [anon_sym_JMP2] = ACTIONS(1),
    [anon_sym_JCN2] = ACTIONS(1),
    [anon_sym_JSR2] = ACTIONS(1),
    [anon_sym_STH2] = ACTIONS(1),
    [anon_sym_LDZ2] = ACTIONS(1),
    [anon_sym_STZ2] = ACTIONS(1),
    [anon_sym_LDR2] = ACTIONS(1),
    [anon_sym_STR2] = ACTIONS(1),
    [anon_sym_LDA2] = ACTIONS(1),
    [anon_sym_STA2] = ACTIONS(1),
    [anon_sym_DEI2] = ACTIONS(1),
    [anon_sym_DEO2] = ACTIONS(1),
    [anon_sym_ADD2] = ACTIONS(1),
    [anon_sym_SUB2] = ACTIONS(1),
    [anon_sym_MUL2] = ACTIONS(1),
    [anon_sym_DIV2] = ACTIONS(1),
    [anon_sym_AND2] = ACTIONS(1),
    [anon_sym_ORA2] = ACTIONS(1),
    [anon_sym_EOR2] = ACTIONS(1),
    [anon_sym_SFT2] = ACTIONS(1),
    [anon_sym_JMI] = ACTIONS(1),
    [anon_sym_INCr] = ACTIONS(1),
    [anon_sym_POPr] = ACTIONS(1),
    [anon_sym_NIPr] = ACTIONS(1),
    [anon_sym_SWPr] = ACTIONS(1),
    [anon_sym_ROTr] = ACTIONS(1),
    [anon_sym_DUPr] = ACTIONS(1),
    [anon_sym_OVRr] = ACTIONS(1),
    [anon_sym_EQUr] = ACTIONS(1),
    [anon_sym_NEQr] = ACTIONS(1),
    [anon_sym_GTHr] = ACTIONS(1),
    [anon_sym_LTHr] = ACTIONS(1),
    [anon_sym_JMPr] = ACTIONS(1),
    [anon_sym_JCNr] = ACTIONS(1),
    [anon_sym_JSRr] = ACTIONS(1),
    [anon_sym_STHr] = ACTIONS(1),
    [anon_sym_LDZr] = ACTIONS(1),
    [anon_sym_STZr] = ACTIONS(1),
    [anon_sym_LDRr] = ACTIONS(1),
    [anon_sym_STRr] = ACTIONS(1),
    [anon_sym_LDAr] = ACTIONS(1),
    [anon_sym_STAr] = ACTIONS(1),
    [anon_sym_DEIr] = ACTIONS(1),
    [anon_sym_DEOr] = ACTIONS(1),
    [anon_sym_ADDr] = ACTIONS(1),
    [anon_sym_SUBr] = ACTIONS(1),
    [anon_sym_MULr] = ACTIONS(1),
    [anon_sym_DIVr] = ACTIONS(1),
    [anon_sym_ANDr] = ACTIONS(1),
    [anon_sym_ORAr] = ACTIONS(1),
    [anon_sym_EORr] = ACTIONS(1),
    [anon_sym_SFTr] = ACTIONS(1),
    [anon_sym_JSI] = ACTIONS(1),
    [anon_sym_INC2r] = ACTIONS(1),
    [anon_sym_POP2r] = ACTIONS(1),
    [anon_sym_NIP2r] = ACTIONS(1),
    [anon_sym_SWP2r] = ACTIONS(1),
    [anon_sym_ROT2r] = ACTIONS(1),
    [anon_sym_DUP2r] = ACTIONS(1),
    [anon_sym_OVR2r] = ACTIONS(1),
    [anon_sym_EQU2r] = ACTIONS(1),
    [anon_sym_NEQ2r] = ACTIONS(1),
    [anon_sym_GTH2r] = ACTIONS(1),
    [anon_sym_LTH2r] = ACTIONS(1),
    [anon_sym_JMP2r] = ACTIONS(1),
    [anon_sym_JCN2r] = ACTIONS(1),
    [anon_sym_JSR2r] = ACTIONS(1),
    [anon_sym_STH2r] = ACTIONS(1),
    [anon_sym_LDZ2r] = ACTIONS(1),
    [anon_sym_STZ2r] = ACTIONS(1),
    [anon_sym_LDR2r] = ACTIONS(1),
    [anon_sym_STR2r] = ACTIONS(1),
    [anon_sym_LDA2r] = ACTIONS(1),
    [anon_sym_STA2r] = ACTIONS(1),
    [anon_sym_DEI2r] = ACTIONS(1),
    [anon_sym_DEO2r] = ACTIONS(1),
    [anon_sym_ADD2r] = ACTIONS(1),
    [anon_sym_SUB2r] = ACTIONS(1),
    [anon_sym_MUL2r] = ACTIONS(1),
    [anon_sym_DIV2r] = ACTIONS(1),
    [anon_sym_AND2r] = ACTIONS(1),
    [anon_sym_ORA2r] = ACTIONS(1),
    [anon_sym_EOR2r] = ACTIONS(1),
    [anon_sym_SFT2r] = ACTIONS(1),
    [anon_sym_LIT] = ACTIONS(1),
    [anon_sym_INCk] = ACTIONS(1),
    [anon_sym_POPk] = ACTIONS(1),
    [anon_sym_NIPk] = ACTIONS(1),
    [anon_sym_SWPk] = ACTIONS(1),
    [anon_sym_ROTk] = ACTIONS(1),
    [anon_sym_DUPk] = ACTIONS(1),
    [anon_sym_OVRk] = ACTIONS(1),
    [anon_sym_EQUk] = ACTIONS(1),
    [anon_sym_NEQk] = ACTIONS(1),
    [anon_sym_GTHk] = ACTIONS(1),
    [anon_sym_LTHk] = ACTIONS(1),
    [anon_sym_JMPk] = ACTIONS(1),
    [anon_sym_JCNk] = ACTIONS(1),
    [anon_sym_JSRk] = ACTIONS(1),
    [anon_sym_STHk] = ACTIONS(1),
    [anon_sym_LDZk] = ACTIONS(1),
    [anon_sym_STZk] = ACTIONS(1),
    [anon_sym_LDRk] = ACTIONS(1),
    [anon_sym_STRk] = ACTIONS(1),
    [anon_sym_LDAk] = ACTIONS(1),
    [anon_sym_STAk] = ACTIONS(1),
    [anon_sym_DEIk] = ACTIONS(1),
    [anon_sym_DEOk] = ACTIONS(1),
    [anon_sym_ADDk] = ACTIONS(1),
    [anon_sym_SUBk] = ACTIONS(1),
    [anon_sym_MULk] = ACTIONS(1),
    [anon_sym_DIVk] = ACTIONS(1),
    [anon_sym_ANDk] = ACTIONS(1),
    [anon_sym_ORAk] = ACTIONS(1),
    [anon_sym_EORk] = ACTIONS(1),
    [anon_sym_SFTk] = ACTIONS(1),
    [anon_sym_LIT2] = ACTIONS(1),
    [anon_sym_INC2k] = ACTIONS(1),
    [anon_sym_POP2k] = ACTIONS(1),
    [anon_sym_NIP2k] = ACTIONS(1),
    [anon_sym_SWP2k] = ACTIONS(1),
    [anon_sym_ROT2k] = ACTIONS(1),
    [anon_sym_DUP2k] = ACTIONS(1),
    [anon_sym_OVR2k] = ACTIONS(1),
    [anon_sym_EQU2k] = ACTIONS(1),
    [anon_sym_NEQ2k] = ACTIONS(1),
    [anon_sym_GTH2k] = ACTIONS(1),
    [anon_sym_LTH2k] = ACTIONS(1),
    [anon_sym_JMP2k] = ACTIONS(1),
    [anon_sym_JCN2k] = ACTIONS(1),
    [anon_sym_JSR2k] = ACTIONS(1),
    [anon_sym_STH2k] = ACTIONS(1),
    [anon_sym_LDZ2k] = ACTIONS(1),
    [anon_sym_STZ2k] = ACTIONS(1),
    [anon_sym_LDR2k] = ACTIONS(1),
    [anon_sym_STR2k] = ACTIONS(1),
    [anon_sym_LDA2k] = ACTIONS(1),
    [anon_sym_STA2k] = ACTIONS(1),
    [anon_sym_DEI2k] = ACTIONS(1),
    [anon_sym_DEO2k] = ACTIONS(1),
    [anon_sym_ADD2k] = ACTIONS(1),
    [anon_sym_SUB2k] = ACTIONS(1),
    [anon_sym_MUL2k] = ACTIONS(1),
    [anon_sym_DIV2k] = ACTIONS(1),
    [anon_sym_AND2k] = ACTIONS(1),
    [anon_sym_ORA2k] = ACTIONS(1),
    [anon_sym_EOR2k] = ACTIONS(1),
    [anon_sym_SFT2k] = ACTIONS(1),
    [anon_sym_LITr] = ACTIONS(1),
    [anon_sym_INCkr] = ACTIONS(1),
    [anon_sym_POPkr] = ACTIONS(1),
    [anon_sym_NIPkr] = ACTIONS(1),
    [anon_sym_SWPkr] = ACTIONS(1),
    [anon_sym_ROTkr] = ACTIONS(1),
    [anon_sym_DUPkr] = ACTIONS(1),
    [anon_sym_OVRkr] = ACTIONS(1),
    [anon_sym_EQUkr] = ACTIONS(1),
    [anon_sym_NEQkr] = ACTIONS(1),
    [anon_sym_GTHkr] = ACTIONS(1),
    [anon_sym_LTHkr] = ACTIONS(1),
    [anon_sym_JMPkr] = ACTIONS(1),
    [anon_sym_JCNkr] = ACTIONS(1),
    [anon_sym_JSRkr] = ACTIONS(1),
    [anon_sym_STHkr] = ACTIONS(1),
    [anon_sym_LDZkr] = ACTIONS(1),
    [anon_sym_STZkr] = ACTIONS(1),
    [anon_sym_LDRkr] = ACTIONS(1),
    [anon_sym_STRkr] = ACTIONS(1),
    [anon_sym_LDAkr] = ACTIONS(1),
    [anon_sym_STAkr] = ACTIONS(1),
    [anon_sym_DEIkr] = ACTIONS(1),
    [anon_sym_DEOkr] = ACTIONS(1),
    [anon_sym_ADDkr] = ACTIONS(1),
    [anon_sym_SUBkr] = ACTIONS(1),
    [anon_sym_MULkr] = ACTIONS(1),
    [anon_sym_DIVkr] = ACTIONS(1),
    [anon_sym_ANDkr] = ACTIONS(1),
    [anon_sym_ORAkr] = ACTIONS(1),
    [anon_sym_EORkr] = ACTIONS(1),
    [anon_sym_SFTkr] = ACTIONS(1),
    [anon_sym_LIT2r] = ACTIONS(1),
    [anon_sym_INC2kr] = ACTIONS(1),
    [anon_sym_POP2kr] = ACTIONS(1),
    [anon_sym_NIP2kr] = ACTIONS(1),
    [anon_sym_SWP2kr] = ACTIONS(1),
    [anon_sym_ROT2kr] = ACTIONS(1),
    [anon_sym_DUP2kr] = ACTIONS(1),
    [anon_sym_OVR2kr] = ACTIONS(1),
    [anon_sym_EQU2kr] = ACTIONS(1),
    [anon_sym_NEQ2kr] = ACTIONS(1),
    [anon_sym_GTH2kr] = ACTIONS(1),
    [anon_sym_LTH2kr] = ACTIONS(1),
    [anon_sym_JMP2kr] = ACTIONS(1),
    [anon_sym_JCN2kr] = ACTIONS(1),
    [anon_sym_JSR2kr] = ACTIONS(1),
    [anon_sym_STH2kr] = ACTIONS(1),
    [anon_sym_LDZ2kr] = ACTIONS(1),
    [anon_sym_STZ2kr] = ACTIONS(1),
    [anon_sym_LDR2kr] = ACTIONS(1),
    [anon_sym_STR2kr] = ACTIONS(1),
    [anon_sym_LDA2kr] = ACTIONS(1),
    [anon_sym_STA2kr] = ACTIONS(1),
    [anon_sym_DEI2kr] = ACTIONS(1),
    [anon_sym_DEO2kr] = ACTIONS(1),
    [anon_sym_ADD2kr] = ACTIONS(1),
    [anon_sym_SUB2kr] = ACTIONS(1),
    [anon_sym_MUL2kr] = ACTIONS(1),
    [anon_sym_DIV2kr] = ACTIONS(1),
    [anon_sym_AND2kr] = ACTIONS(1),
    [anon_sym_ORA2kr] = ACTIONS(1),
    [anon_sym_EOR2kr] = ACTIONS(1),
    [anon_sym_SFT2kr] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_DOLLAR] = ACTIONS(1),
    [anon_sym_POUND] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym__] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_QMARK] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_DQUOTE] = ACTIONS(1),
    [sym_number] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
  },
  [1] = {
    [sym_program] = STATE(42),
    [sym_memory_execution] = STATE(29),
    [sym_subroutine] = STATE(29),
    [sym_macro] = STATE(29),
    [sym_include] = STATE(29),
    [sym_absolute_pad_operation] = STATE(3),
    [sym_label] = STATE(4),
    [aux_sym_program_repeat1] = STATE(29),
    [ts_builtin_sym_end] = ACTIONS(5),
    [anon_sym_PERCENT] = ACTIONS(7),
    [anon_sym_TILDE] = ACTIONS(9),
    [anon_sym_PIPE] = ACTIONS(11),
    [anon_sym_AT] = ACTIONS(13),
    [sym_comment] = ACTIONS(3),
  },
  [2] = {
    [sym__non_toplevel_statement] = STATE(2),
    [sym_opcode] = STATE(2),
    [sym_relative_pad_operation] = STATE(2),
    [sym_hex_literal] = STATE(2),
    [sym_sublabel_reference] = STATE(2),
    [sym_rune] = STATE(2),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(2),
    [sym_raw_ascii] = STATE(2),
    [aux_sym_memory_execution_repeat1] = STATE(2),
    [ts_builtin_sym_end] = ACTIONS(15),
    [sym_identifier] = ACTIONS(17),
    [anon_sym_PERCENT] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(15),
    [anon_sym_TILDE] = ACTIONS(15),
    [anon_sym_BRK] = ACTIONS(20),
    [anon_sym_INC] = ACTIONS(23),
    [anon_sym_POP] = ACTIONS(23),
    [anon_sym_NIP] = ACTIONS(23),
    [anon_sym_SWP] = ACTIONS(23),
    [anon_sym_ROT] = ACTIONS(23),
    [anon_sym_DUP] = ACTIONS(23),
    [anon_sym_OVR] = ACTIONS(23),
    [anon_sym_EQU] = ACTIONS(23),
    [anon_sym_NEQ] = ACTIONS(23),
    [anon_sym_GTH] = ACTIONS(23),
    [anon_sym_LTH] = ACTIONS(23),
    [anon_sym_JMP] = ACTIONS(23),
    [anon_sym_JCN] = ACTIONS(23),
    [anon_sym_JSR] = ACTIONS(23),
    [anon_sym_STH] = ACTIONS(23),
    [anon_sym_LDZ] = ACTIONS(23),
    [anon_sym_STZ] = ACTIONS(23),
    [anon_sym_LDR] = ACTIONS(23),
    [anon_sym_STR] = ACTIONS(23),
    [anon_sym_LDA] = ACTIONS(23),
    [anon_sym_STA] = ACTIONS(23),
    [anon_sym_DEI] = ACTIONS(23),
    [anon_sym_DEO] = ACTIONS(23),
    [anon_sym_ADD] = ACTIONS(23),
    [anon_sym_SUB] = ACTIONS(23),
    [anon_sym_MUL] = ACTIONS(23),
    [anon_sym_DIV] = ACTIONS(23),
    [anon_sym_AND] = ACTIONS(23),
    [anon_sym_ORA] = ACTIONS(23),
    [anon_sym_EOR] = ACTIONS(23),
    [anon_sym_SFT] = ACTIONS(23),
    [anon_sym_JCI] = ACTIONS(20),
    [anon_sym_INC2] = ACTIONS(23),
    [anon_sym_POP2] = ACTIONS(23),
    [anon_sym_NIP2] = ACTIONS(23),
    [anon_sym_SWP2] = ACTIONS(23),
    [anon_sym_ROT2] = ACTIONS(23),
    [anon_sym_DUP2] = ACTIONS(23),
    [anon_sym_OVR2] = ACTIONS(23),
    [anon_sym_EQU2] = ACTIONS(23),
    [anon_sym_NEQ2] = ACTIONS(23),
    [anon_sym_GTH2] = ACTIONS(23),
    [anon_sym_LTH2] = ACTIONS(23),
    [anon_sym_JMP2] = ACTIONS(23),
    [anon_sym_JCN2] = ACTIONS(23),
    [anon_sym_JSR2] = ACTIONS(23),
    [anon_sym_STH2] = ACTIONS(23),
    [anon_sym_LDZ2] = ACTIONS(23),
    [anon_sym_STZ2] = ACTIONS(23),
    [anon_sym_LDR2] = ACTIONS(23),
    [anon_sym_STR2] = ACTIONS(23),
    [anon_sym_LDA2] = ACTIONS(23),
    [anon_sym_STA2] = ACTIONS(23),
    [anon_sym_DEI2] = ACTIONS(23),
    [anon_sym_DEO2] = ACTIONS(23),
    [anon_sym_ADD2] = ACTIONS(23),
    [anon_sym_SUB2] = ACTIONS(23),
    [anon_sym_MUL2] = ACTIONS(23),
    [anon_sym_DIV2] = ACTIONS(23),
    [anon_sym_AND2] = ACTIONS(23),
    [anon_sym_ORA2] = ACTIONS(23),
    [anon_sym_EOR2] = ACTIONS(23),
    [anon_sym_SFT2] = ACTIONS(23),
    [anon_sym_JMI] = ACTIONS(20),
    [anon_sym_INCr] = ACTIONS(20),
    [anon_sym_POPr] = ACTIONS(20),
    [anon_sym_NIPr] = ACTIONS(20),
    [anon_sym_SWPr] = ACTIONS(20),
    [anon_sym_ROTr] = ACTIONS(20),
    [anon_sym_DUPr] = ACTIONS(20),
    [anon_sym_OVRr] = ACTIONS(20),
    [anon_sym_EQUr] = ACTIONS(20),
    [anon_sym_NEQr] = ACTIONS(20),
    [anon_sym_GTHr] = ACTIONS(20),
    [anon_sym_LTHr] = ACTIONS(20),
    [anon_sym_JMPr] = ACTIONS(20),
    [anon_sym_JCNr] = ACTIONS(20),
    [anon_sym_JSRr] = ACTIONS(20),
    [anon_sym_STHr] = ACTIONS(20),
    [anon_sym_LDZr] = ACTIONS(20),
    [anon_sym_STZr] = ACTIONS(20),
    [anon_sym_LDRr] = ACTIONS(20),
    [anon_sym_STRr] = ACTIONS(20),
    [anon_sym_LDAr] = ACTIONS(20),
    [anon_sym_STAr] = ACTIONS(20),
    [anon_sym_DEIr] = ACTIONS(20),
    [anon_sym_DEOr] = ACTIONS(20),
    [anon_sym_ADDr] = ACTIONS(20),
    [anon_sym_SUBr] = ACTIONS(20),
    [anon_sym_MULr] = ACTIONS(20),
    [anon_sym_DIVr] = ACTIONS(20),
    [anon_sym_ANDr] = ACTIONS(20),
    [anon_sym_ORAr] = ACTIONS(20),
    [anon_sym_EORr] = ACTIONS(20),
    [anon_sym_SFTr] = ACTIONS(20),
    [anon_sym_JSI] = ACTIONS(20),
    [anon_sym_INC2r] = ACTIONS(20),
    [anon_sym_POP2r] = ACTIONS(20),
    [anon_sym_NIP2r] = ACTIONS(20),
    [anon_sym_SWP2r] = ACTIONS(20),
    [anon_sym_ROT2r] = ACTIONS(20),
    [anon_sym_DUP2r] = ACTIONS(20),
    [anon_sym_OVR2r] = ACTIONS(20),
    [anon_sym_EQU2r] = ACTIONS(20),
    [anon_sym_NEQ2r] = ACTIONS(20),
    [anon_sym_GTH2r] = ACTIONS(20),
    [anon_sym_LTH2r] = ACTIONS(20),
    [anon_sym_JMP2r] = ACTIONS(20),
    [anon_sym_JCN2r] = ACTIONS(20),
    [anon_sym_JSR2r] = ACTIONS(20),
    [anon_sym_STH2r] = ACTIONS(20),
    [anon_sym_LDZ2r] = ACTIONS(20),
    [anon_sym_STZ2r] = ACTIONS(20),
    [anon_sym_LDR2r] = ACTIONS(20),
    [anon_sym_STR2r] = ACTIONS(20),
    [anon_sym_LDA2r] = ACTIONS(20),
    [anon_sym_STA2r] = ACTIONS(20),
    [anon_sym_DEI2r] = ACTIONS(20),
    [anon_sym_DEO2r] = ACTIONS(20),
    [anon_sym_ADD2r] = ACTIONS(20),
    [anon_sym_SUB2r] = ACTIONS(20),
    [anon_sym_MUL2r] = ACTIONS(20),
    [anon_sym_DIV2r] = ACTIONS(20),
    [anon_sym_AND2r] = ACTIONS(20),
    [anon_sym_ORA2r] = ACTIONS(20),
    [anon_sym_EOR2r] = ACTIONS(20),
    [anon_sym_SFT2r] = ACTIONS(20),
    [anon_sym_LIT] = ACTIONS(23),
    [anon_sym_INCk] = ACTIONS(23),
    [anon_sym_POPk] = ACTIONS(23),
    [anon_sym_NIPk] = ACTIONS(23),
    [anon_sym_SWPk] = ACTIONS(23),
    [anon_sym_ROTk] = ACTIONS(23),
    [anon_sym_DUPk] = ACTIONS(23),
    [anon_sym_OVRk] = ACTIONS(23),
    [anon_sym_EQUk] = ACTIONS(23),
    [anon_sym_NEQk] = ACTIONS(23),
    [anon_sym_GTHk] = ACTIONS(23),
    [anon_sym_LTHk] = ACTIONS(23),
    [anon_sym_JMPk] = ACTIONS(23),
    [anon_sym_JCNk] = ACTIONS(23),
    [anon_sym_JSRk] = ACTIONS(23),
    [anon_sym_STHk] = ACTIONS(23),
    [anon_sym_LDZk] = ACTIONS(23),
    [anon_sym_STZk] = ACTIONS(23),
    [anon_sym_LDRk] = ACTIONS(23),
    [anon_sym_STRk] = ACTIONS(23),
    [anon_sym_LDAk] = ACTIONS(23),
    [anon_sym_STAk] = ACTIONS(23),
    [anon_sym_DEIk] = ACTIONS(23),
    [anon_sym_DEOk] = ACTIONS(23),
    [anon_sym_ADDk] = ACTIONS(23),
    [anon_sym_SUBk] = ACTIONS(23),
    [anon_sym_MULk] = ACTIONS(23),
    [anon_sym_DIVk] = ACTIONS(23),
    [anon_sym_ANDk] = ACTIONS(23),
    [anon_sym_ORAk] = ACTIONS(23),
    [anon_sym_EORk] = ACTIONS(23),
    [anon_sym_SFTk] = ACTIONS(23),
    [anon_sym_LIT2] = ACTIONS(23),
    [anon_sym_INC2k] = ACTIONS(23),
    [anon_sym_POP2k] = ACTIONS(23),
    [anon_sym_NIP2k] = ACTIONS(23),
    [anon_sym_SWP2k] = ACTIONS(23),
    [anon_sym_ROT2k] = ACTIONS(23),
    [anon_sym_DUP2k] = ACTIONS(23),
    [anon_sym_OVR2k] = ACTIONS(23),
    [anon_sym_EQU2k] = ACTIONS(23),
    [anon_sym_NEQ2k] = ACTIONS(23),
    [anon_sym_GTH2k] = ACTIONS(23),
    [anon_sym_LTH2k] = ACTIONS(23),
    [anon_sym_JMP2k] = ACTIONS(23),
    [anon_sym_JCN2k] = ACTIONS(23),
    [anon_sym_JSR2k] = ACTIONS(23),
    [anon_sym_STH2k] = ACTIONS(23),
    [anon_sym_LDZ2k] = ACTIONS(23),
    [anon_sym_STZ2k] = ACTIONS(23),
    [anon_sym_LDR2k] = ACTIONS(23),
    [anon_sym_STR2k] = ACTIONS(23),
    [anon_sym_LDA2k] = ACTIONS(23),
    [anon_sym_STA2k] = ACTIONS(23),
    [anon_sym_DEI2k] = ACTIONS(23),
    [anon_sym_DEO2k] = ACTIONS(23),
    [anon_sym_ADD2k] = ACTIONS(23),
    [anon_sym_SUB2k] = ACTIONS(23),
    [anon_sym_MUL2k] = ACTIONS(23),
    [anon_sym_DIV2k] = ACTIONS(23),
    [anon_sym_AND2k] = ACTIONS(23),
    [anon_sym_ORA2k] = ACTIONS(23),
    [anon_sym_EOR2k] = ACTIONS(23),
    [anon_sym_SFT2k] = ACTIONS(23),
    [anon_sym_LITr] = ACTIONS(20),
    [anon_sym_INCkr] = ACTIONS(20),
    [anon_sym_POPkr] = ACTIONS(20),
    [anon_sym_NIPkr] = ACTIONS(20),
    [anon_sym_SWPkr] = ACTIONS(20),
    [anon_sym_ROTkr] = ACTIONS(20),
    [anon_sym_DUPkr] = ACTIONS(20),
    [anon_sym_OVRkr] = ACTIONS(20),
    [anon_sym_EQUkr] = ACTIONS(20),
    [anon_sym_NEQkr] = ACTIONS(20),
    [anon_sym_GTHkr] = ACTIONS(20),
    [anon_sym_LTHkr] = ACTIONS(20),
    [anon_sym_JMPkr] = ACTIONS(20),
    [anon_sym_JCNkr] = ACTIONS(20),
    [anon_sym_JSRkr] = ACTIONS(20),
    [anon_sym_STHkr] = ACTIONS(20),
    [anon_sym_LDZkr] = ACTIONS(20),
    [anon_sym_STZkr] = ACTIONS(20),
    [anon_sym_LDRkr] = ACTIONS(20),
    [anon_sym_STRkr] = ACTIONS(20),
    [anon_sym_LDAkr] = ACTIONS(20),
    [anon_sym_STAkr] = ACTIONS(20),
    [anon_sym_DEIkr] = ACTIONS(20),
    [anon_sym_DEOkr] = ACTIONS(20),
    [anon_sym_ADDkr] = ACTIONS(20),
    [anon_sym_SUBkr] = ACTIONS(20),
    [anon_sym_MULkr] = ACTIONS(20),
    [anon_sym_DIVkr] = ACTIONS(20),
    [anon_sym_ANDkr] = ACTIONS(20),
    [anon_sym_ORAkr] = ACTIONS(20),
    [anon_sym_EORkr] = ACTIONS(20),
    [anon_sym_SFTkr] = ACTIONS(20),
    [anon_sym_LIT2r] = ACTIONS(20),
    [anon_sym_INC2kr] = ACTIONS(20),
    [anon_sym_POP2kr] = ACTIONS(20),
    [anon_sym_NIP2kr] = ACTIONS(20),
    [anon_sym_SWP2kr] = ACTIONS(20),
    [anon_sym_ROT2kr] = ACTIONS(20),
    [anon_sym_DUP2kr] = ACTIONS(20),
    [anon_sym_OVR2kr] = ACTIONS(20),
    [anon_sym_EQU2kr] = ACTIONS(20),
    [anon_sym_NEQ2kr] = ACTIONS(20),
    [anon_sym_GTH2kr] = ACTIONS(20),
    [anon_sym_LTH2kr] = ACTIONS(20),
    [anon_sym_JMP2kr] = ACTIONS(20),
    [anon_sym_JCN2kr] = ACTIONS(20),
    [anon_sym_JSR2kr] = ACTIONS(20),
    [anon_sym_STH2kr] = ACTIONS(20),
    [anon_sym_LDZ2kr] = ACTIONS(20),
    [anon_sym_STZ2kr] = ACTIONS(20),
    [anon_sym_LDR2kr] = ACTIONS(20),
    [anon_sym_STR2kr] = ACTIONS(20),
    [anon_sym_LDA2kr] = ACTIONS(20),
    [anon_sym_STA2kr] = ACTIONS(20),
    [anon_sym_DEI2kr] = ACTIONS(20),
    [anon_sym_DEO2kr] = ACTIONS(20),
    [anon_sym_ADD2kr] = ACTIONS(20),
    [anon_sym_SUB2kr] = ACTIONS(20),
    [anon_sym_MUL2kr] = ACTIONS(20),
    [anon_sym_DIV2kr] = ACTIONS(20),
    [anon_sym_AND2kr] = ACTIONS(20),
    [anon_sym_ORA2kr] = ACTIONS(20),
    [anon_sym_EOR2kr] = ACTIONS(20),
    [anon_sym_SFT2kr] = ACTIONS(20),
    [anon_sym_PIPE] = ACTIONS(15),
    [anon_sym_DOLLAR] = ACTIONS(26),
    [anon_sym_POUND] = ACTIONS(29),
    [anon_sym_AT] = ACTIONS(15),
    [anon_sym_COMMA] = ACTIONS(32),
    [anon_sym__] = ACTIONS(32),
    [anon_sym_DOT] = ACTIONS(35),
    [anon_sym_DASH] = ACTIONS(35),
    [anon_sym_SEMI] = ACTIONS(38),
    [anon_sym_EQ] = ACTIONS(38),
    [anon_sym_BANG] = ACTIONS(41),
    [anon_sym_QMARK] = ACTIONS(41),
    [anon_sym_AMP] = ACTIONS(44),
    [anon_sym_LBRACK] = ACTIONS(47),
    [anon_sym_RBRACK] = ACTIONS(15),
    [anon_sym_DQUOTE] = ACTIONS(50),
    [sym_number] = ACTIONS(53),
    [sym_comment] = ACTIONS(3),
  },
  [3] = {
    [sym__non_toplevel_statement] = STATE(6),
    [sym_opcode] = STATE(6),
    [sym_relative_pad_operation] = STATE(6),
    [sym_hex_literal] = STATE(6),
    [sym_sublabel_reference] = STATE(6),
    [sym_rune] = STATE(6),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(6),
    [sym_raw_ascii] = STATE(6),
    [aux_sym_memory_execution_repeat1] = STATE(6),
    [ts_builtin_sym_end] = ACTIONS(56),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_PERCENT] = ACTIONS(56),
    [anon_sym_TILDE] = ACTIONS(56),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_PIPE] = ACTIONS(56),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_AT] = ACTIONS(56),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(82),
    [sym_comment] = ACTIONS(3),
  },
  [4] = {
    [sym__non_toplevel_statement] = STATE(5),
    [sym_opcode] = STATE(5),
    [sym_relative_pad_operation] = STATE(5),
    [sym_hex_literal] = STATE(5),
    [sym_sublabel_reference] = STATE(5),
    [sym_rune] = STATE(5),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(5),
    [sym_raw_ascii] = STATE(5),
    [aux_sym_memory_execution_repeat1] = STATE(5),
    [ts_builtin_sym_end] = ACTIONS(84),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_PERCENT] = ACTIONS(84),
    [anon_sym_TILDE] = ACTIONS(84),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_PIPE] = ACTIONS(84),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_AT] = ACTIONS(84),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(86),
    [sym_comment] = ACTIONS(3),
  },
  [5] = {
    [sym__non_toplevel_statement] = STATE(2),
    [sym_opcode] = STATE(2),
    [sym_relative_pad_operation] = STATE(2),
    [sym_hex_literal] = STATE(2),
    [sym_sublabel_reference] = STATE(2),
    [sym_rune] = STATE(2),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(2),
    [sym_raw_ascii] = STATE(2),
    [aux_sym_memory_execution_repeat1] = STATE(2),
    [ts_builtin_sym_end] = ACTIONS(88),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_PERCENT] = ACTIONS(88),
    [anon_sym_TILDE] = ACTIONS(88),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_PIPE] = ACTIONS(88),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_AT] = ACTIONS(88),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(90),
    [sym_comment] = ACTIONS(3),
  },
  [6] = {
    [sym__non_toplevel_statement] = STATE(2),
    [sym_opcode] = STATE(2),
    [sym_relative_pad_operation] = STATE(2),
    [sym_hex_literal] = STATE(2),
    [sym_sublabel_reference] = STATE(2),
    [sym_rune] = STATE(2),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(2),
    [sym_raw_ascii] = STATE(2),
    [aux_sym_memory_execution_repeat1] = STATE(2),
    [ts_builtin_sym_end] = ACTIONS(92),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_PERCENT] = ACTIONS(92),
    [anon_sym_TILDE] = ACTIONS(92),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_PIPE] = ACTIONS(92),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_AT] = ACTIONS(92),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(90),
    [sym_comment] = ACTIONS(3),
  },
  [7] = {
    [sym__non_toplevel_statement] = STATE(9),
    [sym_opcode] = STATE(9),
    [sym_relative_pad_operation] = STATE(9),
    [sym_hex_literal] = STATE(9),
    [sym_sublabel_reference] = STATE(9),
    [sym_rune] = STATE(9),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(9),
    [sym_raw_ascii] = STATE(9),
    [aux_sym_memory_execution_repeat1] = STATE(9),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_RBRACK] = ACTIONS(94),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(96),
    [sym_comment] = ACTIONS(3),
  },
  [8] = {
    [sym__non_toplevel_statement] = STATE(2),
    [sym_opcode] = STATE(2),
    [sym_relative_pad_operation] = STATE(2),
    [sym_hex_literal] = STATE(2),
    [sym_sublabel_reference] = STATE(2),
    [sym_rune] = STATE(2),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(2),
    [sym_raw_ascii] = STATE(2),
    [aux_sym_memory_execution_repeat1] = STATE(2),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_RBRACE] = ACTIONS(98),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(90),
    [sym_comment] = ACTIONS(3),
  },
  [9] = {
    [sym__non_toplevel_statement] = STATE(2),
    [sym_opcode] = STATE(2),
    [sym_relative_pad_operation] = STATE(2),
    [sym_hex_literal] = STATE(2),
    [sym_sublabel_reference] = STATE(2),
    [sym_rune] = STATE(2),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(2),
    [sym_raw_ascii] = STATE(2),
    [aux_sym_memory_execution_repeat1] = STATE(2),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_RBRACK] = ACTIONS(100),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(90),
    [sym_comment] = ACTIONS(3),
  },
  [10] = {
    [sym__non_toplevel_statement] = STATE(8),
    [sym_opcode] = STATE(8),
    [sym_relative_pad_operation] = STATE(8),
    [sym_hex_literal] = STATE(8),
    [sym_sublabel_reference] = STATE(8),
    [sym_rune] = STATE(8),
    [sym_rune_char] = STATE(25),
    [sym_brackets] = STATE(8),
    [sym_raw_ascii] = STATE(8),
    [aux_sym_memory_execution_repeat1] = STATE(8),
    [sym_identifier] = ACTIONS(58),
    [anon_sym_RBRACE] = ACTIONS(102),
    [anon_sym_BRK] = ACTIONS(60),
    [anon_sym_INC] = ACTIONS(62),
    [anon_sym_POP] = ACTIONS(62),
    [anon_sym_NIP] = ACTIONS(62),
    [anon_sym_SWP] = ACTIONS(62),
    [anon_sym_ROT] = ACTIONS(62),
    [anon_sym_DUP] = ACTIONS(62),
    [anon_sym_OVR] = ACTIONS(62),
    [anon_sym_EQU] = ACTIONS(62),
    [anon_sym_NEQ] = ACTIONS(62),
    [anon_sym_GTH] = ACTIONS(62),
    [anon_sym_LTH] = ACTIONS(62),
    [anon_sym_JMP] = ACTIONS(62),
    [anon_sym_JCN] = ACTIONS(62),
    [anon_sym_JSR] = ACTIONS(62),
    [anon_sym_STH] = ACTIONS(62),
    [anon_sym_LDZ] = ACTIONS(62),
    [anon_sym_STZ] = ACTIONS(62),
    [anon_sym_LDR] = ACTIONS(62),
    [anon_sym_STR] = ACTIONS(62),
    [anon_sym_LDA] = ACTIONS(62),
    [anon_sym_STA] = ACTIONS(62),
    [anon_sym_DEI] = ACTIONS(62),
    [anon_sym_DEO] = ACTIONS(62),
    [anon_sym_ADD] = ACTIONS(62),
    [anon_sym_SUB] = ACTIONS(62),
    [anon_sym_MUL] = ACTIONS(62),
    [anon_sym_DIV] = ACTIONS(62),
    [anon_sym_AND] = ACTIONS(62),
    [anon_sym_ORA] = ACTIONS(62),
    [anon_sym_EOR] = ACTIONS(62),
    [anon_sym_SFT] = ACTIONS(62),
    [anon_sym_JCI] = ACTIONS(60),
    [anon_sym_INC2] = ACTIONS(62),
    [anon_sym_POP2] = ACTIONS(62),
    [anon_sym_NIP2] = ACTIONS(62),
    [anon_sym_SWP2] = ACTIONS(62),
    [anon_sym_ROT2] = ACTIONS(62),
    [anon_sym_DUP2] = ACTIONS(62),
    [anon_sym_OVR2] = ACTIONS(62),
    [anon_sym_EQU2] = ACTIONS(62),
    [anon_sym_NEQ2] = ACTIONS(62),
    [anon_sym_GTH2] = ACTIONS(62),
    [anon_sym_LTH2] = ACTIONS(62),
    [anon_sym_JMP2] = ACTIONS(62),
    [anon_sym_JCN2] = ACTIONS(62),
    [anon_sym_JSR2] = ACTIONS(62),
    [anon_sym_STH2] = ACTIONS(62),
    [anon_sym_LDZ2] = ACTIONS(62),
    [anon_sym_STZ2] = ACTIONS(62),
    [anon_sym_LDR2] = ACTIONS(62),
    [anon_sym_STR2] = ACTIONS(62),
    [anon_sym_LDA2] = ACTIONS(62),
    [anon_sym_STA2] = ACTIONS(62),
    [anon_sym_DEI2] = ACTIONS(62),
    [anon_sym_DEO2] = ACTIONS(62),
    [anon_sym_ADD2] = ACTIONS(62),
    [anon_sym_SUB2] = ACTIONS(62),
    [anon_sym_MUL2] = ACTIONS(62),
    [anon_sym_DIV2] = ACTIONS(62),
    [anon_sym_AND2] = ACTIONS(62),
    [anon_sym_ORA2] = ACTIONS(62),
    [anon_sym_EOR2] = ACTIONS(62),
    [anon_sym_SFT2] = ACTIONS(62),
    [anon_sym_JMI] = ACTIONS(60),
    [anon_sym_INCr] = ACTIONS(60),
    [anon_sym_POPr] = ACTIONS(60),
    [anon_sym_NIPr] = ACTIONS(60),
    [anon_sym_SWPr] = ACTIONS(60),
    [anon_sym_ROTr] = ACTIONS(60),
    [anon_sym_DUPr] = ACTIONS(60),
    [anon_sym_OVRr] = ACTIONS(60),
    [anon_sym_EQUr] = ACTIONS(60),
    [anon_sym_NEQr] = ACTIONS(60),
    [anon_sym_GTHr] = ACTIONS(60),
    [anon_sym_LTHr] = ACTIONS(60),
    [anon_sym_JMPr] = ACTIONS(60),
    [anon_sym_JCNr] = ACTIONS(60),
    [anon_sym_JSRr] = ACTIONS(60),
    [anon_sym_STHr] = ACTIONS(60),
    [anon_sym_LDZr] = ACTIONS(60),
    [anon_sym_STZr] = ACTIONS(60),
    [anon_sym_LDRr] = ACTIONS(60),
    [anon_sym_STRr] = ACTIONS(60),
    [anon_sym_LDAr] = ACTIONS(60),
    [anon_sym_STAr] = ACTIONS(60),
    [anon_sym_DEIr] = ACTIONS(60),
    [anon_sym_DEOr] = ACTIONS(60),
    [anon_sym_ADDr] = ACTIONS(60),
    [anon_sym_SUBr] = ACTIONS(60),
    [anon_sym_MULr] = ACTIONS(60),
    [anon_sym_DIVr] = ACTIONS(60),
    [anon_sym_ANDr] = ACTIONS(60),
    [anon_sym_ORAr] = ACTIONS(60),
    [anon_sym_EORr] = ACTIONS(60),
    [anon_sym_SFTr] = ACTIONS(60),
    [anon_sym_JSI] = ACTIONS(60),
    [anon_sym_INC2r] = ACTIONS(60),
    [anon_sym_POP2r] = ACTIONS(60),
    [anon_sym_NIP2r] = ACTIONS(60),
    [anon_sym_SWP2r] = ACTIONS(60),
    [anon_sym_ROT2r] = ACTIONS(60),
    [anon_sym_DUP2r] = ACTIONS(60),
    [anon_sym_OVR2r] = ACTIONS(60),
    [anon_sym_EQU2r] = ACTIONS(60),
    [anon_sym_NEQ2r] = ACTIONS(60),
    [anon_sym_GTH2r] = ACTIONS(60),
    [anon_sym_LTH2r] = ACTIONS(60),
    [anon_sym_JMP2r] = ACTIONS(60),
    [anon_sym_JCN2r] = ACTIONS(60),
    [anon_sym_JSR2r] = ACTIONS(60),
    [anon_sym_STH2r] = ACTIONS(60),
    [anon_sym_LDZ2r] = ACTIONS(60),
    [anon_sym_STZ2r] = ACTIONS(60),
    [anon_sym_LDR2r] = ACTIONS(60),
    [anon_sym_STR2r] = ACTIONS(60),
    [anon_sym_LDA2r] = ACTIONS(60),
    [anon_sym_STA2r] = ACTIONS(60),
    [anon_sym_DEI2r] = ACTIONS(60),
    [anon_sym_DEO2r] = ACTIONS(60),
    [anon_sym_ADD2r] = ACTIONS(60),
    [anon_sym_SUB2r] = ACTIONS(60),
    [anon_sym_MUL2r] = ACTIONS(60),
    [anon_sym_DIV2r] = ACTIONS(60),
    [anon_sym_AND2r] = ACTIONS(60),
    [anon_sym_ORA2r] = ACTIONS(60),
    [anon_sym_EOR2r] = ACTIONS(60),
    [anon_sym_SFT2r] = ACTIONS(60),
    [anon_sym_LIT] = ACTIONS(62),
    [anon_sym_INCk] = ACTIONS(62),
    [anon_sym_POPk] = ACTIONS(62),
    [anon_sym_NIPk] = ACTIONS(62),
    [anon_sym_SWPk] = ACTIONS(62),
    [anon_sym_ROTk] = ACTIONS(62),
    [anon_sym_DUPk] = ACTIONS(62),
    [anon_sym_OVRk] = ACTIONS(62),
    [anon_sym_EQUk] = ACTIONS(62),
    [anon_sym_NEQk] = ACTIONS(62),
    [anon_sym_GTHk] = ACTIONS(62),
    [anon_sym_LTHk] = ACTIONS(62),
    [anon_sym_JMPk] = ACTIONS(62),
    [anon_sym_JCNk] = ACTIONS(62),
    [anon_sym_JSRk] = ACTIONS(62),
    [anon_sym_STHk] = ACTIONS(62),
    [anon_sym_LDZk] = ACTIONS(62),
    [anon_sym_STZk] = ACTIONS(62),
    [anon_sym_LDRk] = ACTIONS(62),
    [anon_sym_STRk] = ACTIONS(62),
    [anon_sym_LDAk] = ACTIONS(62),
    [anon_sym_STAk] = ACTIONS(62),
    [anon_sym_DEIk] = ACTIONS(62),
    [anon_sym_DEOk] = ACTIONS(62),
    [anon_sym_ADDk] = ACTIONS(62),
    [anon_sym_SUBk] = ACTIONS(62),
    [anon_sym_MULk] = ACTIONS(62),
    [anon_sym_DIVk] = ACTIONS(62),
    [anon_sym_ANDk] = ACTIONS(62),
    [anon_sym_ORAk] = ACTIONS(62),
    [anon_sym_EORk] = ACTIONS(62),
    [anon_sym_SFTk] = ACTIONS(62),
    [anon_sym_LIT2] = ACTIONS(62),
    [anon_sym_INC2k] = ACTIONS(62),
    [anon_sym_POP2k] = ACTIONS(62),
    [anon_sym_NIP2k] = ACTIONS(62),
    [anon_sym_SWP2k] = ACTIONS(62),
    [anon_sym_ROT2k] = ACTIONS(62),
    [anon_sym_DUP2k] = ACTIONS(62),
    [anon_sym_OVR2k] = ACTIONS(62),
    [anon_sym_EQU2k] = ACTIONS(62),
    [anon_sym_NEQ2k] = ACTIONS(62),
    [anon_sym_GTH2k] = ACTIONS(62),
    [anon_sym_LTH2k] = ACTIONS(62),
    [anon_sym_JMP2k] = ACTIONS(62),
    [anon_sym_JCN2k] = ACTIONS(62),
    [anon_sym_JSR2k] = ACTIONS(62),
    [anon_sym_STH2k] = ACTIONS(62),
    [anon_sym_LDZ2k] = ACTIONS(62),
    [anon_sym_STZ2k] = ACTIONS(62),
    [anon_sym_LDR2k] = ACTIONS(62),
    [anon_sym_STR2k] = ACTIONS(62),
    [anon_sym_LDA2k] = ACTIONS(62),
    [anon_sym_STA2k] = ACTIONS(62),
    [anon_sym_DEI2k] = ACTIONS(62),
    [anon_sym_DEO2k] = ACTIONS(62),
    [anon_sym_ADD2k] = ACTIONS(62),
    [anon_sym_SUB2k] = ACTIONS(62),
    [anon_sym_MUL2k] = ACTIONS(62),
    [anon_sym_DIV2k] = ACTIONS(62),
    [anon_sym_AND2k] = ACTIONS(62),
    [anon_sym_ORA2k] = ACTIONS(62),
    [anon_sym_EOR2k] = ACTIONS(62),
    [anon_sym_SFT2k] = ACTIONS(62),
    [anon_sym_LITr] = ACTIONS(60),
    [anon_sym_INCkr] = ACTIONS(60),
    [anon_sym_POPkr] = ACTIONS(60),
    [anon_sym_NIPkr] = ACTIONS(60),
    [anon_sym_SWPkr] = ACTIONS(60),
    [anon_sym_ROTkr] = ACTIONS(60),
    [anon_sym_DUPkr] = ACTIONS(60),
    [anon_sym_OVRkr] = ACTIONS(60),
    [anon_sym_EQUkr] = ACTIONS(60),
    [anon_sym_NEQkr] = ACTIONS(60),
    [anon_sym_GTHkr] = ACTIONS(60),
    [anon_sym_LTHkr] = ACTIONS(60),
    [anon_sym_JMPkr] = ACTIONS(60),
    [anon_sym_JCNkr] = ACTIONS(60),
    [anon_sym_JSRkr] = ACTIONS(60),
    [anon_sym_STHkr] = ACTIONS(60),
    [anon_sym_LDZkr] = ACTIONS(60),
    [anon_sym_STZkr] = ACTIONS(60),
    [anon_sym_LDRkr] = ACTIONS(60),
    [anon_sym_STRkr] = ACTIONS(60),
    [anon_sym_LDAkr] = ACTIONS(60),
    [anon_sym_STAkr] = ACTIONS(60),
    [anon_sym_DEIkr] = ACTIONS(60),
    [anon_sym_DEOkr] = ACTIONS(60),
    [anon_sym_ADDkr] = ACTIONS(60),
    [anon_sym_SUBkr] = ACTIONS(60),
    [anon_sym_MULkr] = ACTIONS(60),
    [anon_sym_DIVkr] = ACTIONS(60),
    [anon_sym_ANDkr] = ACTIONS(60),
    [anon_sym_ORAkr] = ACTIONS(60),
    [anon_sym_EORkr] = ACTIONS(60),
    [anon_sym_SFTkr] = ACTIONS(60),
    [anon_sym_LIT2r] = ACTIONS(60),
    [anon_sym_INC2kr] = ACTIONS(60),
    [anon_sym_POP2kr] = ACTIONS(60),
    [anon_sym_NIP2kr] = ACTIONS(60),
    [anon_sym_SWP2kr] = ACTIONS(60),
    [anon_sym_ROT2kr] = ACTIONS(60),
    [anon_sym_DUP2kr] = ACTIONS(60),
    [anon_sym_OVR2kr] = ACTIONS(60),
    [anon_sym_EQU2kr] = ACTIONS(60),
    [anon_sym_NEQ2kr] = ACTIONS(60),
    [anon_sym_GTH2kr] = ACTIONS(60),
    [anon_sym_LTH2kr] = ACTIONS(60),
    [anon_sym_JMP2kr] = ACTIONS(60),
    [anon_sym_JCN2kr] = ACTIONS(60),
    [anon_sym_JSR2kr] = ACTIONS(60),
    [anon_sym_STH2kr] = ACTIONS(60),
    [anon_sym_LDZ2kr] = ACTIONS(60),
    [anon_sym_STZ2kr] = ACTIONS(60),
    [anon_sym_LDR2kr] = ACTIONS(60),
    [anon_sym_STR2kr] = ACTIONS(60),
    [anon_sym_LDA2kr] = ACTIONS(60),
    [anon_sym_STA2kr] = ACTIONS(60),
    [anon_sym_DEI2kr] = ACTIONS(60),
    [anon_sym_DEO2kr] = ACTIONS(60),
    [anon_sym_ADD2kr] = ACTIONS(60),
    [anon_sym_SUB2kr] = ACTIONS(60),
    [anon_sym_MUL2kr] = ACTIONS(60),
    [anon_sym_DIV2kr] = ACTIONS(60),
    [anon_sym_AND2kr] = ACTIONS(60),
    [anon_sym_ORA2kr] = ACTIONS(60),
    [anon_sym_EOR2kr] = ACTIONS(60),
    [anon_sym_SFT2kr] = ACTIONS(60),
    [anon_sym_DOLLAR] = ACTIONS(64),
    [anon_sym_POUND] = ACTIONS(66),
    [anon_sym_COMMA] = ACTIONS(68),
    [anon_sym__] = ACTIONS(68),
    [anon_sym_DOT] = ACTIONS(70),
    [anon_sym_DASH] = ACTIONS(70),
    [anon_sym_SEMI] = ACTIONS(72),
    [anon_sym_EQ] = ACTIONS(72),
    [anon_sym_BANG] = ACTIONS(74),
    [anon_sym_QMARK] = ACTIONS(74),
    [anon_sym_AMP] = ACTIONS(76),
    [anon_sym_LBRACK] = ACTIONS(78),
    [anon_sym_DQUOTE] = ACTIONS(80),
    [sym_number] = ACTIONS(104),
    [sym_comment] = ACTIONS(3),
  },
  [11] = {
    [ts_builtin_sym_end] = ACTIONS(106),
    [sym_identifier] = ACTIONS(108),
    [anon_sym_PERCENT] = ACTIONS(106),
    [anon_sym_RBRACE] = ACTIONS(106),
    [anon_sym_TILDE] = ACTIONS(106),
    [anon_sym_BRK] = ACTIONS(106),
    [anon_sym_INC] = ACTIONS(108),
    [anon_sym_POP] = ACTIONS(108),
    [anon_sym_NIP] = ACTIONS(108),
    [anon_sym_SWP] = ACTIONS(108),
    [anon_sym_ROT] = ACTIONS(108),
    [anon_sym_DUP] = ACTIONS(108),
    [anon_sym_OVR] = ACTIONS(108),
    [anon_sym_EQU] = ACTIONS(108),
    [anon_sym_NEQ] = ACTIONS(108),
    [anon_sym_GTH] = ACTIONS(108),
    [anon_sym_LTH] = ACTIONS(108),
    [anon_sym_JMP] = ACTIONS(108),
    [anon_sym_JCN] = ACTIONS(108),
    [anon_sym_JSR] = ACTIONS(108),
    [anon_sym_STH] = ACTIONS(108),
    [anon_sym_LDZ] = ACTIONS(108),
    [anon_sym_STZ] = ACTIONS(108),
    [anon_sym_LDR] = ACTIONS(108),
    [anon_sym_STR] = ACTIONS(108),
    [anon_sym_LDA] = ACTIONS(108),
    [anon_sym_STA] = ACTIONS(108),
    [anon_sym_DEI] = ACTIONS(108),
    [anon_sym_DEO] = ACTIONS(108),
    [anon_sym_ADD] = ACTIONS(108),
    [anon_sym_SUB] = ACTIONS(108),
    [anon_sym_MUL] = ACTIONS(108),
    [anon_sym_DIV] = ACTIONS(108),
    [anon_sym_AND] = ACTIONS(108),
    [anon_sym_ORA] = ACTIONS(108),
    [anon_sym_EOR] = ACTIONS(108),
    [anon_sym_SFT] = ACTIONS(108),
    [anon_sym_JCI] = ACTIONS(106),
    [anon_sym_INC2] = ACTIONS(108),
    [anon_sym_POP2] = ACTIONS(108),
    [anon_sym_NIP2] = ACTIONS(108),
    [anon_sym_SWP2] = ACTIONS(108),
    [anon_sym_ROT2] = ACTIONS(108),
    [anon_sym_DUP2] = ACTIONS(108),
    [anon_sym_OVR2] = ACTIONS(108),
    [anon_sym_EQU2] = ACTIONS(108),
    [anon_sym_NEQ2] = ACTIONS(108),
    [anon_sym_GTH2] = ACTIONS(108),
    [anon_sym_LTH2] = ACTIONS(108),
    [anon_sym_JMP2] = ACTIONS(108),
    [anon_sym_JCN2] = ACTIONS(108),
    [anon_sym_JSR2] = ACTIONS(108),
    [anon_sym_STH2] = ACTIONS(108),
    [anon_sym_LDZ2] = ACTIONS(108),
    [anon_sym_STZ2] = ACTIONS(108),
    [anon_sym_LDR2] = ACTIONS(108),
    [anon_sym_STR2] = ACTIONS(108),
    [anon_sym_LDA2] = ACTIONS(108),
    [anon_sym_STA2] = ACTIONS(108),
    [anon_sym_DEI2] = ACTIONS(108),
    [anon_sym_DEO2] = ACTIONS(108),
    [anon_sym_ADD2] = ACTIONS(108),
    [anon_sym_SUB2] = ACTIONS(108),
    [anon_sym_MUL2] = ACTIONS(108),
    [anon_sym_DIV2] = ACTIONS(108),
    [anon_sym_AND2] = ACTIONS(108),
    [anon_sym_ORA2] = ACTIONS(108),
    [anon_sym_EOR2] = ACTIONS(108),
    [anon_sym_SFT2] = ACTIONS(108),
    [anon_sym_JMI] = ACTIONS(106),
    [anon_sym_INCr] = ACTIONS(106),
    [anon_sym_POPr] = ACTIONS(106),
    [anon_sym_NIPr] = ACTIONS(106),
    [anon_sym_SWPr] = ACTIONS(106),
    [anon_sym_ROTr] = ACTIONS(106),
    [anon_sym_DUPr] = ACTIONS(106),
    [anon_sym_OVRr] = ACTIONS(106),
    [anon_sym_EQUr] = ACTIONS(106),
    [anon_sym_NEQr] = ACTIONS(106),
    [anon_sym_GTHr] = ACTIONS(106),
    [anon_sym_LTHr] = ACTIONS(106),
    [anon_sym_JMPr] = ACTIONS(106),
    [anon_sym_JCNr] = ACTIONS(106),
    [anon_sym_JSRr] = ACTIONS(106),
    [anon_sym_STHr] = ACTIONS(106),
    [anon_sym_LDZr] = ACTIONS(106),
    [anon_sym_STZr] = ACTIONS(106),
    [anon_sym_LDRr] = ACTIONS(106),
    [anon_sym_STRr] = ACTIONS(106),
    [anon_sym_LDAr] = ACTIONS(106),
    [anon_sym_STAr] = ACTIONS(106),
    [anon_sym_DEIr] = ACTIONS(106),
    [anon_sym_DEOr] = ACTIONS(106),
    [anon_sym_ADDr] = ACTIONS(106),
    [anon_sym_SUBr] = ACTIONS(106),
    [anon_sym_MULr] = ACTIONS(106),
    [anon_sym_DIVr] = ACTIONS(106),
    [anon_sym_ANDr] = ACTIONS(106),
    [anon_sym_ORAr] = ACTIONS(106),
    [anon_sym_EORr] = ACTIONS(106),
    [anon_sym_SFTr] = ACTIONS(106),
    [anon_sym_JSI] = ACTIONS(106),
    [anon_sym_INC2r] = ACTIONS(106),
    [anon_sym_POP2r] = ACTIONS(106),
    [anon_sym_NIP2r] = ACTIONS(106),
    [anon_sym_SWP2r] = ACTIONS(106),
    [anon_sym_ROT2r] = ACTIONS(106),
    [anon_sym_DUP2r] = ACTIONS(106),
    [anon_sym_OVR2r] = ACTIONS(106),
    [anon_sym_EQU2r] = ACTIONS(106),
    [anon_sym_NEQ2r] = ACTIONS(106),
    [anon_sym_GTH2r] = ACTIONS(106),
    [anon_sym_LTH2r] = ACTIONS(106),
    [anon_sym_JMP2r] = ACTIONS(106),
    [anon_sym_JCN2r] = ACTIONS(106),
    [anon_sym_JSR2r] = ACTIONS(106),
    [anon_sym_STH2r] = ACTIONS(106),
    [anon_sym_LDZ2r] = ACTIONS(106),
    [anon_sym_STZ2r] = ACTIONS(106),
    [anon_sym_LDR2r] = ACTIONS(106),
    [anon_sym_STR2r] = ACTIONS(106),
    [anon_sym_LDA2r] = ACTIONS(106),
    [anon_sym_STA2r] = ACTIONS(106),
    [anon_sym_DEI2r] = ACTIONS(106),
    [anon_sym_DEO2r] = ACTIONS(106),
    [anon_sym_ADD2r] = ACTIONS(106),
    [anon_sym_SUB2r] = ACTIONS(106),
    [anon_sym_MUL2r] = ACTIONS(106),
    [anon_sym_DIV2r] = ACTIONS(106),
    [anon_sym_AND2r] = ACTIONS(106),
    [anon_sym_ORA2r] = ACTIONS(106),
    [anon_sym_EOR2r] = ACTIONS(106),
    [anon_sym_SFT2r] = ACTIONS(106),
    [anon_sym_LIT] = ACTIONS(108),
    [anon_sym_INCk] = ACTIONS(108),
    [anon_sym_POPk] = ACTIONS(108),
    [anon_sym_NIPk] = ACTIONS(108),
    [anon_sym_SWPk] = ACTIONS(108),
    [anon_sym_ROTk] = ACTIONS(108),
    [anon_sym_DUPk] = ACTIONS(108),
    [anon_sym_OVRk] = ACTIONS(108),
    [anon_sym_EQUk] = ACTIONS(108),
    [anon_sym_NEQk] = ACTIONS(108),
    [anon_sym_GTHk] = ACTIONS(108),
    [anon_sym_LTHk] = ACTIONS(108),
    [anon_sym_JMPk] = ACTIONS(108),
    [anon_sym_JCNk] = ACTIONS(108),
    [anon_sym_JSRk] = ACTIONS(108),
    [anon_sym_STHk] = ACTIONS(108),
    [anon_sym_LDZk] = ACTIONS(108),
    [anon_sym_STZk] = ACTIONS(108),
    [anon_sym_LDRk] = ACTIONS(108),
    [anon_sym_STRk] = ACTIONS(108),
    [anon_sym_LDAk] = ACTIONS(108),
    [anon_sym_STAk] = ACTIONS(108),
    [anon_sym_DEIk] = ACTIONS(108),
    [anon_sym_DEOk] = ACTIONS(108),
    [anon_sym_ADDk] = ACTIONS(108),
    [anon_sym_SUBk] = ACTIONS(108),
    [anon_sym_MULk] = ACTIONS(108),
    [anon_sym_DIVk] = ACTIONS(108),
    [anon_sym_ANDk] = ACTIONS(108),
    [anon_sym_ORAk] = ACTIONS(108),
    [anon_sym_EORk] = ACTIONS(108),
    [anon_sym_SFTk] = ACTIONS(108),
    [anon_sym_LIT2] = ACTIONS(108),
    [anon_sym_INC2k] = ACTIONS(108),
    [anon_sym_POP2k] = ACTIONS(108),
    [anon_sym_NIP2k] = ACTIONS(108),
    [anon_sym_SWP2k] = ACTIONS(108),
    [anon_sym_ROT2k] = ACTIONS(108),
    [anon_sym_DUP2k] = ACTIONS(108),
    [anon_sym_OVR2k] = ACTIONS(108),
    [anon_sym_EQU2k] = ACTIONS(108),
    [anon_sym_NEQ2k] = ACTIONS(108),
    [anon_sym_GTH2k] = ACTIONS(108),
    [anon_sym_LTH2k] = ACTIONS(108),
    [anon_sym_JMP2k] = ACTIONS(108),
    [anon_sym_JCN2k] = ACTIONS(108),
    [anon_sym_JSR2k] = ACTIONS(108),
    [anon_sym_STH2k] = ACTIONS(108),
    [anon_sym_LDZ2k] = ACTIONS(108),
    [anon_sym_STZ2k] = ACTIONS(108),
    [anon_sym_LDR2k] = ACTIONS(108),
    [anon_sym_STR2k] = ACTIONS(108),
    [anon_sym_LDA2k] = ACTIONS(108),
    [anon_sym_STA2k] = ACTIONS(108),
    [anon_sym_DEI2k] = ACTIONS(108),
    [anon_sym_DEO2k] = ACTIONS(108),
    [anon_sym_ADD2k] = ACTIONS(108),
    [anon_sym_SUB2k] = ACTIONS(108),
    [anon_sym_MUL2k] = ACTIONS(108),
    [anon_sym_DIV2k] = ACTIONS(108),
    [anon_sym_AND2k] = ACTIONS(108),
    [anon_sym_ORA2k] = ACTIONS(108),
    [anon_sym_EOR2k] = ACTIONS(108),
    [anon_sym_SFT2k] = ACTIONS(108),
    [anon_sym_LITr] = ACTIONS(106),
    [anon_sym_INCkr] = ACTIONS(106),
    [anon_sym_POPkr] = ACTIONS(106),
    [anon_sym_NIPkr] = ACTIONS(106),
    [anon_sym_SWPkr] = ACTIONS(106),
    [anon_sym_ROTkr] = ACTIONS(106),
    [anon_sym_DUPkr] = ACTIONS(106),
    [anon_sym_OVRkr] = ACTIONS(106),
    [anon_sym_EQUkr] = ACTIONS(106),
    [anon_sym_NEQkr] = ACTIONS(106),
    [anon_sym_GTHkr] = ACTIONS(106),
    [anon_sym_LTHkr] = ACTIONS(106),
    [anon_sym_JMPkr] = ACTIONS(106),
    [anon_sym_JCNkr] = ACTIONS(106),
    [anon_sym_JSRkr] = ACTIONS(106),
    [anon_sym_STHkr] = ACTIONS(106),
    [anon_sym_LDZkr] = ACTIONS(106),
    [anon_sym_STZkr] = ACTIONS(106),
    [anon_sym_LDRkr] = ACTIONS(106),
    [anon_sym_STRkr] = ACTIONS(106),
    [anon_sym_LDAkr] = ACTIONS(106),
    [anon_sym_STAkr] = ACTIONS(106),
    [anon_sym_DEIkr] = ACTIONS(106),
    [anon_sym_DEOkr] = ACTIONS(106),
    [anon_sym_ADDkr] = ACTIONS(106),
    [anon_sym_SUBkr] = ACTIONS(106),
    [anon_sym_MULkr] = ACTIONS(106),
    [anon_sym_DIVkr] = ACTIONS(106),
    [anon_sym_ANDkr] = ACTIONS(106),
    [anon_sym_ORAkr] = ACTIONS(106),
    [anon_sym_EORkr] = ACTIONS(106),
    [anon_sym_SFTkr] = ACTIONS(106),
    [anon_sym_LIT2r] = ACTIONS(106),
    [anon_sym_INC2kr] = ACTIONS(106),
    [anon_sym_POP2kr] = ACTIONS(106),
    [anon_sym_NIP2kr] = ACTIONS(106),
    [anon_sym_SWP2kr] = ACTIONS(106),
    [anon_sym_ROT2kr] = ACTIONS(106),
    [anon_sym_DUP2kr] = ACTIONS(106),
    [anon_sym_OVR2kr] = ACTIONS(106),
    [anon_sym_EQU2kr] = ACTIONS(106),
    [anon_sym_NEQ2kr] = ACTIONS(106),
    [anon_sym_GTH2kr] = ACTIONS(106),
    [anon_sym_LTH2kr] = ACTIONS(106),
    [anon_sym_JMP2kr] = ACTIONS(106),
    [anon_sym_JCN2kr] = ACTIONS(106),
    [anon_sym_JSR2kr] = ACTIONS(106),
    [anon_sym_STH2kr] = ACTIONS(106),
    [anon_sym_LDZ2kr] = ACTIONS(106),
    [anon_sym_STZ2kr] = ACTIONS(106),
    [anon_sym_LDR2kr] = ACTIONS(106),
    [anon_sym_STR2kr] = ACTIONS(106),
    [anon_sym_LDA2kr] = ACTIONS(106),
    [anon_sym_STA2kr] = ACTIONS(106),
    [anon_sym_DEI2kr] = ACTIONS(106),
    [anon_sym_DEO2kr] = ACTIONS(106),
    [anon_sym_ADD2kr] = ACTIONS(106),
    [anon_sym_SUB2kr] = ACTIONS(106),
    [anon_sym_MUL2kr] = ACTIONS(106),
    [anon_sym_DIV2kr] = ACTIONS(106),
    [anon_sym_AND2kr] = ACTIONS(106),
    [anon_sym_ORA2kr] = ACTIONS(106),
    [anon_sym_EOR2kr] = ACTIONS(106),
    [anon_sym_SFT2kr] = ACTIONS(106),
    [anon_sym_PIPE] = ACTIONS(106),
    [anon_sym_DOLLAR] = ACTIONS(106),
    [anon_sym_POUND] = ACTIONS(106),
    [anon_sym_AT] = ACTIONS(106),
    [anon_sym_SLASH] = ACTIONS(110),
    [anon_sym_COMMA] = ACTIONS(106),
    [anon_sym__] = ACTIONS(106),
    [anon_sym_DOT] = ACTIONS(106),
    [anon_sym_DASH] = ACTIONS(106),
    [anon_sym_SEMI] = ACTIONS(106),
    [anon_sym_EQ] = ACTIONS(106),
    [anon_sym_BANG] = ACTIONS(106),
    [anon_sym_QMARK] = ACTIONS(106),
    [anon_sym_AMP] = ACTIONS(106),
    [anon_sym_LBRACK] = ACTIONS(106),
    [anon_sym_RBRACK] = ACTIONS(106),
    [anon_sym_DQUOTE] = ACTIONS(106),
    [sym_number] = ACTIONS(106),
    [sym_comment] = ACTIONS(3),
  },
  [12] = {
    [ts_builtin_sym_end] = ACTIONS(112),
    [sym_identifier] = ACTIONS(114),
    [anon_sym_PERCENT] = ACTIONS(112),
    [anon_sym_RBRACE] = ACTIONS(112),
    [anon_sym_TILDE] = ACTIONS(112),
    [anon_sym_BRK] = ACTIONS(112),
    [anon_sym_INC] = ACTIONS(114),
    [anon_sym_POP] = ACTIONS(114),
    [anon_sym_NIP] = ACTIONS(114),
    [anon_sym_SWP] = ACTIONS(114),
    [anon_sym_ROT] = ACTIONS(114),
    [anon_sym_DUP] = ACTIONS(114),
    [anon_sym_OVR] = ACTIONS(114),
    [anon_sym_EQU] = ACTIONS(114),
    [anon_sym_NEQ] = ACTIONS(114),
    [anon_sym_GTH] = ACTIONS(114),
    [anon_sym_LTH] = ACTIONS(114),
    [anon_sym_JMP] = ACTIONS(114),
    [anon_sym_JCN] = ACTIONS(114),
    [anon_sym_JSR] = ACTIONS(114),
    [anon_sym_STH] = ACTIONS(114),
    [anon_sym_LDZ] = ACTIONS(114),
    [anon_sym_STZ] = ACTIONS(114),
    [anon_sym_LDR] = ACTIONS(114),
    [anon_sym_STR] = ACTIONS(114),
    [anon_sym_LDA] = ACTIONS(114),
    [anon_sym_STA] = ACTIONS(114),
    [anon_sym_DEI] = ACTIONS(114),
    [anon_sym_DEO] = ACTIONS(114),
    [anon_sym_ADD] = ACTIONS(114),
    [anon_sym_SUB] = ACTIONS(114),
    [anon_sym_MUL] = ACTIONS(114),
    [anon_sym_DIV] = ACTIONS(114),
    [anon_sym_AND] = ACTIONS(114),
    [anon_sym_ORA] = ACTIONS(114),
    [anon_sym_EOR] = ACTIONS(114),
    [anon_sym_SFT] = ACTIONS(114),
    [anon_sym_JCI] = ACTIONS(112),
    [anon_sym_INC2] = ACTIONS(114),
    [anon_sym_POP2] = ACTIONS(114),
    [anon_sym_NIP2] = ACTIONS(114),
    [anon_sym_SWP2] = ACTIONS(114),
    [anon_sym_ROT2] = ACTIONS(114),
    [anon_sym_DUP2] = ACTIONS(114),
    [anon_sym_OVR2] = ACTIONS(114),
    [anon_sym_EQU2] = ACTIONS(114),
    [anon_sym_NEQ2] = ACTIONS(114),
    [anon_sym_GTH2] = ACTIONS(114),
    [anon_sym_LTH2] = ACTIONS(114),
    [anon_sym_JMP2] = ACTIONS(114),
    [anon_sym_JCN2] = ACTIONS(114),
    [anon_sym_JSR2] = ACTIONS(114),
    [anon_sym_STH2] = ACTIONS(114),
    [anon_sym_LDZ2] = ACTIONS(114),
    [anon_sym_STZ2] = ACTIONS(114),
    [anon_sym_LDR2] = ACTIONS(114),
    [anon_sym_STR2] = ACTIONS(114),
    [anon_sym_LDA2] = ACTIONS(114),
    [anon_sym_STA2] = ACTIONS(114),
    [anon_sym_DEI2] = ACTIONS(114),
    [anon_sym_DEO2] = ACTIONS(114),
    [anon_sym_ADD2] = ACTIONS(114),
    [anon_sym_SUB2] = ACTIONS(114),
    [anon_sym_MUL2] = ACTIONS(114),
    [anon_sym_DIV2] = ACTIONS(114),
    [anon_sym_AND2] = ACTIONS(114),
    [anon_sym_ORA2] = ACTIONS(114),
    [anon_sym_EOR2] = ACTIONS(114),
    [anon_sym_SFT2] = ACTIONS(114),
    [anon_sym_JMI] = ACTIONS(112),
    [anon_sym_INCr] = ACTIONS(112),
    [anon_sym_POPr] = ACTIONS(112),
    [anon_sym_NIPr] = ACTIONS(112),
    [anon_sym_SWPr] = ACTIONS(112),
    [anon_sym_ROTr] = ACTIONS(112),
    [anon_sym_DUPr] = ACTIONS(112),
    [anon_sym_OVRr] = ACTIONS(112),
    [anon_sym_EQUr] = ACTIONS(112),
    [anon_sym_NEQr] = ACTIONS(112),
    [anon_sym_GTHr] = ACTIONS(112),
    [anon_sym_LTHr] = ACTIONS(112),
    [anon_sym_JMPr] = ACTIONS(112),
    [anon_sym_JCNr] = ACTIONS(112),
    [anon_sym_JSRr] = ACTIONS(112),
    [anon_sym_STHr] = ACTIONS(112),
    [anon_sym_LDZr] = ACTIONS(112),
    [anon_sym_STZr] = ACTIONS(112),
    [anon_sym_LDRr] = ACTIONS(112),
    [anon_sym_STRr] = ACTIONS(112),
    [anon_sym_LDAr] = ACTIONS(112),
    [anon_sym_STAr] = ACTIONS(112),
    [anon_sym_DEIr] = ACTIONS(112),
    [anon_sym_DEOr] = ACTIONS(112),
    [anon_sym_ADDr] = ACTIONS(112),
    [anon_sym_SUBr] = ACTIONS(112),
    [anon_sym_MULr] = ACTIONS(112),
    [anon_sym_DIVr] = ACTIONS(112),
    [anon_sym_ANDr] = ACTIONS(112),
    [anon_sym_ORAr] = ACTIONS(112),
    [anon_sym_EORr] = ACTIONS(112),
    [anon_sym_SFTr] = ACTIONS(112),
    [anon_sym_JSI] = ACTIONS(112),
    [anon_sym_INC2r] = ACTIONS(112),
    [anon_sym_POP2r] = ACTIONS(112),
    [anon_sym_NIP2r] = ACTIONS(112),
    [anon_sym_SWP2r] = ACTIONS(112),
    [anon_sym_ROT2r] = ACTIONS(112),
    [anon_sym_DUP2r] = ACTIONS(112),
    [anon_sym_OVR2r] = ACTIONS(112),
    [anon_sym_EQU2r] = ACTIONS(112),
    [anon_sym_NEQ2r] = ACTIONS(112),
    [anon_sym_GTH2r] = ACTIONS(112),
    [anon_sym_LTH2r] = ACTIONS(112),
    [anon_sym_JMP2r] = ACTIONS(112),
    [anon_sym_JCN2r] = ACTIONS(112),
    [anon_sym_JSR2r] = ACTIONS(112),
    [anon_sym_STH2r] = ACTIONS(112),
    [anon_sym_LDZ2r] = ACTIONS(112),
    [anon_sym_STZ2r] = ACTIONS(112),
    [anon_sym_LDR2r] = ACTIONS(112),
    [anon_sym_STR2r] = ACTIONS(112),
    [anon_sym_LDA2r] = ACTIONS(112),
    [anon_sym_STA2r] = ACTIONS(112),
    [anon_sym_DEI2r] = ACTIONS(112),
    [anon_sym_DEO2r] = ACTIONS(112),
    [anon_sym_ADD2r] = ACTIONS(112),
    [anon_sym_SUB2r] = ACTIONS(112),
    [anon_sym_MUL2r] = ACTIONS(112),
    [anon_sym_DIV2r] = ACTIONS(112),
    [anon_sym_AND2r] = ACTIONS(112),
    [anon_sym_ORA2r] = ACTIONS(112),
    [anon_sym_EOR2r] = ACTIONS(112),
    [anon_sym_SFT2r] = ACTIONS(112),
    [anon_sym_LIT] = ACTIONS(114),
    [anon_sym_INCk] = ACTIONS(114),
    [anon_sym_POPk] = ACTIONS(114),
    [anon_sym_NIPk] = ACTIONS(114),
    [anon_sym_SWPk] = ACTIONS(114),
    [anon_sym_ROTk] = ACTIONS(114),
    [anon_sym_DUPk] = ACTIONS(114),
    [anon_sym_OVRk] = ACTIONS(114),
    [anon_sym_EQUk] = ACTIONS(114),
    [anon_sym_NEQk] = ACTIONS(114),
    [anon_sym_GTHk] = ACTIONS(114),
    [anon_sym_LTHk] = ACTIONS(114),
    [anon_sym_JMPk] = ACTIONS(114),
    [anon_sym_JCNk] = ACTIONS(114),
    [anon_sym_JSRk] = ACTIONS(114),
    [anon_sym_STHk] = ACTIONS(114),
    [anon_sym_LDZk] = ACTIONS(114),
    [anon_sym_STZk] = ACTIONS(114),
    [anon_sym_LDRk] = ACTIONS(114),
    [anon_sym_STRk] = ACTIONS(114),
    [anon_sym_LDAk] = ACTIONS(114),
    [anon_sym_STAk] = ACTIONS(114),
    [anon_sym_DEIk] = ACTIONS(114),
    [anon_sym_DEOk] = ACTIONS(114),
    [anon_sym_ADDk] = ACTIONS(114),
    [anon_sym_SUBk] = ACTIONS(114),
    [anon_sym_MULk] = ACTIONS(114),
    [anon_sym_DIVk] = ACTIONS(114),
    [anon_sym_ANDk] = ACTIONS(114),
    [anon_sym_ORAk] = ACTIONS(114),
    [anon_sym_EORk] = ACTIONS(114),
    [anon_sym_SFTk] = ACTIONS(114),
    [anon_sym_LIT2] = ACTIONS(114),
    [anon_sym_INC2k] = ACTIONS(114),
    [anon_sym_POP2k] = ACTIONS(114),
    [anon_sym_NIP2k] = ACTIONS(114),
    [anon_sym_SWP2k] = ACTIONS(114),
    [anon_sym_ROT2k] = ACTIONS(114),
    [anon_sym_DUP2k] = ACTIONS(114),
    [anon_sym_OVR2k] = ACTIONS(114),
    [anon_sym_EQU2k] = ACTIONS(114),
    [anon_sym_NEQ2k] = ACTIONS(114),
    [anon_sym_GTH2k] = ACTIONS(114),
    [anon_sym_LTH2k] = ACTIONS(114),
    [anon_sym_JMP2k] = ACTIONS(114),
    [anon_sym_JCN2k] = ACTIONS(114),
    [anon_sym_JSR2k] = ACTIONS(114),
    [anon_sym_STH2k] = ACTIONS(114),
    [anon_sym_LDZ2k] = ACTIONS(114),
    [anon_sym_STZ2k] = ACTIONS(114),
    [anon_sym_LDR2k] = ACTIONS(114),
    [anon_sym_STR2k] = ACTIONS(114),
    [anon_sym_LDA2k] = ACTIONS(114),
    [anon_sym_STA2k] = ACTIONS(114),
    [anon_sym_DEI2k] = ACTIONS(114),
    [anon_sym_DEO2k] = ACTIONS(114),
    [anon_sym_ADD2k] = ACTIONS(114),
    [anon_sym_SUB2k] = ACTIONS(114),
    [anon_sym_MUL2k] = ACTIONS(114),
    [anon_sym_DIV2k] = ACTIONS(114),
    [anon_sym_AND2k] = ACTIONS(114),
    [anon_sym_ORA2k] = ACTIONS(114),
    [anon_sym_EOR2k] = ACTIONS(114),
    [anon_sym_SFT2k] = ACTIONS(114),
    [anon_sym_LITr] = ACTIONS(112),
    [anon_sym_INCkr] = ACTIONS(112),
    [anon_sym_POPkr] = ACTIONS(112),
    [anon_sym_NIPkr] = ACTIONS(112),
    [anon_sym_SWPkr] = ACTIONS(112),
    [anon_sym_ROTkr] = ACTIONS(112),
    [anon_sym_DUPkr] = ACTIONS(112),
    [anon_sym_OVRkr] = ACTIONS(112),
    [anon_sym_EQUkr] = ACTIONS(112),
    [anon_sym_NEQkr] = ACTIONS(112),
    [anon_sym_GTHkr] = ACTIONS(112),
    [anon_sym_LTHkr] = ACTIONS(112),
    [anon_sym_JMPkr] = ACTIONS(112),
    [anon_sym_JCNkr] = ACTIONS(112),
    [anon_sym_JSRkr] = ACTIONS(112),
    [anon_sym_STHkr] = ACTIONS(112),
    [anon_sym_LDZkr] = ACTIONS(112),
    [anon_sym_STZkr] = ACTIONS(112),
    [anon_sym_LDRkr] = ACTIONS(112),
    [anon_sym_STRkr] = ACTIONS(112),
    [anon_sym_LDAkr] = ACTIONS(112),
    [anon_sym_STAkr] = ACTIONS(112),
    [anon_sym_DEIkr] = ACTIONS(112),
    [anon_sym_DEOkr] = ACTIONS(112),
    [anon_sym_ADDkr] = ACTIONS(112),
    [anon_sym_SUBkr] = ACTIONS(112),
    [anon_sym_MULkr] = ACTIONS(112),
    [anon_sym_DIVkr] = ACTIONS(112),
    [anon_sym_ANDkr] = ACTIONS(112),
    [anon_sym_ORAkr] = ACTIONS(112),
    [anon_sym_EORkr] = ACTIONS(112),
    [anon_sym_SFTkr] = ACTIONS(112),
    [anon_sym_LIT2r] = ACTIONS(112),
    [anon_sym_INC2kr] = ACTIONS(112),
    [anon_sym_POP2kr] = ACTIONS(112),
    [anon_sym_NIP2kr] = ACTIONS(112),
    [anon_sym_SWP2kr] = ACTIONS(112),
    [anon_sym_ROT2kr] = ACTIONS(112),
    [anon_sym_DUP2kr] = ACTIONS(112),
    [anon_sym_OVR2kr] = ACTIONS(112),
    [anon_sym_EQU2kr] = ACTIONS(112),
    [anon_sym_NEQ2kr] = ACTIONS(112),
    [anon_sym_GTH2kr] = ACTIONS(112),
    [anon_sym_LTH2kr] = ACTIONS(112),
    [anon_sym_JMP2kr] = ACTIONS(112),
    [anon_sym_JCN2kr] = ACTIONS(112),
    [anon_sym_JSR2kr] = ACTIONS(112),
    [anon_sym_STH2kr] = ACTIONS(112),
    [anon_sym_LDZ2kr] = ACTIONS(112),
    [anon_sym_STZ2kr] = ACTIONS(112),
    [anon_sym_LDR2kr] = ACTIONS(112),
    [anon_sym_STR2kr] = ACTIONS(112),
    [anon_sym_LDA2kr] = ACTIONS(112),
    [anon_sym_STA2kr] = ACTIONS(112),
    [anon_sym_DEI2kr] = ACTIONS(112),
    [anon_sym_DEO2kr] = ACTIONS(112),
    [anon_sym_ADD2kr] = ACTIONS(112),
    [anon_sym_SUB2kr] = ACTIONS(112),
    [anon_sym_MUL2kr] = ACTIONS(112),
    [anon_sym_DIV2kr] = ACTIONS(112),
    [anon_sym_AND2kr] = ACTIONS(112),
    [anon_sym_ORA2kr] = ACTIONS(112),
    [anon_sym_EOR2kr] = ACTIONS(112),
    [anon_sym_SFT2kr] = ACTIONS(112),
    [anon_sym_PIPE] = ACTIONS(112),
    [anon_sym_DOLLAR] = ACTIONS(112),
    [anon_sym_POUND] = ACTIONS(112),
    [anon_sym_AT] = ACTIONS(112),
    [anon_sym_SLASH] = ACTIONS(110),
    [anon_sym_COMMA] = ACTIONS(112),
    [anon_sym__] = ACTIONS(112),
    [anon_sym_DOT] = ACTIONS(112),
    [anon_sym_DASH] = ACTIONS(112),
    [anon_sym_SEMI] = ACTIONS(112),
    [anon_sym_EQ] = ACTIONS(112),
    [anon_sym_BANG] = ACTIONS(112),
    [anon_sym_QMARK] = ACTIONS(112),
    [anon_sym_AMP] = ACTIONS(112),
    [anon_sym_LBRACK] = ACTIONS(112),
    [anon_sym_RBRACK] = ACTIONS(112),
    [anon_sym_DQUOTE] = ACTIONS(112),
    [sym_number] = ACTIONS(112),
    [sym_comment] = ACTIONS(3),
  },
  [13] = {
    [ts_builtin_sym_end] = ACTIONS(116),
    [sym_identifier] = ACTIONS(118),
    [anon_sym_PERCENT] = ACTIONS(116),
    [anon_sym_RBRACE] = ACTIONS(116),
    [anon_sym_TILDE] = ACTIONS(116),
    [anon_sym_BRK] = ACTIONS(116),
    [anon_sym_INC] = ACTIONS(118),
    [anon_sym_POP] = ACTIONS(118),
    [anon_sym_NIP] = ACTIONS(118),
    [anon_sym_SWP] = ACTIONS(118),
    [anon_sym_ROT] = ACTIONS(118),
    [anon_sym_DUP] = ACTIONS(118),
    [anon_sym_OVR] = ACTIONS(118),
    [anon_sym_EQU] = ACTIONS(118),
    [anon_sym_NEQ] = ACTIONS(118),
    [anon_sym_GTH] = ACTIONS(118),
    [anon_sym_LTH] = ACTIONS(118),
    [anon_sym_JMP] = ACTIONS(118),
    [anon_sym_JCN] = ACTIONS(118),
    [anon_sym_JSR] = ACTIONS(118),
    [anon_sym_STH] = ACTIONS(118),
    [anon_sym_LDZ] = ACTIONS(118),
    [anon_sym_STZ] = ACTIONS(118),
    [anon_sym_LDR] = ACTIONS(118),
    [anon_sym_STR] = ACTIONS(118),
    [anon_sym_LDA] = ACTIONS(118),
    [anon_sym_STA] = ACTIONS(118),
    [anon_sym_DEI] = ACTIONS(118),
    [anon_sym_DEO] = ACTIONS(118),
    [anon_sym_ADD] = ACTIONS(118),
    [anon_sym_SUB] = ACTIONS(118),
    [anon_sym_MUL] = ACTIONS(118),
    [anon_sym_DIV] = ACTIONS(118),
    [anon_sym_AND] = ACTIONS(118),
    [anon_sym_ORA] = ACTIONS(118),
    [anon_sym_EOR] = ACTIONS(118),
    [anon_sym_SFT] = ACTIONS(118),
    [anon_sym_JCI] = ACTIONS(116),
    [anon_sym_INC2] = ACTIONS(118),
    [anon_sym_POP2] = ACTIONS(118),
    [anon_sym_NIP2] = ACTIONS(118),
    [anon_sym_SWP2] = ACTIONS(118),
    [anon_sym_ROT2] = ACTIONS(118),
    [anon_sym_DUP2] = ACTIONS(118),
    [anon_sym_OVR2] = ACTIONS(118),
    [anon_sym_EQU2] = ACTIONS(118),
    [anon_sym_NEQ2] = ACTIONS(118),
    [anon_sym_GTH2] = ACTIONS(118),
    [anon_sym_LTH2] = ACTIONS(118),
    [anon_sym_JMP2] = ACTIONS(118),
    [anon_sym_JCN2] = ACTIONS(118),
    [anon_sym_JSR2] = ACTIONS(118),
    [anon_sym_STH2] = ACTIONS(118),
    [anon_sym_LDZ2] = ACTIONS(118),
    [anon_sym_STZ2] = ACTIONS(118),
    [anon_sym_LDR2] = ACTIONS(118),
    [anon_sym_STR2] = ACTIONS(118),
    [anon_sym_LDA2] = ACTIONS(118),
    [anon_sym_STA2] = ACTIONS(118),
    [anon_sym_DEI2] = ACTIONS(118),
    [anon_sym_DEO2] = ACTIONS(118),
    [anon_sym_ADD2] = ACTIONS(118),
    [anon_sym_SUB2] = ACTIONS(118),
    [anon_sym_MUL2] = ACTIONS(118),
    [anon_sym_DIV2] = ACTIONS(118),
    [anon_sym_AND2] = ACTIONS(118),
    [anon_sym_ORA2] = ACTIONS(118),
    [anon_sym_EOR2] = ACTIONS(118),
    [anon_sym_SFT2] = ACTIONS(118),
    [anon_sym_JMI] = ACTIONS(116),
    [anon_sym_INCr] = ACTIONS(116),
    [anon_sym_POPr] = ACTIONS(116),
    [anon_sym_NIPr] = ACTIONS(116),
    [anon_sym_SWPr] = ACTIONS(116),
    [anon_sym_ROTr] = ACTIONS(116),
    [anon_sym_DUPr] = ACTIONS(116),
    [anon_sym_OVRr] = ACTIONS(116),
    [anon_sym_EQUr] = ACTIONS(116),
    [anon_sym_NEQr] = ACTIONS(116),
    [anon_sym_GTHr] = ACTIONS(116),
    [anon_sym_LTHr] = ACTIONS(116),
    [anon_sym_JMPr] = ACTIONS(116),
    [anon_sym_JCNr] = ACTIONS(116),
    [anon_sym_JSRr] = ACTIONS(116),
    [anon_sym_STHr] = ACTIONS(116),
    [anon_sym_LDZr] = ACTIONS(116),
    [anon_sym_STZr] = ACTIONS(116),
    [anon_sym_LDRr] = ACTIONS(116),
    [anon_sym_STRr] = ACTIONS(116),
    [anon_sym_LDAr] = ACTIONS(116),
    [anon_sym_STAr] = ACTIONS(116),
    [anon_sym_DEIr] = ACTIONS(116),
    [anon_sym_DEOr] = ACTIONS(116),
    [anon_sym_ADDr] = ACTIONS(116),
    [anon_sym_SUBr] = ACTIONS(116),
    [anon_sym_MULr] = ACTIONS(116),
    [anon_sym_DIVr] = ACTIONS(116),
    [anon_sym_ANDr] = ACTIONS(116),
    [anon_sym_ORAr] = ACTIONS(116),
    [anon_sym_EORr] = ACTIONS(116),
    [anon_sym_SFTr] = ACTIONS(116),
    [anon_sym_JSI] = ACTIONS(116),
    [anon_sym_INC2r] = ACTIONS(116),
    [anon_sym_POP2r] = ACTIONS(116),
    [anon_sym_NIP2r] = ACTIONS(116),
    [anon_sym_SWP2r] = ACTIONS(116),
    [anon_sym_ROT2r] = ACTIONS(116),
    [anon_sym_DUP2r] = ACTIONS(116),
    [anon_sym_OVR2r] = ACTIONS(116),
    [anon_sym_EQU2r] = ACTIONS(116),
    [anon_sym_NEQ2r] = ACTIONS(116),
    [anon_sym_GTH2r] = ACTIONS(116),
    [anon_sym_LTH2r] = ACTIONS(116),
    [anon_sym_JMP2r] = ACTIONS(116),
    [anon_sym_JCN2r] = ACTIONS(116),
    [anon_sym_JSR2r] = ACTIONS(116),
    [anon_sym_STH2r] = ACTIONS(116),
    [anon_sym_LDZ2r] = ACTIONS(116),
    [anon_sym_STZ2r] = ACTIONS(116),
    [anon_sym_LDR2r] = ACTIONS(116),
    [anon_sym_STR2r] = ACTIONS(116),
    [anon_sym_LDA2r] = ACTIONS(116),
    [anon_sym_STA2r] = ACTIONS(116),
    [anon_sym_DEI2r] = ACTIONS(116),
    [anon_sym_DEO2r] = ACTIONS(116),
    [anon_sym_ADD2r] = ACTIONS(116),
    [anon_sym_SUB2r] = ACTIONS(116),
    [anon_sym_MUL2r] = ACTIONS(116),
    [anon_sym_DIV2r] = ACTIONS(116),
    [anon_sym_AND2r] = ACTIONS(116),
    [anon_sym_ORA2r] = ACTIONS(116),
    [anon_sym_EOR2r] = ACTIONS(116),
    [anon_sym_SFT2r] = ACTIONS(116),
    [anon_sym_LIT] = ACTIONS(118),
    [anon_sym_INCk] = ACTIONS(118),
    [anon_sym_POPk] = ACTIONS(118),
    [anon_sym_NIPk] = ACTIONS(118),
    [anon_sym_SWPk] = ACTIONS(118),
    [anon_sym_ROTk] = ACTIONS(118),
    [anon_sym_DUPk] = ACTIONS(118),
    [anon_sym_OVRk] = ACTIONS(118),
    [anon_sym_EQUk] = ACTIONS(118),
    [anon_sym_NEQk] = ACTIONS(118),
    [anon_sym_GTHk] = ACTIONS(118),
    [anon_sym_LTHk] = ACTIONS(118),
    [anon_sym_JMPk] = ACTIONS(118),
    [anon_sym_JCNk] = ACTIONS(118),
    [anon_sym_JSRk] = ACTIONS(118),
    [anon_sym_STHk] = ACTIONS(118),
    [anon_sym_LDZk] = ACTIONS(118),
    [anon_sym_STZk] = ACTIONS(118),
    [anon_sym_LDRk] = ACTIONS(118),
    [anon_sym_STRk] = ACTIONS(118),
    [anon_sym_LDAk] = ACTIONS(118),
    [anon_sym_STAk] = ACTIONS(118),
    [anon_sym_DEIk] = ACTIONS(118),
    [anon_sym_DEOk] = ACTIONS(118),
    [anon_sym_ADDk] = ACTIONS(118),
    [anon_sym_SUBk] = ACTIONS(118),
    [anon_sym_MULk] = ACTIONS(118),
    [anon_sym_DIVk] = ACTIONS(118),
    [anon_sym_ANDk] = ACTIONS(118),
    [anon_sym_ORAk] = ACTIONS(118),
    [anon_sym_EORk] = ACTIONS(118),
    [anon_sym_SFTk] = ACTIONS(118),
    [anon_sym_LIT2] = ACTIONS(118),
    [anon_sym_INC2k] = ACTIONS(118),
    [anon_sym_POP2k] = ACTIONS(118),
    [anon_sym_NIP2k] = ACTIONS(118),
    [anon_sym_SWP2k] = ACTIONS(118),
    [anon_sym_ROT2k] = ACTIONS(118),
    [anon_sym_DUP2k] = ACTIONS(118),
    [anon_sym_OVR2k] = ACTIONS(118),
    [anon_sym_EQU2k] = ACTIONS(118),
    [anon_sym_NEQ2k] = ACTIONS(118),
    [anon_sym_GTH2k] = ACTIONS(118),
    [anon_sym_LTH2k] = ACTIONS(118),
    [anon_sym_JMP2k] = ACTIONS(118),
    [anon_sym_JCN2k] = ACTIONS(118),
    [anon_sym_JSR2k] = ACTIONS(118),
    [anon_sym_STH2k] = ACTIONS(118),
    [anon_sym_LDZ2k] = ACTIONS(118),
    [anon_sym_STZ2k] = ACTIONS(118),
    [anon_sym_LDR2k] = ACTIONS(118),
    [anon_sym_STR2k] = ACTIONS(118),
    [anon_sym_LDA2k] = ACTIONS(118),
    [anon_sym_STA2k] = ACTIONS(118),
    [anon_sym_DEI2k] = ACTIONS(118),
    [anon_sym_DEO2k] = ACTIONS(118),
    [anon_sym_ADD2k] = ACTIONS(118),
    [anon_sym_SUB2k] = ACTIONS(118),
    [anon_sym_MUL2k] = ACTIONS(118),
    [anon_sym_DIV2k] = ACTIONS(118),
    [anon_sym_AND2k] = ACTIONS(118),
    [anon_sym_ORA2k] = ACTIONS(118),
    [anon_sym_EOR2k] = ACTIONS(118),
    [anon_sym_SFT2k] = ACTIONS(118),
    [anon_sym_LITr] = ACTIONS(116),
    [anon_sym_INCkr] = ACTIONS(116),
    [anon_sym_POPkr] = ACTIONS(116),
    [anon_sym_NIPkr] = ACTIONS(116),
    [anon_sym_SWPkr] = ACTIONS(116),
    [anon_sym_ROTkr] = ACTIONS(116),
    [anon_sym_DUPkr] = ACTIONS(116),
    [anon_sym_OVRkr] = ACTIONS(116),
    [anon_sym_EQUkr] = ACTIONS(116),
    [anon_sym_NEQkr] = ACTIONS(116),
    [anon_sym_GTHkr] = ACTIONS(116),
    [anon_sym_LTHkr] = ACTIONS(116),
    [anon_sym_JMPkr] = ACTIONS(116),
    [anon_sym_JCNkr] = ACTIONS(116),
    [anon_sym_JSRkr] = ACTIONS(116),
    [anon_sym_STHkr] = ACTIONS(116),
    [anon_sym_LDZkr] = ACTIONS(116),
    [anon_sym_STZkr] = ACTIONS(116),
    [anon_sym_LDRkr] = ACTIONS(116),
    [anon_sym_STRkr] = ACTIONS(116),
    [anon_sym_LDAkr] = ACTIONS(116),
    [anon_sym_STAkr] = ACTIONS(116),
    [anon_sym_DEIkr] = ACTIONS(116),
    [anon_sym_DEOkr] = ACTIONS(116),
    [anon_sym_ADDkr] = ACTIONS(116),
    [anon_sym_SUBkr] = ACTIONS(116),
    [anon_sym_MULkr] = ACTIONS(116),
    [anon_sym_DIVkr] = ACTIONS(116),
    [anon_sym_ANDkr] = ACTIONS(116),
    [anon_sym_ORAkr] = ACTIONS(116),
    [anon_sym_EORkr] = ACTIONS(116),
    [anon_sym_SFTkr] = ACTIONS(116),
    [anon_sym_LIT2r] = ACTIONS(116),
    [anon_sym_INC2kr] = ACTIONS(116),
    [anon_sym_POP2kr] = ACTIONS(116),
    [anon_sym_NIP2kr] = ACTIONS(116),
    [anon_sym_SWP2kr] = ACTIONS(116),
    [anon_sym_ROT2kr] = ACTIONS(116),
    [anon_sym_DUP2kr] = ACTIONS(116),
    [anon_sym_OVR2kr] = ACTIONS(116),
    [anon_sym_EQU2kr] = ACTIONS(116),
    [anon_sym_NEQ2kr] = ACTIONS(116),
    [anon_sym_GTH2kr] = ACTIONS(116),
    [anon_sym_LTH2kr] = ACTIONS(116),
    [anon_sym_JMP2kr] = ACTIONS(116),
    [anon_sym_JCN2kr] = ACTIONS(116),
    [anon_sym_JSR2kr] = ACTIONS(116),
    [anon_sym_STH2kr] = ACTIONS(116),
    [anon_sym_LDZ2kr] = ACTIONS(116),
    [anon_sym_STZ2kr] = ACTIONS(116),
    [anon_sym_LDR2kr] = ACTIONS(116),
    [anon_sym_STR2kr] = ACTIONS(116),
    [anon_sym_LDA2kr] = ACTIONS(116),
    [anon_sym_STA2kr] = ACTIONS(116),
    [anon_sym_DEI2kr] = ACTIONS(116),
    [anon_sym_DEO2kr] = ACTIONS(116),
    [anon_sym_ADD2kr] = ACTIONS(116),
    [anon_sym_SUB2kr] = ACTIONS(116),
    [anon_sym_MUL2kr] = ACTIONS(116),
    [anon_sym_DIV2kr] = ACTIONS(116),
    [anon_sym_AND2kr] = ACTIONS(116),
    [anon_sym_ORA2kr] = ACTIONS(116),
    [anon_sym_EOR2kr] = ACTIONS(116),
    [anon_sym_SFT2kr] = ACTIONS(116),
    [anon_sym_PIPE] = ACTIONS(116),
    [anon_sym_DOLLAR] = ACTIONS(116),
    [anon_sym_POUND] = ACTIONS(116),
    [anon_sym_AT] = ACTIONS(116),
    [anon_sym_SLASH] = ACTIONS(110),
    [anon_sym_COMMA] = ACTIONS(116),
    [anon_sym__] = ACTIONS(116),
    [anon_sym_DOT] = ACTIONS(116),
    [anon_sym_DASH] = ACTIONS(116),
    [anon_sym_SEMI] = ACTIONS(116),
    [anon_sym_EQ] = ACTIONS(116),
    [anon_sym_BANG] = ACTIONS(116),
    [anon_sym_QMARK] = ACTIONS(116),
    [anon_sym_AMP] = ACTIONS(116),
    [anon_sym_LBRACK] = ACTIONS(116),
    [anon_sym_RBRACK] = ACTIONS(116),
    [anon_sym_DQUOTE] = ACTIONS(116),
    [sym_number] = ACTIONS(116),
    [sym_comment] = ACTIONS(3),
  },
  [14] = {
    [ts_builtin_sym_end] = ACTIONS(112),
    [sym_identifier] = ACTIONS(114),
    [anon_sym_PERCENT] = ACTIONS(112),
    [anon_sym_RBRACE] = ACTIONS(112),
    [anon_sym_TILDE] = ACTIONS(112),
    [anon_sym_BRK] = ACTIONS(112),
    [anon_sym_INC] = ACTIONS(114),
    [anon_sym_POP] = ACTIONS(114),
    [anon_sym_NIP] = ACTIONS(114),
    [anon_sym_SWP] = ACTIONS(114),
    [anon_sym_ROT] = ACTIONS(114),
    [anon_sym_DUP] = ACTIONS(114),
    [anon_sym_OVR] = ACTIONS(114),
    [anon_sym_EQU] = ACTIONS(114),
    [anon_sym_NEQ] = ACTIONS(114),
    [anon_sym_GTH] = ACTIONS(114),
    [anon_sym_LTH] = ACTIONS(114),
    [anon_sym_JMP] = ACTIONS(114),
    [anon_sym_JCN] = ACTIONS(114),
    [anon_sym_JSR] = ACTIONS(114),
    [anon_sym_STH] = ACTIONS(114),
    [anon_sym_LDZ] = ACTIONS(114),
    [anon_sym_STZ] = ACTIONS(114),
    [anon_sym_LDR] = ACTIONS(114),
    [anon_sym_STR] = ACTIONS(114),
    [anon_sym_LDA] = ACTIONS(114),
    [anon_sym_STA] = ACTIONS(114),
    [anon_sym_DEI] = ACTIONS(114),
    [anon_sym_DEO] = ACTIONS(114),
    [anon_sym_ADD] = ACTIONS(114),
    [anon_sym_SUB] = ACTIONS(114),
    [anon_sym_MUL] = ACTIONS(114),
    [anon_sym_DIV] = ACTIONS(114),
    [anon_sym_AND] = ACTIONS(114),
    [anon_sym_ORA] = ACTIONS(114),
    [anon_sym_EOR] = ACTIONS(114),
    [anon_sym_SFT] = ACTIONS(114),
    [anon_sym_JCI] = ACTIONS(112),
    [anon_sym_INC2] = ACTIONS(114),
    [anon_sym_POP2] = ACTIONS(114),
    [anon_sym_NIP2] = ACTIONS(114),
    [anon_sym_SWP2] = ACTIONS(114),
    [anon_sym_ROT2] = ACTIONS(114),
    [anon_sym_DUP2] = ACTIONS(114),
    [anon_sym_OVR2] = ACTIONS(114),
    [anon_sym_EQU2] = ACTIONS(114),
    [anon_sym_NEQ2] = ACTIONS(114),
    [anon_sym_GTH2] = ACTIONS(114),
    [anon_sym_LTH2] = ACTIONS(114),
    [anon_sym_JMP2] = ACTIONS(114),
    [anon_sym_JCN2] = ACTIONS(114),
    [anon_sym_JSR2] = ACTIONS(114),
    [anon_sym_STH2] = ACTIONS(114),
    [anon_sym_LDZ2] = ACTIONS(114),
    [anon_sym_STZ2] = ACTIONS(114),
    [anon_sym_LDR2] = ACTIONS(114),
    [anon_sym_STR2] = ACTIONS(114),
    [anon_sym_LDA2] = ACTIONS(114),
    [anon_sym_STA2] = ACTIONS(114),
    [anon_sym_DEI2] = ACTIONS(114),
    [anon_sym_DEO2] = ACTIONS(114),
    [anon_sym_ADD2] = ACTIONS(114),
    [anon_sym_SUB2] = ACTIONS(114),
    [anon_sym_MUL2] = ACTIONS(114),
    [anon_sym_DIV2] = ACTIONS(114),
    [anon_sym_AND2] = ACTIONS(114),
    [anon_sym_ORA2] = ACTIONS(114),
    [anon_sym_EOR2] = ACTIONS(114),
    [anon_sym_SFT2] = ACTIONS(114),
    [anon_sym_JMI] = ACTIONS(112),
    [anon_sym_INCr] = ACTIONS(112),
    [anon_sym_POPr] = ACTIONS(112),
    [anon_sym_NIPr] = ACTIONS(112),
    [anon_sym_SWPr] = ACTIONS(112),
    [anon_sym_ROTr] = ACTIONS(112),
    [anon_sym_DUPr] = ACTIONS(112),
    [anon_sym_OVRr] = ACTIONS(112),
    [anon_sym_EQUr] = ACTIONS(112),
    [anon_sym_NEQr] = ACTIONS(112),
    [anon_sym_GTHr] = ACTIONS(112),
    [anon_sym_LTHr] = ACTIONS(112),
    [anon_sym_JMPr] = ACTIONS(112),
    [anon_sym_JCNr] = ACTIONS(112),
    [anon_sym_JSRr] = ACTIONS(112),
    [anon_sym_STHr] = ACTIONS(112),
    [anon_sym_LDZr] = ACTIONS(112),
    [anon_sym_STZr] = ACTIONS(112),
    [anon_sym_LDRr] = ACTIONS(112),
    [anon_sym_STRr] = ACTIONS(112),
    [anon_sym_LDAr] = ACTIONS(112),
    [anon_sym_STAr] = ACTIONS(112),
    [anon_sym_DEIr] = ACTIONS(112),
    [anon_sym_DEOr] = ACTIONS(112),
    [anon_sym_ADDr] = ACTIONS(112),
    [anon_sym_SUBr] = ACTIONS(112),
    [anon_sym_MULr] = ACTIONS(112),
    [anon_sym_DIVr] = ACTIONS(112),
    [anon_sym_ANDr] = ACTIONS(112),
    [anon_sym_ORAr] = ACTIONS(112),
    [anon_sym_EORr] = ACTIONS(112),
    [anon_sym_SFTr] = ACTIONS(112),
    [anon_sym_JSI] = ACTIONS(112),
    [anon_sym_INC2r] = ACTIONS(112),
    [anon_sym_POP2r] = ACTIONS(112),
    [anon_sym_NIP2r] = ACTIONS(112),
    [anon_sym_SWP2r] = ACTIONS(112),
    [anon_sym_ROT2r] = ACTIONS(112),
    [anon_sym_DUP2r] = ACTIONS(112),
    [anon_sym_OVR2r] = ACTIONS(112),
    [anon_sym_EQU2r] = ACTIONS(112),
    [anon_sym_NEQ2r] = ACTIONS(112),
    [anon_sym_GTH2r] = ACTIONS(112),
    [anon_sym_LTH2r] = ACTIONS(112),
    [anon_sym_JMP2r] = ACTIONS(112),
    [anon_sym_JCN2r] = ACTIONS(112),
    [anon_sym_JSR2r] = ACTIONS(112),
    [anon_sym_STH2r] = ACTIONS(112),
    [anon_sym_LDZ2r] = ACTIONS(112),
    [anon_sym_STZ2r] = ACTIONS(112),
    [anon_sym_LDR2r] = ACTIONS(112),
    [anon_sym_STR2r] = ACTIONS(112),
    [anon_sym_LDA2r] = ACTIONS(112),
    [anon_sym_STA2r] = ACTIONS(112),
    [anon_sym_DEI2r] = ACTIONS(112),
    [anon_sym_DEO2r] = ACTIONS(112),
    [anon_sym_ADD2r] = ACTIONS(112),
    [anon_sym_SUB2r] = ACTIONS(112),
    [anon_sym_MUL2r] = ACTIONS(112),
    [anon_sym_DIV2r] = ACTIONS(112),
    [anon_sym_AND2r] = ACTIONS(112),
    [anon_sym_ORA2r] = ACTIONS(112),
    [anon_sym_EOR2r] = ACTIONS(112),
    [anon_sym_SFT2r] = ACTIONS(112),
    [anon_sym_LIT] = ACTIONS(114),
    [anon_sym_INCk] = ACTIONS(114),
    [anon_sym_POPk] = ACTIONS(114),
    [anon_sym_NIPk] = ACTIONS(114),
    [anon_sym_SWPk] = ACTIONS(114),
    [anon_sym_ROTk] = ACTIONS(114),
    [anon_sym_DUPk] = ACTIONS(114),
    [anon_sym_OVRk] = ACTIONS(114),
    [anon_sym_EQUk] = ACTIONS(114),
    [anon_sym_NEQk] = ACTIONS(114),
    [anon_sym_GTHk] = ACTIONS(114),
    [anon_sym_LTHk] = ACTIONS(114),
    [anon_sym_JMPk] = ACTIONS(114),
    [anon_sym_JCNk] = ACTIONS(114),
    [anon_sym_JSRk] = ACTIONS(114),
    [anon_sym_STHk] = ACTIONS(114),
    [anon_sym_LDZk] = ACTIONS(114),
    [anon_sym_STZk] = ACTIONS(114),
    [anon_sym_LDRk] = ACTIONS(114),
    [anon_sym_STRk] = ACTIONS(114),
    [anon_sym_LDAk] = ACTIONS(114),
    [anon_sym_STAk] = ACTIONS(114),
    [anon_sym_DEIk] = ACTIONS(114),
    [anon_sym_DEOk] = ACTIONS(114),
    [anon_sym_ADDk] = ACTIONS(114),
    [anon_sym_SUBk] = ACTIONS(114),
    [anon_sym_MULk] = ACTIONS(114),
    [anon_sym_DIVk] = ACTIONS(114),
    [anon_sym_ANDk] = ACTIONS(114),
    [anon_sym_ORAk] = ACTIONS(114),
    [anon_sym_EORk] = ACTIONS(114),
    [anon_sym_SFTk] = ACTIONS(114),
    [anon_sym_LIT2] = ACTIONS(114),
    [anon_sym_INC2k] = ACTIONS(114),
    [anon_sym_POP2k] = ACTIONS(114),
    [anon_sym_NIP2k] = ACTIONS(114),
    [anon_sym_SWP2k] = ACTIONS(114),
    [anon_sym_ROT2k] = ACTIONS(114),
    [anon_sym_DUP2k] = ACTIONS(114),
    [anon_sym_OVR2k] = ACTIONS(114),
    [anon_sym_EQU2k] = ACTIONS(114),
    [anon_sym_NEQ2k] = ACTIONS(114),
    [anon_sym_GTH2k] = ACTIONS(114),
    [anon_sym_LTH2k] = ACTIONS(114),
    [anon_sym_JMP2k] = ACTIONS(114),
    [anon_sym_JCN2k] = ACTIONS(114),
    [anon_sym_JSR2k] = ACTIONS(114),
    [anon_sym_STH2k] = ACTIONS(114),
    [anon_sym_LDZ2k] = ACTIONS(114),
    [anon_sym_STZ2k] = ACTIONS(114),
    [anon_sym_LDR2k] = ACTIONS(114),
    [anon_sym_STR2k] = ACTIONS(114),
    [anon_sym_LDA2k] = ACTIONS(114),
    [anon_sym_STA2k] = ACTIONS(114),
    [anon_sym_DEI2k] = ACTIONS(114),
    [anon_sym_DEO2k] = ACTIONS(114),
    [anon_sym_ADD2k] = ACTIONS(114),
    [anon_sym_SUB2k] = ACTIONS(114),
    [anon_sym_MUL2k] = ACTIONS(114),
    [anon_sym_DIV2k] = ACTIONS(114),
    [anon_sym_AND2k] = ACTIONS(114),
    [anon_sym_ORA2k] = ACTIONS(114),
    [anon_sym_EOR2k] = ACTIONS(114),
    [anon_sym_SFT2k] = ACTIONS(114),
    [anon_sym_LITr] = ACTIONS(112),
    [anon_sym_INCkr] = ACTIONS(112),
    [anon_sym_POPkr] = ACTIONS(112),
    [anon_sym_NIPkr] = ACTIONS(112),
    [anon_sym_SWPkr] = ACTIONS(112),
    [anon_sym_ROTkr] = ACTIONS(112),
    [anon_sym_DUPkr] = ACTIONS(112),
    [anon_sym_OVRkr] = ACTIONS(112),
    [anon_sym_EQUkr] = ACTIONS(112),
    [anon_sym_NEQkr] = ACTIONS(112),
    [anon_sym_GTHkr] = ACTIONS(112),
    [anon_sym_LTHkr] = ACTIONS(112),
    [anon_sym_JMPkr] = ACTIONS(112),
    [anon_sym_JCNkr] = ACTIONS(112),
    [anon_sym_JSRkr] = ACTIONS(112),
    [anon_sym_STHkr] = ACTIONS(112),
    [anon_sym_LDZkr] = ACTIONS(112),
    [anon_sym_STZkr] = ACTIONS(112),
    [anon_sym_LDRkr] = ACTIONS(112),
    [anon_sym_STRkr] = ACTIONS(112),
    [anon_sym_LDAkr] = ACTIONS(112),
    [anon_sym_STAkr] = ACTIONS(112),
    [anon_sym_DEIkr] = ACTIONS(112),
    [anon_sym_DEOkr] = ACTIONS(112),
    [anon_sym_ADDkr] = ACTIONS(112),
    [anon_sym_SUBkr] = ACTIONS(112),
    [anon_sym_MULkr] = ACTIONS(112),
    [anon_sym_DIVkr] = ACTIONS(112),
    [anon_sym_ANDkr] = ACTIONS(112),
    [anon_sym_ORAkr] = ACTIONS(112),
    [anon_sym_EORkr] = ACTIONS(112),
    [anon_sym_SFTkr] = ACTIONS(112),
    [anon_sym_LIT2r] = ACTIONS(112),
    [anon_sym_INC2kr] = ACTIONS(112),
    [anon_sym_POP2kr] = ACTIONS(112),
    [anon_sym_NIP2kr] = ACTIONS(112),
    [anon_sym_SWP2kr] = ACTIONS(112),
    [anon_sym_ROT2kr] = ACTIONS(112),
    [anon_sym_DUP2kr] = ACTIONS(112),
    [anon_sym_OVR2kr] = ACTIONS(112),
    [anon_sym_EQU2kr] = ACTIONS(112),
    [anon_sym_NEQ2kr] = ACTIONS(112),
    [anon_sym_GTH2kr] = ACTIONS(112),
    [anon_sym_LTH2kr] = ACTIONS(112),
    [anon_sym_JMP2kr] = ACTIONS(112),
    [anon_sym_JCN2kr] = ACTIONS(112),
    [anon_sym_JSR2kr] = ACTIONS(112),
    [anon_sym_STH2kr] = ACTIONS(112),
    [anon_sym_LDZ2kr] = ACTIONS(112),
    [anon_sym_STZ2kr] = ACTIONS(112),
    [anon_sym_LDR2kr] = ACTIONS(112),
    [anon_sym_STR2kr] = ACTIONS(112),
    [anon_sym_LDA2kr] = ACTIONS(112),
    [anon_sym_STA2kr] = ACTIONS(112),
    [anon_sym_DEI2kr] = ACTIONS(112),
    [anon_sym_DEO2kr] = ACTIONS(112),
    [anon_sym_ADD2kr] = ACTIONS(112),
    [anon_sym_SUB2kr] = ACTIONS(112),
    [anon_sym_MUL2kr] = ACTIONS(112),
    [anon_sym_DIV2kr] = ACTIONS(112),
    [anon_sym_AND2kr] = ACTIONS(112),
    [anon_sym_ORA2kr] = ACTIONS(112),
    [anon_sym_EOR2kr] = ACTIONS(112),
    [anon_sym_SFT2kr] = ACTIONS(112),
    [anon_sym_PIPE] = ACTIONS(112),
    [anon_sym_DOLLAR] = ACTIONS(112),
    [anon_sym_POUND] = ACTIONS(112),
    [anon_sym_AT] = ACTIONS(112),
    [anon_sym_COMMA] = ACTIONS(112),
    [anon_sym__] = ACTIONS(112),
    [anon_sym_DOT] = ACTIONS(112),
    [anon_sym_DASH] = ACTIONS(112),
    [anon_sym_SEMI] = ACTIONS(112),
    [anon_sym_EQ] = ACTIONS(112),
    [anon_sym_BANG] = ACTIONS(112),
    [anon_sym_QMARK] = ACTIONS(112),
    [anon_sym_AMP] = ACTIONS(112),
    [anon_sym_LBRACK] = ACTIONS(112),
    [anon_sym_RBRACK] = ACTIONS(112),
    [anon_sym_DQUOTE] = ACTIONS(112),
    [sym_number] = ACTIONS(112),
    [sym_comment] = ACTIONS(3),
  },
  [15] = {
    [ts_builtin_sym_end] = ACTIONS(120),
    [sym_identifier] = ACTIONS(122),
    [anon_sym_PERCENT] = ACTIONS(120),
    [anon_sym_RBRACE] = ACTIONS(120),
    [anon_sym_TILDE] = ACTIONS(120),
    [anon_sym_BRK] = ACTIONS(120),
    [anon_sym_INC] = ACTIONS(122),
    [anon_sym_POP] = ACTIONS(122),
    [anon_sym_NIP] = ACTIONS(122),
    [anon_sym_SWP] = ACTIONS(122),
    [anon_sym_ROT] = ACTIONS(122),
    [anon_sym_DUP] = ACTIONS(122),
    [anon_sym_OVR] = ACTIONS(122),
    [anon_sym_EQU] = ACTIONS(122),
    [anon_sym_NEQ] = ACTIONS(122),
    [anon_sym_GTH] = ACTIONS(122),
    [anon_sym_LTH] = ACTIONS(122),
    [anon_sym_JMP] = ACTIONS(122),
    [anon_sym_JCN] = ACTIONS(122),
    [anon_sym_JSR] = ACTIONS(122),
    [anon_sym_STH] = ACTIONS(122),
    [anon_sym_LDZ] = ACTIONS(122),
    [anon_sym_STZ] = ACTIONS(122),
    [anon_sym_LDR] = ACTIONS(122),
    [anon_sym_STR] = ACTIONS(122),
    [anon_sym_LDA] = ACTIONS(122),
    [anon_sym_STA] = ACTIONS(122),
    [anon_sym_DEI] = ACTIONS(122),
    [anon_sym_DEO] = ACTIONS(122),
    [anon_sym_ADD] = ACTIONS(122),
    [anon_sym_SUB] = ACTIONS(122),
    [anon_sym_MUL] = ACTIONS(122),
    [anon_sym_DIV] = ACTIONS(122),
    [anon_sym_AND] = ACTIONS(122),
    [anon_sym_ORA] = ACTIONS(122),
    [anon_sym_EOR] = ACTIONS(122),
    [anon_sym_SFT] = ACTIONS(122),
    [anon_sym_JCI] = ACTIONS(120),
    [anon_sym_INC2] = ACTIONS(122),
    [anon_sym_POP2] = ACTIONS(122),
    [anon_sym_NIP2] = ACTIONS(122),
    [anon_sym_SWP2] = ACTIONS(122),
    [anon_sym_ROT2] = ACTIONS(122),
    [anon_sym_DUP2] = ACTIONS(122),
    [anon_sym_OVR2] = ACTIONS(122),
    [anon_sym_EQU2] = ACTIONS(122),
    [anon_sym_NEQ2] = ACTIONS(122),
    [anon_sym_GTH2] = ACTIONS(122),
    [anon_sym_LTH2] = ACTIONS(122),
    [anon_sym_JMP2] = ACTIONS(122),
    [anon_sym_JCN2] = ACTIONS(122),
    [anon_sym_JSR2] = ACTIONS(122),
    [anon_sym_STH2] = ACTIONS(122),
    [anon_sym_LDZ2] = ACTIONS(122),
    [anon_sym_STZ2] = ACTIONS(122),
    [anon_sym_LDR2] = ACTIONS(122),
    [anon_sym_STR2] = ACTIONS(122),
    [anon_sym_LDA2] = ACTIONS(122),
    [anon_sym_STA2] = ACTIONS(122),
    [anon_sym_DEI2] = ACTIONS(122),
    [anon_sym_DEO2] = ACTIONS(122),
    [anon_sym_ADD2] = ACTIONS(122),
    [anon_sym_SUB2] = ACTIONS(122),
    [anon_sym_MUL2] = ACTIONS(122),
    [anon_sym_DIV2] = ACTIONS(122),
    [anon_sym_AND2] = ACTIONS(122),
    [anon_sym_ORA2] = ACTIONS(122),
    [anon_sym_EOR2] = ACTIONS(122),
    [anon_sym_SFT2] = ACTIONS(122),
    [anon_sym_JMI] = ACTIONS(120),
    [anon_sym_INCr] = ACTIONS(120),
    [anon_sym_POPr] = ACTIONS(120),
    [anon_sym_NIPr] = ACTIONS(120),
    [anon_sym_SWPr] = ACTIONS(120),
    [anon_sym_ROTr] = ACTIONS(120),
    [anon_sym_DUPr] = ACTIONS(120),
    [anon_sym_OVRr] = ACTIONS(120),
    [anon_sym_EQUr] = ACTIONS(120),
    [anon_sym_NEQr] = ACTIONS(120),
    [anon_sym_GTHr] = ACTIONS(120),
    [anon_sym_LTHr] = ACTIONS(120),
    [anon_sym_JMPr] = ACTIONS(120),
    [anon_sym_JCNr] = ACTIONS(120),
    [anon_sym_JSRr] = ACTIONS(120),
    [anon_sym_STHr] = ACTIONS(120),
    [anon_sym_LDZr] = ACTIONS(120),
    [anon_sym_STZr] = ACTIONS(120),
    [anon_sym_LDRr] = ACTIONS(120),
    [anon_sym_STRr] = ACTIONS(120),
    [anon_sym_LDAr] = ACTIONS(120),
    [anon_sym_STAr] = ACTIONS(120),
    [anon_sym_DEIr] = ACTIONS(120),
    [anon_sym_DEOr] = ACTIONS(120),
    [anon_sym_ADDr] = ACTIONS(120),
    [anon_sym_SUBr] = ACTIONS(120),
    [anon_sym_MULr] = ACTIONS(120),
    [anon_sym_DIVr] = ACTIONS(120),
    [anon_sym_ANDr] = ACTIONS(120),
    [anon_sym_ORAr] = ACTIONS(120),
    [anon_sym_EORr] = ACTIONS(120),
    [anon_sym_SFTr] = ACTIONS(120),
    [anon_sym_JSI] = ACTIONS(120),
    [anon_sym_INC2r] = ACTIONS(120),
    [anon_sym_POP2r] = ACTIONS(120),
    [anon_sym_NIP2r] = ACTIONS(120),
    [anon_sym_SWP2r] = ACTIONS(120),
    [anon_sym_ROT2r] = ACTIONS(120),
    [anon_sym_DUP2r] = ACTIONS(120),
    [anon_sym_OVR2r] = ACTIONS(120),
    [anon_sym_EQU2r] = ACTIONS(120),
    [anon_sym_NEQ2r] = ACTIONS(120),
    [anon_sym_GTH2r] = ACTIONS(120),
    [anon_sym_LTH2r] = ACTIONS(120),
    [anon_sym_JMP2r] = ACTIONS(120),
    [anon_sym_JCN2r] = ACTIONS(120),
    [anon_sym_JSR2r] = ACTIONS(120),
    [anon_sym_STH2r] = ACTIONS(120),
    [anon_sym_LDZ2r] = ACTIONS(120),
    [anon_sym_STZ2r] = ACTIONS(120),
    [anon_sym_LDR2r] = ACTIONS(120),
    [anon_sym_STR2r] = ACTIONS(120),
    [anon_sym_LDA2r] = ACTIONS(120),
    [anon_sym_STA2r] = ACTIONS(120),
    [anon_sym_DEI2r] = ACTIONS(120),
    [anon_sym_DEO2r] = ACTIONS(120),
    [anon_sym_ADD2r] = ACTIONS(120),
    [anon_sym_SUB2r] = ACTIONS(120),
    [anon_sym_MUL2r] = ACTIONS(120),
    [anon_sym_DIV2r] = ACTIONS(120),
    [anon_sym_AND2r] = ACTIONS(120),
    [anon_sym_ORA2r] = ACTIONS(120),
    [anon_sym_EOR2r] = ACTIONS(120),
    [anon_sym_SFT2r] = ACTIONS(120),
    [anon_sym_LIT] = ACTIONS(122),
    [anon_sym_INCk] = ACTIONS(122),
    [anon_sym_POPk] = ACTIONS(122),
    [anon_sym_NIPk] = ACTIONS(122),
    [anon_sym_SWPk] = ACTIONS(122),
    [anon_sym_ROTk] = ACTIONS(122),
    [anon_sym_DUPk] = ACTIONS(122),
    [anon_sym_OVRk] = ACTIONS(122),
    [anon_sym_EQUk] = ACTIONS(122),
    [anon_sym_NEQk] = ACTIONS(122),
    [anon_sym_GTHk] = ACTIONS(122),
    [anon_sym_LTHk] = ACTIONS(122),
    [anon_sym_JMPk] = ACTIONS(122),
    [anon_sym_JCNk] = ACTIONS(122),
    [anon_sym_JSRk] = ACTIONS(122),
    [anon_sym_STHk] = ACTIONS(122),
    [anon_sym_LDZk] = ACTIONS(122),
    [anon_sym_STZk] = ACTIONS(122),
    [anon_sym_LDRk] = ACTIONS(122),
    [anon_sym_STRk] = ACTIONS(122),
    [anon_sym_LDAk] = ACTIONS(122),
    [anon_sym_STAk] = ACTIONS(122),
    [anon_sym_DEIk] = ACTIONS(122),
    [anon_sym_DEOk] = ACTIONS(122),
    [anon_sym_ADDk] = ACTIONS(122),
    [anon_sym_SUBk] = ACTIONS(122),
    [anon_sym_MULk] = ACTIONS(122),
    [anon_sym_DIVk] = ACTIONS(122),
    [anon_sym_ANDk] = ACTIONS(122),
    [anon_sym_ORAk] = ACTIONS(122),
    [anon_sym_EORk] = ACTIONS(122),
    [anon_sym_SFTk] = ACTIONS(122),
    [anon_sym_LIT2] = ACTIONS(122),
    [anon_sym_INC2k] = ACTIONS(122),
    [anon_sym_POP2k] = ACTIONS(122),
    [anon_sym_NIP2k] = ACTIONS(122),
    [anon_sym_SWP2k] = ACTIONS(122),
    [anon_sym_ROT2k] = ACTIONS(122),
    [anon_sym_DUP2k] = ACTIONS(122),
    [anon_sym_OVR2k] = ACTIONS(122),
    [anon_sym_EQU2k] = ACTIONS(122),
    [anon_sym_NEQ2k] = ACTIONS(122),
    [anon_sym_GTH2k] = ACTIONS(122),
    [anon_sym_LTH2k] = ACTIONS(122),
    [anon_sym_JMP2k] = ACTIONS(122),
    [anon_sym_JCN2k] = ACTIONS(122),
    [anon_sym_JSR2k] = ACTIONS(122),
    [anon_sym_STH2k] = ACTIONS(122),
    [anon_sym_LDZ2k] = ACTIONS(122),
    [anon_sym_STZ2k] = ACTIONS(122),
    [anon_sym_LDR2k] = ACTIONS(122),
    [anon_sym_STR2k] = ACTIONS(122),
    [anon_sym_LDA2k] = ACTIONS(122),
    [anon_sym_STA2k] = ACTIONS(122),
    [anon_sym_DEI2k] = ACTIONS(122),
    [anon_sym_DEO2k] = ACTIONS(122),
    [anon_sym_ADD2k] = ACTIONS(122),
    [anon_sym_SUB2k] = ACTIONS(122),
    [anon_sym_MUL2k] = ACTIONS(122),
    [anon_sym_DIV2k] = ACTIONS(122),
    [anon_sym_AND2k] = ACTIONS(122),
    [anon_sym_ORA2k] = ACTIONS(122),
    [anon_sym_EOR2k] = ACTIONS(122),
    [anon_sym_SFT2k] = ACTIONS(122),
    [anon_sym_LITr] = ACTIONS(120),
    [anon_sym_INCkr] = ACTIONS(120),
    [anon_sym_POPkr] = ACTIONS(120),
    [anon_sym_NIPkr] = ACTIONS(120),
    [anon_sym_SWPkr] = ACTIONS(120),
    [anon_sym_ROTkr] = ACTIONS(120),
    [anon_sym_DUPkr] = ACTIONS(120),
    [anon_sym_OVRkr] = ACTIONS(120),
    [anon_sym_EQUkr] = ACTIONS(120),
    [anon_sym_NEQkr] = ACTIONS(120),
    [anon_sym_GTHkr] = ACTIONS(120),
    [anon_sym_LTHkr] = ACTIONS(120),
    [anon_sym_JMPkr] = ACTIONS(120),
    [anon_sym_JCNkr] = ACTIONS(120),
    [anon_sym_JSRkr] = ACTIONS(120),
    [anon_sym_STHkr] = ACTIONS(120),
    [anon_sym_LDZkr] = ACTIONS(120),
    [anon_sym_STZkr] = ACTIONS(120),
    [anon_sym_LDRkr] = ACTIONS(120),
    [anon_sym_STRkr] = ACTIONS(120),
    [anon_sym_LDAkr] = ACTIONS(120),
    [anon_sym_STAkr] = ACTIONS(120),
    [anon_sym_DEIkr] = ACTIONS(120),
    [anon_sym_DEOkr] = ACTIONS(120),
    [anon_sym_ADDkr] = ACTIONS(120),
    [anon_sym_SUBkr] = ACTIONS(120),
    [anon_sym_MULkr] = ACTIONS(120),
    [anon_sym_DIVkr] = ACTIONS(120),
    [anon_sym_ANDkr] = ACTIONS(120),
    [anon_sym_ORAkr] = ACTIONS(120),
    [anon_sym_EORkr] = ACTIONS(120),
    [anon_sym_SFTkr] = ACTIONS(120),
    [anon_sym_LIT2r] = ACTIONS(120),
    [anon_sym_INC2kr] = ACTIONS(120),
    [anon_sym_POP2kr] = ACTIONS(120),
    [anon_sym_NIP2kr] = ACTIONS(120),
    [anon_sym_SWP2kr] = ACTIONS(120),
    [anon_sym_ROT2kr] = ACTIONS(120),
    [anon_sym_DUP2kr] = ACTIONS(120),
    [anon_sym_OVR2kr] = ACTIONS(120),
    [anon_sym_EQU2kr] = ACTIONS(120),
    [anon_sym_NEQ2kr] = ACTIONS(120),
    [anon_sym_GTH2kr] = ACTIONS(120),
    [anon_sym_LTH2kr] = ACTIONS(120),
    [anon_sym_JMP2kr] = ACTIONS(120),
    [anon_sym_JCN2kr] = ACTIONS(120),
    [anon_sym_JSR2kr] = ACTIONS(120),
    [anon_sym_STH2kr] = ACTIONS(120),
    [anon_sym_LDZ2kr] = ACTIONS(120),
    [anon_sym_STZ2kr] = ACTIONS(120),
    [anon_sym_LDR2kr] = ACTIONS(120),
    [anon_sym_STR2kr] = ACTIONS(120),
    [anon_sym_LDA2kr] = ACTIONS(120),
    [anon_sym_STA2kr] = ACTIONS(120),
    [anon_sym_DEI2kr] = ACTIONS(120),
    [anon_sym_DEO2kr] = ACTIONS(120),
    [anon_sym_ADD2kr] = ACTIONS(120),
    [anon_sym_SUB2kr] = ACTIONS(120),
    [anon_sym_MUL2kr] = ACTIONS(120),
    [anon_sym_DIV2kr] = ACTIONS(120),
    [anon_sym_AND2kr] = ACTIONS(120),
    [anon_sym_ORA2kr] = ACTIONS(120),
    [anon_sym_EOR2kr] = ACTIONS(120),
    [anon_sym_SFT2kr] = ACTIONS(120),
    [anon_sym_PIPE] = ACTIONS(120),
    [anon_sym_DOLLAR] = ACTIONS(120),
    [anon_sym_POUND] = ACTIONS(120),
    [anon_sym_AT] = ACTIONS(120),
    [anon_sym_COMMA] = ACTIONS(120),
    [anon_sym__] = ACTIONS(120),
    [anon_sym_DOT] = ACTIONS(120),
    [anon_sym_DASH] = ACTIONS(120),
    [anon_sym_SEMI] = ACTIONS(120),
    [anon_sym_EQ] = ACTIONS(120),
    [anon_sym_BANG] = ACTIONS(120),
    [anon_sym_QMARK] = ACTIONS(120),
    [anon_sym_AMP] = ACTIONS(120),
    [anon_sym_LBRACK] = ACTIONS(120),
    [anon_sym_RBRACK] = ACTIONS(120),
    [anon_sym_DQUOTE] = ACTIONS(120),
    [sym_number] = ACTIONS(120),
    [sym_comment] = ACTIONS(3),
  },
  [16] = {
    [ts_builtin_sym_end] = ACTIONS(124),
    [sym_identifier] = ACTIONS(126),
    [anon_sym_PERCENT] = ACTIONS(124),
    [anon_sym_RBRACE] = ACTIONS(124),
    [anon_sym_TILDE] = ACTIONS(124),
    [anon_sym_BRK] = ACTIONS(124),
    [anon_sym_INC] = ACTIONS(126),
    [anon_sym_POP] = ACTIONS(126),
    [anon_sym_NIP] = ACTIONS(126),
    [anon_sym_SWP] = ACTIONS(126),
    [anon_sym_ROT] = ACTIONS(126),
    [anon_sym_DUP] = ACTIONS(126),
    [anon_sym_OVR] = ACTIONS(126),
    [anon_sym_EQU] = ACTIONS(126),
    [anon_sym_NEQ] = ACTIONS(126),
    [anon_sym_GTH] = ACTIONS(126),
    [anon_sym_LTH] = ACTIONS(126),
    [anon_sym_JMP] = ACTIONS(126),
    [anon_sym_JCN] = ACTIONS(126),
    [anon_sym_JSR] = ACTIONS(126),
    [anon_sym_STH] = ACTIONS(126),
    [anon_sym_LDZ] = ACTIONS(126),
    [anon_sym_STZ] = ACTIONS(126),
    [anon_sym_LDR] = ACTIONS(126),
    [anon_sym_STR] = ACTIONS(126),
    [anon_sym_LDA] = ACTIONS(126),
    [anon_sym_STA] = ACTIONS(126),
    [anon_sym_DEI] = ACTIONS(126),
    [anon_sym_DEO] = ACTIONS(126),
    [anon_sym_ADD] = ACTIONS(126),
    [anon_sym_SUB] = ACTIONS(126),
    [anon_sym_MUL] = ACTIONS(126),
    [anon_sym_DIV] = ACTIONS(126),
    [anon_sym_AND] = ACTIONS(126),
    [anon_sym_ORA] = ACTIONS(126),
    [anon_sym_EOR] = ACTIONS(126),
    [anon_sym_SFT] = ACTIONS(126),
    [anon_sym_JCI] = ACTIONS(124),
    [anon_sym_INC2] = ACTIONS(126),
    [anon_sym_POP2] = ACTIONS(126),
    [anon_sym_NIP2] = ACTIONS(126),
    [anon_sym_SWP2] = ACTIONS(126),
    [anon_sym_ROT2] = ACTIONS(126),
    [anon_sym_DUP2] = ACTIONS(126),
    [anon_sym_OVR2] = ACTIONS(126),
    [anon_sym_EQU2] = ACTIONS(126),
    [anon_sym_NEQ2] = ACTIONS(126),
    [anon_sym_GTH2] = ACTIONS(126),
    [anon_sym_LTH2] = ACTIONS(126),
    [anon_sym_JMP2] = ACTIONS(126),
    [anon_sym_JCN2] = ACTIONS(126),
    [anon_sym_JSR2] = ACTIONS(126),
    [anon_sym_STH2] = ACTIONS(126),
    [anon_sym_LDZ2] = ACTIONS(126),
    [anon_sym_STZ2] = ACTIONS(126),
    [anon_sym_LDR2] = ACTIONS(126),
    [anon_sym_STR2] = ACTIONS(126),
    [anon_sym_LDA2] = ACTIONS(126),
    [anon_sym_STA2] = ACTIONS(126),
    [anon_sym_DEI2] = ACTIONS(126),
    [anon_sym_DEO2] = ACTIONS(126),
    [anon_sym_ADD2] = ACTIONS(126),
    [anon_sym_SUB2] = ACTIONS(126),
    [anon_sym_MUL2] = ACTIONS(126),
    [anon_sym_DIV2] = ACTIONS(126),
    [anon_sym_AND2] = ACTIONS(126),
    [anon_sym_ORA2] = ACTIONS(126),
    [anon_sym_EOR2] = ACTIONS(126),
    [anon_sym_SFT2] = ACTIONS(126),
    [anon_sym_JMI] = ACTIONS(124),
    [anon_sym_INCr] = ACTIONS(124),
    [anon_sym_POPr] = ACTIONS(124),
    [anon_sym_NIPr] = ACTIONS(124),
    [anon_sym_SWPr] = ACTIONS(124),
    [anon_sym_ROTr] = ACTIONS(124),
    [anon_sym_DUPr] = ACTIONS(124),
    [anon_sym_OVRr] = ACTIONS(124),
    [anon_sym_EQUr] = ACTIONS(124),
    [anon_sym_NEQr] = ACTIONS(124),
    [anon_sym_GTHr] = ACTIONS(124),
    [anon_sym_LTHr] = ACTIONS(124),
    [anon_sym_JMPr] = ACTIONS(124),
    [anon_sym_JCNr] = ACTIONS(124),
    [anon_sym_JSRr] = ACTIONS(124),
    [anon_sym_STHr] = ACTIONS(124),
    [anon_sym_LDZr] = ACTIONS(124),
    [anon_sym_STZr] = ACTIONS(124),
    [anon_sym_LDRr] = ACTIONS(124),
    [anon_sym_STRr] = ACTIONS(124),
    [anon_sym_LDAr] = ACTIONS(124),
    [anon_sym_STAr] = ACTIONS(124),
    [anon_sym_DEIr] = ACTIONS(124),
    [anon_sym_DEOr] = ACTIONS(124),
    [anon_sym_ADDr] = ACTIONS(124),
    [anon_sym_SUBr] = ACTIONS(124),
    [anon_sym_MULr] = ACTIONS(124),
    [anon_sym_DIVr] = ACTIONS(124),
    [anon_sym_ANDr] = ACTIONS(124),
    [anon_sym_ORAr] = ACTIONS(124),
    [anon_sym_EORr] = ACTIONS(124),
    [anon_sym_SFTr] = ACTIONS(124),
    [anon_sym_JSI] = ACTIONS(124),
    [anon_sym_INC2r] = ACTIONS(124),
    [anon_sym_POP2r] = ACTIONS(124),
    [anon_sym_NIP2r] = ACTIONS(124),
    [anon_sym_SWP2r] = ACTIONS(124),
    [anon_sym_ROT2r] = ACTIONS(124),
    [anon_sym_DUP2r] = ACTIONS(124),
    [anon_sym_OVR2r] = ACTIONS(124),
    [anon_sym_EQU2r] = ACTIONS(124),
    [anon_sym_NEQ2r] = ACTIONS(124),
    [anon_sym_GTH2r] = ACTIONS(124),
    [anon_sym_LTH2r] = ACTIONS(124),
    [anon_sym_JMP2r] = ACTIONS(124),
    [anon_sym_JCN2r] = ACTIONS(124),
    [anon_sym_JSR2r] = ACTIONS(124),
    [anon_sym_STH2r] = ACTIONS(124),
    [anon_sym_LDZ2r] = ACTIONS(124),
    [anon_sym_STZ2r] = ACTIONS(124),
    [anon_sym_LDR2r] = ACTIONS(124),
    [anon_sym_STR2r] = ACTIONS(124),
    [anon_sym_LDA2r] = ACTIONS(124),
    [anon_sym_STA2r] = ACTIONS(124),
    [anon_sym_DEI2r] = ACTIONS(124),
    [anon_sym_DEO2r] = ACTIONS(124),
    [anon_sym_ADD2r] = ACTIONS(124),
    [anon_sym_SUB2r] = ACTIONS(124),
    [anon_sym_MUL2r] = ACTIONS(124),
    [anon_sym_DIV2r] = ACTIONS(124),
    [anon_sym_AND2r] = ACTIONS(124),
    [anon_sym_ORA2r] = ACTIONS(124),
    [anon_sym_EOR2r] = ACTIONS(124),
    [anon_sym_SFT2r] = ACTIONS(124),
    [anon_sym_LIT] = ACTIONS(126),
    [anon_sym_INCk] = ACTIONS(126),
    [anon_sym_POPk] = ACTIONS(126),
    [anon_sym_NIPk] = ACTIONS(126),
    [anon_sym_SWPk] = ACTIONS(126),
    [anon_sym_ROTk] = ACTIONS(126),
    [anon_sym_DUPk] = ACTIONS(126),
    [anon_sym_OVRk] = ACTIONS(126),
    [anon_sym_EQUk] = ACTIONS(126),
    [anon_sym_NEQk] = ACTIONS(126),
    [anon_sym_GTHk] = ACTIONS(126),
    [anon_sym_LTHk] = ACTIONS(126),
    [anon_sym_JMPk] = ACTIONS(126),
    [anon_sym_JCNk] = ACTIONS(126),
    [anon_sym_JSRk] = ACTIONS(126),
    [anon_sym_STHk] = ACTIONS(126),
    [anon_sym_LDZk] = ACTIONS(126),
    [anon_sym_STZk] = ACTIONS(126),
    [anon_sym_LDRk] = ACTIONS(126),
    [anon_sym_STRk] = ACTIONS(126),
    [anon_sym_LDAk] = ACTIONS(126),
    [anon_sym_STAk] = ACTIONS(126),
    [anon_sym_DEIk] = ACTIONS(126),
    [anon_sym_DEOk] = ACTIONS(126),
    [anon_sym_ADDk] = ACTIONS(126),
    [anon_sym_SUBk] = ACTIONS(126),
    [anon_sym_MULk] = ACTIONS(126),
    [anon_sym_DIVk] = ACTIONS(126),
    [anon_sym_ANDk] = ACTIONS(126),
    [anon_sym_ORAk] = ACTIONS(126),
    [anon_sym_EORk] = ACTIONS(126),
    [anon_sym_SFTk] = ACTIONS(126),
    [anon_sym_LIT2] = ACTIONS(126),
    [anon_sym_INC2k] = ACTIONS(126),
    [anon_sym_POP2k] = ACTIONS(126),
    [anon_sym_NIP2k] = ACTIONS(126),
    [anon_sym_SWP2k] = ACTIONS(126),
    [anon_sym_ROT2k] = ACTIONS(126),
    [anon_sym_DUP2k] = ACTIONS(126),
    [anon_sym_OVR2k] = ACTIONS(126),
    [anon_sym_EQU2k] = ACTIONS(126),
    [anon_sym_NEQ2k] = ACTIONS(126),
    [anon_sym_GTH2k] = ACTIONS(126),
    [anon_sym_LTH2k] = ACTIONS(126),
    [anon_sym_JMP2k] = ACTIONS(126),
    [anon_sym_JCN2k] = ACTIONS(126),
    [anon_sym_JSR2k] = ACTIONS(126),
    [anon_sym_STH2k] = ACTIONS(126),
    [anon_sym_LDZ2k] = ACTIONS(126),
    [anon_sym_STZ2k] = ACTIONS(126),
    [anon_sym_LDR2k] = ACTIONS(126),
    [anon_sym_STR2k] = ACTIONS(126),
    [anon_sym_LDA2k] = ACTIONS(126),
    [anon_sym_STA2k] = ACTIONS(126),
    [anon_sym_DEI2k] = ACTIONS(126),
    [anon_sym_DEO2k] = ACTIONS(126),
    [anon_sym_ADD2k] = ACTIONS(126),
    [anon_sym_SUB2k] = ACTIONS(126),
    [anon_sym_MUL2k] = ACTIONS(126),
    [anon_sym_DIV2k] = ACTIONS(126),
    [anon_sym_AND2k] = ACTIONS(126),
    [anon_sym_ORA2k] = ACTIONS(126),
    [anon_sym_EOR2k] = ACTIONS(126),
    [anon_sym_SFT2k] = ACTIONS(126),
    [anon_sym_LITr] = ACTIONS(124),
    [anon_sym_INCkr] = ACTIONS(124),
    [anon_sym_POPkr] = ACTIONS(124),
    [anon_sym_NIPkr] = ACTIONS(124),
    [anon_sym_SWPkr] = ACTIONS(124),
    [anon_sym_ROTkr] = ACTIONS(124),
    [anon_sym_DUPkr] = ACTIONS(124),
    [anon_sym_OVRkr] = ACTIONS(124),
    [anon_sym_EQUkr] = ACTIONS(124),
    [anon_sym_NEQkr] = ACTIONS(124),
    [anon_sym_GTHkr] = ACTIONS(124),
    [anon_sym_LTHkr] = ACTIONS(124),
    [anon_sym_JMPkr] = ACTIONS(124),
    [anon_sym_JCNkr] = ACTIONS(124),
    [anon_sym_JSRkr] = ACTIONS(124),
    [anon_sym_STHkr] = ACTIONS(124),
    [anon_sym_LDZkr] = ACTIONS(124),
    [anon_sym_STZkr] = ACTIONS(124),
    [anon_sym_LDRkr] = ACTIONS(124),
    [anon_sym_STRkr] = ACTIONS(124),
    [anon_sym_LDAkr] = ACTIONS(124),
    [anon_sym_STAkr] = ACTIONS(124),
    [anon_sym_DEIkr] = ACTIONS(124),
    [anon_sym_DEOkr] = ACTIONS(124),
    [anon_sym_ADDkr] = ACTIONS(124),
    [anon_sym_SUBkr] = ACTIONS(124),
    [anon_sym_MULkr] = ACTIONS(124),
    [anon_sym_DIVkr] = ACTIONS(124),
    [anon_sym_ANDkr] = ACTIONS(124),
    [anon_sym_ORAkr] = ACTIONS(124),
    [anon_sym_EORkr] = ACTIONS(124),
    [anon_sym_SFTkr] = ACTIONS(124),
    [anon_sym_LIT2r] = ACTIONS(124),
    [anon_sym_INC2kr] = ACTIONS(124),
    [anon_sym_POP2kr] = ACTIONS(124),
    [anon_sym_NIP2kr] = ACTIONS(124),
    [anon_sym_SWP2kr] = ACTIONS(124),
    [anon_sym_ROT2kr] = ACTIONS(124),
    [anon_sym_DUP2kr] = ACTIONS(124),
    [anon_sym_OVR2kr] = ACTIONS(124),
    [anon_sym_EQU2kr] = ACTIONS(124),
    [anon_sym_NEQ2kr] = ACTIONS(124),
    [anon_sym_GTH2kr] = ACTIONS(124),
    [anon_sym_LTH2kr] = ACTIONS(124),
    [anon_sym_JMP2kr] = ACTIONS(124),
    [anon_sym_JCN2kr] = ACTIONS(124),
    [anon_sym_JSR2kr] = ACTIONS(124),
    [anon_sym_STH2kr] = ACTIONS(124),
    [anon_sym_LDZ2kr] = ACTIONS(124),
    [anon_sym_STZ2kr] = ACTIONS(124),
    [anon_sym_LDR2kr] = ACTIONS(124),
    [anon_sym_STR2kr] = ACTIONS(124),
    [anon_sym_LDA2kr] = ACTIONS(124),
    [anon_sym_STA2kr] = ACTIONS(124),
    [anon_sym_DEI2kr] = ACTIONS(124),
    [anon_sym_DEO2kr] = ACTIONS(124),
    [anon_sym_ADD2kr] = ACTIONS(124),
    [anon_sym_SUB2kr] = ACTIONS(124),
    [anon_sym_MUL2kr] = ACTIONS(124),
    [anon_sym_DIV2kr] = ACTIONS(124),
    [anon_sym_AND2kr] = ACTIONS(124),
    [anon_sym_ORA2kr] = ACTIONS(124),
    [anon_sym_EOR2kr] = ACTIONS(124),
    [anon_sym_SFT2kr] = ACTIONS(124),
    [anon_sym_PIPE] = ACTIONS(124),
    [anon_sym_DOLLAR] = ACTIONS(124),
    [anon_sym_POUND] = ACTIONS(124),
    [anon_sym_AT] = ACTIONS(124),
    [anon_sym_COMMA] = ACTIONS(124),
    [anon_sym__] = ACTIONS(124),
    [anon_sym_DOT] = ACTIONS(124),
    [anon_sym_DASH] = ACTIONS(124),
    [anon_sym_SEMI] = ACTIONS(124),
    [anon_sym_EQ] = ACTIONS(124),
    [anon_sym_BANG] = ACTIONS(124),
    [anon_sym_QMARK] = ACTIONS(124),
    [anon_sym_AMP] = ACTIONS(124),
    [anon_sym_LBRACK] = ACTIONS(124),
    [anon_sym_RBRACK] = ACTIONS(124),
    [anon_sym_DQUOTE] = ACTIONS(124),
    [sym_number] = ACTIONS(124),
    [sym_comment] = ACTIONS(3),
  },
  [17] = {
    [ts_builtin_sym_end] = ACTIONS(128),
    [sym_identifier] = ACTIONS(130),
    [anon_sym_PERCENT] = ACTIONS(128),
    [anon_sym_RBRACE] = ACTIONS(128),
    [anon_sym_TILDE] = ACTIONS(128),
    [anon_sym_BRK] = ACTIONS(128),
    [anon_sym_INC] = ACTIONS(130),
    [anon_sym_POP] = ACTIONS(130),
    [anon_sym_NIP] = ACTIONS(130),
    [anon_sym_SWP] = ACTIONS(130),
    [anon_sym_ROT] = ACTIONS(130),
    [anon_sym_DUP] = ACTIONS(130),
    [anon_sym_OVR] = ACTIONS(130),
    [anon_sym_EQU] = ACTIONS(130),
    [anon_sym_NEQ] = ACTIONS(130),
    [anon_sym_GTH] = ACTIONS(130),
    [anon_sym_LTH] = ACTIONS(130),
    [anon_sym_JMP] = ACTIONS(130),
    [anon_sym_JCN] = ACTIONS(130),
    [anon_sym_JSR] = ACTIONS(130),
    [anon_sym_STH] = ACTIONS(130),
    [anon_sym_LDZ] = ACTIONS(130),
    [anon_sym_STZ] = ACTIONS(130),
    [anon_sym_LDR] = ACTIONS(130),
    [anon_sym_STR] = ACTIONS(130),
    [anon_sym_LDA] = ACTIONS(130),
    [anon_sym_STA] = ACTIONS(130),
    [anon_sym_DEI] = ACTIONS(130),
    [anon_sym_DEO] = ACTIONS(130),
    [anon_sym_ADD] = ACTIONS(130),
    [anon_sym_SUB] = ACTIONS(130),
    [anon_sym_MUL] = ACTIONS(130),
    [anon_sym_DIV] = ACTIONS(130),
    [anon_sym_AND] = ACTIONS(130),
    [anon_sym_ORA] = ACTIONS(130),
    [anon_sym_EOR] = ACTIONS(130),
    [anon_sym_SFT] = ACTIONS(130),
    [anon_sym_JCI] = ACTIONS(128),
    [anon_sym_INC2] = ACTIONS(130),
    [anon_sym_POP2] = ACTIONS(130),
    [anon_sym_NIP2] = ACTIONS(130),
    [anon_sym_SWP2] = ACTIONS(130),
    [anon_sym_ROT2] = ACTIONS(130),
    [anon_sym_DUP2] = ACTIONS(130),
    [anon_sym_OVR2] = ACTIONS(130),
    [anon_sym_EQU2] = ACTIONS(130),
    [anon_sym_NEQ2] = ACTIONS(130),
    [anon_sym_GTH2] = ACTIONS(130),
    [anon_sym_LTH2] = ACTIONS(130),
    [anon_sym_JMP2] = ACTIONS(130),
    [anon_sym_JCN2] = ACTIONS(130),
    [anon_sym_JSR2] = ACTIONS(130),
    [anon_sym_STH2] = ACTIONS(130),
    [anon_sym_LDZ2] = ACTIONS(130),
    [anon_sym_STZ2] = ACTIONS(130),
    [anon_sym_LDR2] = ACTIONS(130),
    [anon_sym_STR2] = ACTIONS(130),
    [anon_sym_LDA2] = ACTIONS(130),
    [anon_sym_STA2] = ACTIONS(130),
    [anon_sym_DEI2] = ACTIONS(130),
    [anon_sym_DEO2] = ACTIONS(130),
    [anon_sym_ADD2] = ACTIONS(130),
    [anon_sym_SUB2] = ACTIONS(130),
    [anon_sym_MUL2] = ACTIONS(130),
    [anon_sym_DIV2] = ACTIONS(130),
    [anon_sym_AND2] = ACTIONS(130),
    [anon_sym_ORA2] = ACTIONS(130),
    [anon_sym_EOR2] = ACTIONS(130),
    [anon_sym_SFT2] = ACTIONS(130),
    [anon_sym_JMI] = ACTIONS(128),
    [anon_sym_INCr] = ACTIONS(128),
    [anon_sym_POPr] = ACTIONS(128),
    [anon_sym_NIPr] = ACTIONS(128),
    [anon_sym_SWPr] = ACTIONS(128),
    [anon_sym_ROTr] = ACTIONS(128),
    [anon_sym_DUPr] = ACTIONS(128),
    [anon_sym_OVRr] = ACTIONS(128),
    [anon_sym_EQUr] = ACTIONS(128),
    [anon_sym_NEQr] = ACTIONS(128),
    [anon_sym_GTHr] = ACTIONS(128),
    [anon_sym_LTHr] = ACTIONS(128),
    [anon_sym_JMPr] = ACTIONS(128),
    [anon_sym_JCNr] = ACTIONS(128),
    [anon_sym_JSRr] = ACTIONS(128),
    [anon_sym_STHr] = ACTIONS(128),
    [anon_sym_LDZr] = ACTIONS(128),
    [anon_sym_STZr] = ACTIONS(128),
    [anon_sym_LDRr] = ACTIONS(128),
    [anon_sym_STRr] = ACTIONS(128),
    [anon_sym_LDAr] = ACTIONS(128),
    [anon_sym_STAr] = ACTIONS(128),
    [anon_sym_DEIr] = ACTIONS(128),
    [anon_sym_DEOr] = ACTIONS(128),
    [anon_sym_ADDr] = ACTIONS(128),
    [anon_sym_SUBr] = ACTIONS(128),
    [anon_sym_MULr] = ACTIONS(128),
    [anon_sym_DIVr] = ACTIONS(128),
    [anon_sym_ANDr] = ACTIONS(128),
    [anon_sym_ORAr] = ACTIONS(128),
    [anon_sym_EORr] = ACTIONS(128),
    [anon_sym_SFTr] = ACTIONS(128),
    [anon_sym_JSI] = ACTIONS(128),
    [anon_sym_INC2r] = ACTIONS(128),
    [anon_sym_POP2r] = ACTIONS(128),
    [anon_sym_NIP2r] = ACTIONS(128),
    [anon_sym_SWP2r] = ACTIONS(128),
    [anon_sym_ROT2r] = ACTIONS(128),
    [anon_sym_DUP2r] = ACTIONS(128),
    [anon_sym_OVR2r] = ACTIONS(128),
    [anon_sym_EQU2r] = ACTIONS(128),
    [anon_sym_NEQ2r] = ACTIONS(128),
    [anon_sym_GTH2r] = ACTIONS(128),
    [anon_sym_LTH2r] = ACTIONS(128),
    [anon_sym_JMP2r] = ACTIONS(128),
    [anon_sym_JCN2r] = ACTIONS(128),
    [anon_sym_JSR2r] = ACTIONS(128),
    [anon_sym_STH2r] = ACTIONS(128),
    [anon_sym_LDZ2r] = ACTIONS(128),
    [anon_sym_STZ2r] = ACTIONS(128),
    [anon_sym_LDR2r] = ACTIONS(128),
    [anon_sym_STR2r] = ACTIONS(128),
    [anon_sym_LDA2r] = ACTIONS(128),
    [anon_sym_STA2r] = ACTIONS(128),
    [anon_sym_DEI2r] = ACTIONS(128),
    [anon_sym_DEO2r] = ACTIONS(128),
    [anon_sym_ADD2r] = ACTIONS(128),
    [anon_sym_SUB2r] = ACTIONS(128),
    [anon_sym_MUL2r] = ACTIONS(128),
    [anon_sym_DIV2r] = ACTIONS(128),
    [anon_sym_AND2r] = ACTIONS(128),
    [anon_sym_ORA2r] = ACTIONS(128),
    [anon_sym_EOR2r] = ACTIONS(128),
    [anon_sym_SFT2r] = ACTIONS(128),
    [anon_sym_LIT] = ACTIONS(130),
    [anon_sym_INCk] = ACTIONS(130),
    [anon_sym_POPk] = ACTIONS(130),
    [anon_sym_NIPk] = ACTIONS(130),
    [anon_sym_SWPk] = ACTIONS(130),
    [anon_sym_ROTk] = ACTIONS(130),
    [anon_sym_DUPk] = ACTIONS(130),
    [anon_sym_OVRk] = ACTIONS(130),
    [anon_sym_EQUk] = ACTIONS(130),
    [anon_sym_NEQk] = ACTIONS(130),
    [anon_sym_GTHk] = ACTIONS(130),
    [anon_sym_LTHk] = ACTIONS(130),
    [anon_sym_JMPk] = ACTIONS(130),
    [anon_sym_JCNk] = ACTIONS(130),
    [anon_sym_JSRk] = ACTIONS(130),
    [anon_sym_STHk] = ACTIONS(130),
    [anon_sym_LDZk] = ACTIONS(130),
    [anon_sym_STZk] = ACTIONS(130),
    [anon_sym_LDRk] = ACTIONS(130),
    [anon_sym_STRk] = ACTIONS(130),
    [anon_sym_LDAk] = ACTIONS(130),
    [anon_sym_STAk] = ACTIONS(130),
    [anon_sym_DEIk] = ACTIONS(130),
    [anon_sym_DEOk] = ACTIONS(130),
    [anon_sym_ADDk] = ACTIONS(130),
    [anon_sym_SUBk] = ACTIONS(130),
    [anon_sym_MULk] = ACTIONS(130),
    [anon_sym_DIVk] = ACTIONS(130),
    [anon_sym_ANDk] = ACTIONS(130),
    [anon_sym_ORAk] = ACTIONS(130),
    [anon_sym_EORk] = ACTIONS(130),
    [anon_sym_SFTk] = ACTIONS(130),
    [anon_sym_LIT2] = ACTIONS(130),
    [anon_sym_INC2k] = ACTIONS(130),
    [anon_sym_POP2k] = ACTIONS(130),
    [anon_sym_NIP2k] = ACTIONS(130),
    [anon_sym_SWP2k] = ACTIONS(130),
    [anon_sym_ROT2k] = ACTIONS(130),
    [anon_sym_DUP2k] = ACTIONS(130),
    [anon_sym_OVR2k] = ACTIONS(130),
    [anon_sym_EQU2k] = ACTIONS(130),
    [anon_sym_NEQ2k] = ACTIONS(130),
    [anon_sym_GTH2k] = ACTIONS(130),
    [anon_sym_LTH2k] = ACTIONS(130),
    [anon_sym_JMP2k] = ACTIONS(130),
    [anon_sym_JCN2k] = ACTIONS(130),
    [anon_sym_JSR2k] = ACTIONS(130),
    [anon_sym_STH2k] = ACTIONS(130),
    [anon_sym_LDZ2k] = ACTIONS(130),
    [anon_sym_STZ2k] = ACTIONS(130),
    [anon_sym_LDR2k] = ACTIONS(130),
    [anon_sym_STR2k] = ACTIONS(130),
    [anon_sym_LDA2k] = ACTIONS(130),
    [anon_sym_STA2k] = ACTIONS(130),
    [anon_sym_DEI2k] = ACTIONS(130),
    [anon_sym_DEO2k] = ACTIONS(130),
    [anon_sym_ADD2k] = ACTIONS(130),
    [anon_sym_SUB2k] = ACTIONS(130),
    [anon_sym_MUL2k] = ACTIONS(130),
    [anon_sym_DIV2k] = ACTIONS(130),
    [anon_sym_AND2k] = ACTIONS(130),
    [anon_sym_ORA2k] = ACTIONS(130),
    [anon_sym_EOR2k] = ACTIONS(130),
    [anon_sym_SFT2k] = ACTIONS(130),
    [anon_sym_LITr] = ACTIONS(128),
    [anon_sym_INCkr] = ACTIONS(128),
    [anon_sym_POPkr] = ACTIONS(128),
    [anon_sym_NIPkr] = ACTIONS(128),
    [anon_sym_SWPkr] = ACTIONS(128),
    [anon_sym_ROTkr] = ACTIONS(128),
    [anon_sym_DUPkr] = ACTIONS(128),
    [anon_sym_OVRkr] = ACTIONS(128),
    [anon_sym_EQUkr] = ACTIONS(128),
    [anon_sym_NEQkr] = ACTIONS(128),
    [anon_sym_GTHkr] = ACTIONS(128),
    [anon_sym_LTHkr] = ACTIONS(128),
    [anon_sym_JMPkr] = ACTIONS(128),
    [anon_sym_JCNkr] = ACTIONS(128),
    [anon_sym_JSRkr] = ACTIONS(128),
    [anon_sym_STHkr] = ACTIONS(128),
    [anon_sym_LDZkr] = ACTIONS(128),
    [anon_sym_STZkr] = ACTIONS(128),
    [anon_sym_LDRkr] = ACTIONS(128),
    [anon_sym_STRkr] = ACTIONS(128),
    [anon_sym_LDAkr] = ACTIONS(128),
    [anon_sym_STAkr] = ACTIONS(128),
    [anon_sym_DEIkr] = ACTIONS(128),
    [anon_sym_DEOkr] = ACTIONS(128),
    [anon_sym_ADDkr] = ACTIONS(128),
    [anon_sym_SUBkr] = ACTIONS(128),
    [anon_sym_MULkr] = ACTIONS(128),
    [anon_sym_DIVkr] = ACTIONS(128),
    [anon_sym_ANDkr] = ACTIONS(128),
    [anon_sym_ORAkr] = ACTIONS(128),
    [anon_sym_EORkr] = ACTIONS(128),
    [anon_sym_SFTkr] = ACTIONS(128),
    [anon_sym_LIT2r] = ACTIONS(128),
    [anon_sym_INC2kr] = ACTIONS(128),
    [anon_sym_POP2kr] = ACTIONS(128),
    [anon_sym_NIP2kr] = ACTIONS(128),
    [anon_sym_SWP2kr] = ACTIONS(128),
    [anon_sym_ROT2kr] = ACTIONS(128),
    [anon_sym_DUP2kr] = ACTIONS(128),
    [anon_sym_OVR2kr] = ACTIONS(128),
    [anon_sym_EQU2kr] = ACTIONS(128),
    [anon_sym_NEQ2kr] = ACTIONS(128),
    [anon_sym_GTH2kr] = ACTIONS(128),
    [anon_sym_LTH2kr] = ACTIONS(128),
    [anon_sym_JMP2kr] = ACTIONS(128),
    [anon_sym_JCN2kr] = ACTIONS(128),
    [anon_sym_JSR2kr] = ACTIONS(128),
    [anon_sym_STH2kr] = ACTIONS(128),
    [anon_sym_LDZ2kr] = ACTIONS(128),
    [anon_sym_STZ2kr] = ACTIONS(128),
    [anon_sym_LDR2kr] = ACTIONS(128),
    [anon_sym_STR2kr] = ACTIONS(128),
    [anon_sym_LDA2kr] = ACTIONS(128),
    [anon_sym_STA2kr] = ACTIONS(128),
    [anon_sym_DEI2kr] = ACTIONS(128),
    [anon_sym_DEO2kr] = ACTIONS(128),
    [anon_sym_ADD2kr] = ACTIONS(128),
    [anon_sym_SUB2kr] = ACTIONS(128),
    [anon_sym_MUL2kr] = ACTIONS(128),
    [anon_sym_DIV2kr] = ACTIONS(128),
    [anon_sym_AND2kr] = ACTIONS(128),
    [anon_sym_ORA2kr] = ACTIONS(128),
    [anon_sym_EOR2kr] = ACTIONS(128),
    [anon_sym_SFT2kr] = ACTIONS(128),
    [anon_sym_PIPE] = ACTIONS(128),
    [anon_sym_DOLLAR] = ACTIONS(128),
    [anon_sym_POUND] = ACTIONS(128),
    [anon_sym_AT] = ACTIONS(128),
    [anon_sym_COMMA] = ACTIONS(128),
    [anon_sym__] = ACTIONS(128),
    [anon_sym_DOT] = ACTIONS(128),
    [anon_sym_DASH] = ACTIONS(128),
    [anon_sym_SEMI] = ACTIONS(128),
    [anon_sym_EQ] = ACTIONS(128),
    [anon_sym_BANG] = ACTIONS(128),
    [anon_sym_QMARK] = ACTIONS(128),
    [anon_sym_AMP] = ACTIONS(128),
    [anon_sym_LBRACK] = ACTIONS(128),
    [anon_sym_RBRACK] = ACTIONS(128),
    [anon_sym_DQUOTE] = ACTIONS(128),
    [sym_number] = ACTIONS(128),
    [sym_comment] = ACTIONS(3),
  },
  [18] = {
    [ts_builtin_sym_end] = ACTIONS(132),
    [sym_identifier] = ACTIONS(134),
    [anon_sym_PERCENT] = ACTIONS(132),
    [anon_sym_RBRACE] = ACTIONS(132),
    [anon_sym_TILDE] = ACTIONS(132),
    [anon_sym_BRK] = ACTIONS(132),
    [anon_sym_INC] = ACTIONS(134),
    [anon_sym_POP] = ACTIONS(134),
    [anon_sym_NIP] = ACTIONS(134),
    [anon_sym_SWP] = ACTIONS(134),
    [anon_sym_ROT] = ACTIONS(134),
    [anon_sym_DUP] = ACTIONS(134),
    [anon_sym_OVR] = ACTIONS(134),
    [anon_sym_EQU] = ACTIONS(134),
    [anon_sym_NEQ] = ACTIONS(134),
    [anon_sym_GTH] = ACTIONS(134),
    [anon_sym_LTH] = ACTIONS(134),
    [anon_sym_JMP] = ACTIONS(134),
    [anon_sym_JCN] = ACTIONS(134),
    [anon_sym_JSR] = ACTIONS(134),
    [anon_sym_STH] = ACTIONS(134),
    [anon_sym_LDZ] = ACTIONS(134),
    [anon_sym_STZ] = ACTIONS(134),
    [anon_sym_LDR] = ACTIONS(134),
    [anon_sym_STR] = ACTIONS(134),
    [anon_sym_LDA] = ACTIONS(134),
    [anon_sym_STA] = ACTIONS(134),
    [anon_sym_DEI] = ACTIONS(134),
    [anon_sym_DEO] = ACTIONS(134),
    [anon_sym_ADD] = ACTIONS(134),
    [anon_sym_SUB] = ACTIONS(134),
    [anon_sym_MUL] = ACTIONS(134),
    [anon_sym_DIV] = ACTIONS(134),
    [anon_sym_AND] = ACTIONS(134),
    [anon_sym_ORA] = ACTIONS(134),
    [anon_sym_EOR] = ACTIONS(134),
    [anon_sym_SFT] = ACTIONS(134),
    [anon_sym_JCI] = ACTIONS(132),
    [anon_sym_INC2] = ACTIONS(134),
    [anon_sym_POP2] = ACTIONS(134),
    [anon_sym_NIP2] = ACTIONS(134),
    [anon_sym_SWP2] = ACTIONS(134),
    [anon_sym_ROT2] = ACTIONS(134),
    [anon_sym_DUP2] = ACTIONS(134),
    [anon_sym_OVR2] = ACTIONS(134),
    [anon_sym_EQU2] = ACTIONS(134),
    [anon_sym_NEQ2] = ACTIONS(134),
    [anon_sym_GTH2] = ACTIONS(134),
    [anon_sym_LTH2] = ACTIONS(134),
    [anon_sym_JMP2] = ACTIONS(134),
    [anon_sym_JCN2] = ACTIONS(134),
    [anon_sym_JSR2] = ACTIONS(134),
    [anon_sym_STH2] = ACTIONS(134),
    [anon_sym_LDZ2] = ACTIONS(134),
    [anon_sym_STZ2] = ACTIONS(134),
    [anon_sym_LDR2] = ACTIONS(134),
    [anon_sym_STR2] = ACTIONS(134),
    [anon_sym_LDA2] = ACTIONS(134),
    [anon_sym_STA2] = ACTIONS(134),
    [anon_sym_DEI2] = ACTIONS(134),
    [anon_sym_DEO2] = ACTIONS(134),
    [anon_sym_ADD2] = ACTIONS(134),
    [anon_sym_SUB2] = ACTIONS(134),
    [anon_sym_MUL2] = ACTIONS(134),
    [anon_sym_DIV2] = ACTIONS(134),
    [anon_sym_AND2] = ACTIONS(134),
    [anon_sym_ORA2] = ACTIONS(134),
    [anon_sym_EOR2] = ACTIONS(134),
    [anon_sym_SFT2] = ACTIONS(134),
    [anon_sym_JMI] = ACTIONS(132),
    [anon_sym_INCr] = ACTIONS(132),
    [anon_sym_POPr] = ACTIONS(132),
    [anon_sym_NIPr] = ACTIONS(132),
    [anon_sym_SWPr] = ACTIONS(132),
    [anon_sym_ROTr] = ACTIONS(132),
    [anon_sym_DUPr] = ACTIONS(132),
    [anon_sym_OVRr] = ACTIONS(132),
    [anon_sym_EQUr] = ACTIONS(132),
    [anon_sym_NEQr] = ACTIONS(132),
    [anon_sym_GTHr] = ACTIONS(132),
    [anon_sym_LTHr] = ACTIONS(132),
    [anon_sym_JMPr] = ACTIONS(132),
    [anon_sym_JCNr] = ACTIONS(132),
    [anon_sym_JSRr] = ACTIONS(132),
    [anon_sym_STHr] = ACTIONS(132),
    [anon_sym_LDZr] = ACTIONS(132),
    [anon_sym_STZr] = ACTIONS(132),
    [anon_sym_LDRr] = ACTIONS(132),
    [anon_sym_STRr] = ACTIONS(132),
    [anon_sym_LDAr] = ACTIONS(132),
    [anon_sym_STAr] = ACTIONS(132),
    [anon_sym_DEIr] = ACTIONS(132),
    [anon_sym_DEOr] = ACTIONS(132),
    [anon_sym_ADDr] = ACTIONS(132),
    [anon_sym_SUBr] = ACTIONS(132),
    [anon_sym_MULr] = ACTIONS(132),
    [anon_sym_DIVr] = ACTIONS(132),
    [anon_sym_ANDr] = ACTIONS(132),
    [anon_sym_ORAr] = ACTIONS(132),
    [anon_sym_EORr] = ACTIONS(132),
    [anon_sym_SFTr] = ACTIONS(132),
    [anon_sym_JSI] = ACTIONS(132),
    [anon_sym_INC2r] = ACTIONS(132),
    [anon_sym_POP2r] = ACTIONS(132),
    [anon_sym_NIP2r] = ACTIONS(132),
    [anon_sym_SWP2r] = ACTIONS(132),
    [anon_sym_ROT2r] = ACTIONS(132),
    [anon_sym_DUP2r] = ACTIONS(132),
    [anon_sym_OVR2r] = ACTIONS(132),
    [anon_sym_EQU2r] = ACTIONS(132),
    [anon_sym_NEQ2r] = ACTIONS(132),
    [anon_sym_GTH2r] = ACTIONS(132),
    [anon_sym_LTH2r] = ACTIONS(132),
    [anon_sym_JMP2r] = ACTIONS(132),
    [anon_sym_JCN2r] = ACTIONS(132),
    [anon_sym_JSR2r] = ACTIONS(132),
    [anon_sym_STH2r] = ACTIONS(132),
    [anon_sym_LDZ2r] = ACTIONS(132),
    [anon_sym_STZ2r] = ACTIONS(132),
    [anon_sym_LDR2r] = ACTIONS(132),
    [anon_sym_STR2r] = ACTIONS(132),
    [anon_sym_LDA2r] = ACTIONS(132),
    [anon_sym_STA2r] = ACTIONS(132),
    [anon_sym_DEI2r] = ACTIONS(132),
    [anon_sym_DEO2r] = ACTIONS(132),
    [anon_sym_ADD2r] = ACTIONS(132),
    [anon_sym_SUB2r] = ACTIONS(132),
    [anon_sym_MUL2r] = ACTIONS(132),
    [anon_sym_DIV2r] = ACTIONS(132),
    [anon_sym_AND2r] = ACTIONS(132),
    [anon_sym_ORA2r] = ACTIONS(132),
    [anon_sym_EOR2r] = ACTIONS(132),
    [anon_sym_SFT2r] = ACTIONS(132),
    [anon_sym_LIT] = ACTIONS(134),
    [anon_sym_INCk] = ACTIONS(134),
    [anon_sym_POPk] = ACTIONS(134),
    [anon_sym_NIPk] = ACTIONS(134),
    [anon_sym_SWPk] = ACTIONS(134),
    [anon_sym_ROTk] = ACTIONS(134),
    [anon_sym_DUPk] = ACTIONS(134),
    [anon_sym_OVRk] = ACTIONS(134),
    [anon_sym_EQUk] = ACTIONS(134),
    [anon_sym_NEQk] = ACTIONS(134),
    [anon_sym_GTHk] = ACTIONS(134),
    [anon_sym_LTHk] = ACTIONS(134),
    [anon_sym_JMPk] = ACTIONS(134),
    [anon_sym_JCNk] = ACTIONS(134),
    [anon_sym_JSRk] = ACTIONS(134),
    [anon_sym_STHk] = ACTIONS(134),
    [anon_sym_LDZk] = ACTIONS(134),
    [anon_sym_STZk] = ACTIONS(134),
    [anon_sym_LDRk] = ACTIONS(134),
    [anon_sym_STRk] = ACTIONS(134),
    [anon_sym_LDAk] = ACTIONS(134),
    [anon_sym_STAk] = ACTIONS(134),
    [anon_sym_DEIk] = ACTIONS(134),
    [anon_sym_DEOk] = ACTIONS(134),
    [anon_sym_ADDk] = ACTIONS(134),
    [anon_sym_SUBk] = ACTIONS(134),
    [anon_sym_MULk] = ACTIONS(134),
    [anon_sym_DIVk] = ACTIONS(134),
    [anon_sym_ANDk] = ACTIONS(134),
    [anon_sym_ORAk] = ACTIONS(134),
    [anon_sym_EORk] = ACTIONS(134),
    [anon_sym_SFTk] = ACTIONS(134),
    [anon_sym_LIT2] = ACTIONS(134),
    [anon_sym_INC2k] = ACTIONS(134),
    [anon_sym_POP2k] = ACTIONS(134),
    [anon_sym_NIP2k] = ACTIONS(134),
    [anon_sym_SWP2k] = ACTIONS(134),
    [anon_sym_ROT2k] = ACTIONS(134),
    [anon_sym_DUP2k] = ACTIONS(134),
    [anon_sym_OVR2k] = ACTIONS(134),
    [anon_sym_EQU2k] = ACTIONS(134),
    [anon_sym_NEQ2k] = ACTIONS(134),
    [anon_sym_GTH2k] = ACTIONS(134),
    [anon_sym_LTH2k] = ACTIONS(134),
    [anon_sym_JMP2k] = ACTIONS(134),
    [anon_sym_JCN2k] = ACTIONS(134),
    [anon_sym_JSR2k] = ACTIONS(134),
    [anon_sym_STH2k] = ACTIONS(134),
    [anon_sym_LDZ2k] = ACTIONS(134),
    [anon_sym_STZ2k] = ACTIONS(134),
    [anon_sym_LDR2k] = ACTIONS(134),
    [anon_sym_STR2k] = ACTIONS(134),
    [anon_sym_LDA2k] = ACTIONS(134),
    [anon_sym_STA2k] = ACTIONS(134),
    [anon_sym_DEI2k] = ACTIONS(134),
    [anon_sym_DEO2k] = ACTIONS(134),
    [anon_sym_ADD2k] = ACTIONS(134),
    [anon_sym_SUB2k] = ACTIONS(134),
    [anon_sym_MUL2k] = ACTIONS(134),
    [anon_sym_DIV2k] = ACTIONS(134),
    [anon_sym_AND2k] = ACTIONS(134),
    [anon_sym_ORA2k] = ACTIONS(134),
    [anon_sym_EOR2k] = ACTIONS(134),
    [anon_sym_SFT2k] = ACTIONS(134),
    [anon_sym_LITr] = ACTIONS(132),
    [anon_sym_INCkr] = ACTIONS(132),
    [anon_sym_POPkr] = ACTIONS(132),
    [anon_sym_NIPkr] = ACTIONS(132),
    [anon_sym_SWPkr] = ACTIONS(132),
    [anon_sym_ROTkr] = ACTIONS(132),
    [anon_sym_DUPkr] = ACTIONS(132),
    [anon_sym_OVRkr] = ACTIONS(132),
    [anon_sym_EQUkr] = ACTIONS(132),
    [anon_sym_NEQkr] = ACTIONS(132),
    [anon_sym_GTHkr] = ACTIONS(132),
    [anon_sym_LTHkr] = ACTIONS(132),
    [anon_sym_JMPkr] = ACTIONS(132),
    [anon_sym_JCNkr] = ACTIONS(132),
    [anon_sym_JSRkr] = ACTIONS(132),
    [anon_sym_STHkr] = ACTIONS(132),
    [anon_sym_LDZkr] = ACTIONS(132),
    [anon_sym_STZkr] = ACTIONS(132),
    [anon_sym_LDRkr] = ACTIONS(132),
    [anon_sym_STRkr] = ACTIONS(132),
    [anon_sym_LDAkr] = ACTIONS(132),
    [anon_sym_STAkr] = ACTIONS(132),
    [anon_sym_DEIkr] = ACTIONS(132),
    [anon_sym_DEOkr] = ACTIONS(132),
    [anon_sym_ADDkr] = ACTIONS(132),
    [anon_sym_SUBkr] = ACTIONS(132),
    [anon_sym_MULkr] = ACTIONS(132),
    [anon_sym_DIVkr] = ACTIONS(132),
    [anon_sym_ANDkr] = ACTIONS(132),
    [anon_sym_ORAkr] = ACTIONS(132),
    [anon_sym_EORkr] = ACTIONS(132),
    [anon_sym_SFTkr] = ACTIONS(132),
    [anon_sym_LIT2r] = ACTIONS(132),
    [anon_sym_INC2kr] = ACTIONS(132),
    [anon_sym_POP2kr] = ACTIONS(132),
    [anon_sym_NIP2kr] = ACTIONS(132),
    [anon_sym_SWP2kr] = ACTIONS(132),
    [anon_sym_ROT2kr] = ACTIONS(132),
    [anon_sym_DUP2kr] = ACTIONS(132),
    [anon_sym_OVR2kr] = ACTIONS(132),
    [anon_sym_EQU2kr] = ACTIONS(132),
    [anon_sym_NEQ2kr] = ACTIONS(132),
    [anon_sym_GTH2kr] = ACTIONS(132),
    [anon_sym_LTH2kr] = ACTIONS(132),
    [anon_sym_JMP2kr] = ACTIONS(132),
    [anon_sym_JCN2kr] = ACTIONS(132),
    [anon_sym_JSR2kr] = ACTIONS(132),
    [anon_sym_STH2kr] = ACTIONS(132),
    [anon_sym_LDZ2kr] = ACTIONS(132),
    [anon_sym_STZ2kr] = ACTIONS(132),
    [anon_sym_LDR2kr] = ACTIONS(132),
    [anon_sym_STR2kr] = ACTIONS(132),
    [anon_sym_LDA2kr] = ACTIONS(132),
    [anon_sym_STA2kr] = ACTIONS(132),
    [anon_sym_DEI2kr] = ACTIONS(132),
    [anon_sym_DEO2kr] = ACTIONS(132),
    [anon_sym_ADD2kr] = ACTIONS(132),
    [anon_sym_SUB2kr] = ACTIONS(132),
    [anon_sym_MUL2kr] = ACTIONS(132),
    [anon_sym_DIV2kr] = ACTIONS(132),
    [anon_sym_AND2kr] = ACTIONS(132),
    [anon_sym_ORA2kr] = ACTIONS(132),
    [anon_sym_EOR2kr] = ACTIONS(132),
    [anon_sym_SFT2kr] = ACTIONS(132),
    [anon_sym_PIPE] = ACTIONS(132),
    [anon_sym_DOLLAR] = ACTIONS(132),
    [anon_sym_POUND] = ACTIONS(132),
    [anon_sym_AT] = ACTIONS(132),
    [anon_sym_COMMA] = ACTIONS(132),
    [anon_sym__] = ACTIONS(132),
    [anon_sym_DOT] = ACTIONS(132),
    [anon_sym_DASH] = ACTIONS(132),
    [anon_sym_SEMI] = ACTIONS(132),
    [anon_sym_EQ] = ACTIONS(132),
    [anon_sym_BANG] = ACTIONS(132),
    [anon_sym_QMARK] = ACTIONS(132),
    [anon_sym_AMP] = ACTIONS(132),
    [anon_sym_LBRACK] = ACTIONS(132),
    [anon_sym_RBRACK] = ACTIONS(132),
    [anon_sym_DQUOTE] = ACTIONS(132),
    [sym_number] = ACTIONS(132),
    [sym_comment] = ACTIONS(3),
  },
  [19] = {
    [ts_builtin_sym_end] = ACTIONS(136),
    [sym_identifier] = ACTIONS(138),
    [anon_sym_PERCENT] = ACTIONS(136),
    [anon_sym_RBRACE] = ACTIONS(136),
    [anon_sym_TILDE] = ACTIONS(136),
    [anon_sym_BRK] = ACTIONS(136),
    [anon_sym_INC] = ACTIONS(138),
    [anon_sym_POP] = ACTIONS(138),
    [anon_sym_NIP] = ACTIONS(138),
    [anon_sym_SWP] = ACTIONS(138),
    [anon_sym_ROT] = ACTIONS(138),
    [anon_sym_DUP] = ACTIONS(138),
    [anon_sym_OVR] = ACTIONS(138),
    [anon_sym_EQU] = ACTIONS(138),
    [anon_sym_NEQ] = ACTIONS(138),
    [anon_sym_GTH] = ACTIONS(138),
    [anon_sym_LTH] = ACTIONS(138),
    [anon_sym_JMP] = ACTIONS(138),
    [anon_sym_JCN] = ACTIONS(138),
    [anon_sym_JSR] = ACTIONS(138),
    [anon_sym_STH] = ACTIONS(138),
    [anon_sym_LDZ] = ACTIONS(138),
    [anon_sym_STZ] = ACTIONS(138),
    [anon_sym_LDR] = ACTIONS(138),
    [anon_sym_STR] = ACTIONS(138),
    [anon_sym_LDA] = ACTIONS(138),
    [anon_sym_STA] = ACTIONS(138),
    [anon_sym_DEI] = ACTIONS(138),
    [anon_sym_DEO] = ACTIONS(138),
    [anon_sym_ADD] = ACTIONS(138),
    [anon_sym_SUB] = ACTIONS(138),
    [anon_sym_MUL] = ACTIONS(138),
    [anon_sym_DIV] = ACTIONS(138),
    [anon_sym_AND] = ACTIONS(138),
    [anon_sym_ORA] = ACTIONS(138),
    [anon_sym_EOR] = ACTIONS(138),
    [anon_sym_SFT] = ACTIONS(138),
    [anon_sym_JCI] = ACTIONS(136),
    [anon_sym_INC2] = ACTIONS(138),
    [anon_sym_POP2] = ACTIONS(138),
    [anon_sym_NIP2] = ACTIONS(138),
    [anon_sym_SWP2] = ACTIONS(138),
    [anon_sym_ROT2] = ACTIONS(138),
    [anon_sym_DUP2] = ACTIONS(138),
    [anon_sym_OVR2] = ACTIONS(138),
    [anon_sym_EQU2] = ACTIONS(138),
    [anon_sym_NEQ2] = ACTIONS(138),
    [anon_sym_GTH2] = ACTIONS(138),
    [anon_sym_LTH2] = ACTIONS(138),
    [anon_sym_JMP2] = ACTIONS(138),
    [anon_sym_JCN2] = ACTIONS(138),
    [anon_sym_JSR2] = ACTIONS(138),
    [anon_sym_STH2] = ACTIONS(138),
    [anon_sym_LDZ2] = ACTIONS(138),
    [anon_sym_STZ2] = ACTIONS(138),
    [anon_sym_LDR2] = ACTIONS(138),
    [anon_sym_STR2] = ACTIONS(138),
    [anon_sym_LDA2] = ACTIONS(138),
    [anon_sym_STA2] = ACTIONS(138),
    [anon_sym_DEI2] = ACTIONS(138),
    [anon_sym_DEO2] = ACTIONS(138),
    [anon_sym_ADD2] = ACTIONS(138),
    [anon_sym_SUB2] = ACTIONS(138),
    [anon_sym_MUL2] = ACTIONS(138),
    [anon_sym_DIV2] = ACTIONS(138),
    [anon_sym_AND2] = ACTIONS(138),
    [anon_sym_ORA2] = ACTIONS(138),
    [anon_sym_EOR2] = ACTIONS(138),
    [anon_sym_SFT2] = ACTIONS(138),
    [anon_sym_JMI] = ACTIONS(136),
    [anon_sym_INCr] = ACTIONS(136),
    [anon_sym_POPr] = ACTIONS(136),
    [anon_sym_NIPr] = ACTIONS(136),
    [anon_sym_SWPr] = ACTIONS(136),
    [anon_sym_ROTr] = ACTIONS(136),
    [anon_sym_DUPr] = ACTIONS(136),
    [anon_sym_OVRr] = ACTIONS(136),
    [anon_sym_EQUr] = ACTIONS(136),
    [anon_sym_NEQr] = ACTIONS(136),
    [anon_sym_GTHr] = ACTIONS(136),
    [anon_sym_LTHr] = ACTIONS(136),
    [anon_sym_JMPr] = ACTIONS(136),
    [anon_sym_JCNr] = ACTIONS(136),
    [anon_sym_JSRr] = ACTIONS(136),
    [anon_sym_STHr] = ACTIONS(136),
    [anon_sym_LDZr] = ACTIONS(136),
    [anon_sym_STZr] = ACTIONS(136),
    [anon_sym_LDRr] = ACTIONS(136),
    [anon_sym_STRr] = ACTIONS(136),
    [anon_sym_LDAr] = ACTIONS(136),
    [anon_sym_STAr] = ACTIONS(136),
    [anon_sym_DEIr] = ACTIONS(136),
    [anon_sym_DEOr] = ACTIONS(136),
    [anon_sym_ADDr] = ACTIONS(136),
    [anon_sym_SUBr] = ACTIONS(136),
    [anon_sym_MULr] = ACTIONS(136),
    [anon_sym_DIVr] = ACTIONS(136),
    [anon_sym_ANDr] = ACTIONS(136),
    [anon_sym_ORAr] = ACTIONS(136),
    [anon_sym_EORr] = ACTIONS(136),
    [anon_sym_SFTr] = ACTIONS(136),
    [anon_sym_JSI] = ACTIONS(136),
    [anon_sym_INC2r] = ACTIONS(136),
    [anon_sym_POP2r] = ACTIONS(136),
    [anon_sym_NIP2r] = ACTIONS(136),
    [anon_sym_SWP2r] = ACTIONS(136),
    [anon_sym_ROT2r] = ACTIONS(136),
    [anon_sym_DUP2r] = ACTIONS(136),
    [anon_sym_OVR2r] = ACTIONS(136),
    [anon_sym_EQU2r] = ACTIONS(136),
    [anon_sym_NEQ2r] = ACTIONS(136),
    [anon_sym_GTH2r] = ACTIONS(136),
    [anon_sym_LTH2r] = ACTIONS(136),
    [anon_sym_JMP2r] = ACTIONS(136),
    [anon_sym_JCN2r] = ACTIONS(136),
    [anon_sym_JSR2r] = ACTIONS(136),
    [anon_sym_STH2r] = ACTIONS(136),
    [anon_sym_LDZ2r] = ACTIONS(136),
    [anon_sym_STZ2r] = ACTIONS(136),
    [anon_sym_LDR2r] = ACTIONS(136),
    [anon_sym_STR2r] = ACTIONS(136),
    [anon_sym_LDA2r] = ACTIONS(136),
    [anon_sym_STA2r] = ACTIONS(136),
    [anon_sym_DEI2r] = ACTIONS(136),
    [anon_sym_DEO2r] = ACTIONS(136),
    [anon_sym_ADD2r] = ACTIONS(136),
    [anon_sym_SUB2r] = ACTIONS(136),
    [anon_sym_MUL2r] = ACTIONS(136),
    [anon_sym_DIV2r] = ACTIONS(136),
    [anon_sym_AND2r] = ACTIONS(136),
    [anon_sym_ORA2r] = ACTIONS(136),
    [anon_sym_EOR2r] = ACTIONS(136),
    [anon_sym_SFT2r] = ACTIONS(136),
    [anon_sym_LIT] = ACTIONS(138),
    [anon_sym_INCk] = ACTIONS(138),
    [anon_sym_POPk] = ACTIONS(138),
    [anon_sym_NIPk] = ACTIONS(138),
    [anon_sym_SWPk] = ACTIONS(138),
    [anon_sym_ROTk] = ACTIONS(138),
    [anon_sym_DUPk] = ACTIONS(138),
    [anon_sym_OVRk] = ACTIONS(138),
    [anon_sym_EQUk] = ACTIONS(138),
    [anon_sym_NEQk] = ACTIONS(138),
    [anon_sym_GTHk] = ACTIONS(138),
    [anon_sym_LTHk] = ACTIONS(138),
    [anon_sym_JMPk] = ACTIONS(138),
    [anon_sym_JCNk] = ACTIONS(138),
    [anon_sym_JSRk] = ACTIONS(138),
    [anon_sym_STHk] = ACTIONS(138),
    [anon_sym_LDZk] = ACTIONS(138),
    [anon_sym_STZk] = ACTIONS(138),
    [anon_sym_LDRk] = ACTIONS(138),
    [anon_sym_STRk] = ACTIONS(138),
    [anon_sym_LDAk] = ACTIONS(138),
    [anon_sym_STAk] = ACTIONS(138),
    [anon_sym_DEIk] = ACTIONS(138),
    [anon_sym_DEOk] = ACTIONS(138),
    [anon_sym_ADDk] = ACTIONS(138),
    [anon_sym_SUBk] = ACTIONS(138),
    [anon_sym_MULk] = ACTIONS(138),
    [anon_sym_DIVk] = ACTIONS(138),
    [anon_sym_ANDk] = ACTIONS(138),
    [anon_sym_ORAk] = ACTIONS(138),
    [anon_sym_EORk] = ACTIONS(138),
    [anon_sym_SFTk] = ACTIONS(138),
    [anon_sym_LIT2] = ACTIONS(138),
    [anon_sym_INC2k] = ACTIONS(138),
    [anon_sym_POP2k] = ACTIONS(138),
    [anon_sym_NIP2k] = ACTIONS(138),
    [anon_sym_SWP2k] = ACTIONS(138),
    [anon_sym_ROT2k] = ACTIONS(138),
    [anon_sym_DUP2k] = ACTIONS(138),
    [anon_sym_OVR2k] = ACTIONS(138),
    [anon_sym_EQU2k] = ACTIONS(138),
    [anon_sym_NEQ2k] = ACTIONS(138),
    [anon_sym_GTH2k] = ACTIONS(138),
    [anon_sym_LTH2k] = ACTIONS(138),
    [anon_sym_JMP2k] = ACTIONS(138),
    [anon_sym_JCN2k] = ACTIONS(138),
    [anon_sym_JSR2k] = ACTIONS(138),
    [anon_sym_STH2k] = ACTIONS(138),
    [anon_sym_LDZ2k] = ACTIONS(138),
    [anon_sym_STZ2k] = ACTIONS(138),
    [anon_sym_LDR2k] = ACTIONS(138),
    [anon_sym_STR2k] = ACTIONS(138),
    [anon_sym_LDA2k] = ACTIONS(138),
    [anon_sym_STA2k] = ACTIONS(138),
    [anon_sym_DEI2k] = ACTIONS(138),
    [anon_sym_DEO2k] = ACTIONS(138),
    [anon_sym_ADD2k] = ACTIONS(138),
    [anon_sym_SUB2k] = ACTIONS(138),
    [anon_sym_MUL2k] = ACTIONS(138),
    [anon_sym_DIV2k] = ACTIONS(138),
    [anon_sym_AND2k] = ACTIONS(138),
    [anon_sym_ORA2k] = ACTIONS(138),
    [anon_sym_EOR2k] = ACTIONS(138),
    [anon_sym_SFT2k] = ACTIONS(138),
    [anon_sym_LITr] = ACTIONS(136),
    [anon_sym_INCkr] = ACTIONS(136),
    [anon_sym_POPkr] = ACTIONS(136),
    [anon_sym_NIPkr] = ACTIONS(136),
    [anon_sym_SWPkr] = ACTIONS(136),
    [anon_sym_ROTkr] = ACTIONS(136),
    [anon_sym_DUPkr] = ACTIONS(136),
    [anon_sym_OVRkr] = ACTIONS(136),
    [anon_sym_EQUkr] = ACTIONS(136),
    [anon_sym_NEQkr] = ACTIONS(136),
    [anon_sym_GTHkr] = ACTIONS(136),
    [anon_sym_LTHkr] = ACTIONS(136),
    [anon_sym_JMPkr] = ACTIONS(136),
    [anon_sym_JCNkr] = ACTIONS(136),
    [anon_sym_JSRkr] = ACTIONS(136),
    [anon_sym_STHkr] = ACTIONS(136),
    [anon_sym_LDZkr] = ACTIONS(136),
    [anon_sym_STZkr] = ACTIONS(136),
    [anon_sym_LDRkr] = ACTIONS(136),
    [anon_sym_STRkr] = ACTIONS(136),
    [anon_sym_LDAkr] = ACTIONS(136),
    [anon_sym_STAkr] = ACTIONS(136),
    [anon_sym_DEIkr] = ACTIONS(136),
    [anon_sym_DEOkr] = ACTIONS(136),
    [anon_sym_ADDkr] = ACTIONS(136),
    [anon_sym_SUBkr] = ACTIONS(136),
    [anon_sym_MULkr] = ACTIONS(136),
    [anon_sym_DIVkr] = ACTIONS(136),
    [anon_sym_ANDkr] = ACTIONS(136),
    [anon_sym_ORAkr] = ACTIONS(136),
    [anon_sym_EORkr] = ACTIONS(136),
    [anon_sym_SFTkr] = ACTIONS(136),
    [anon_sym_LIT2r] = ACTIONS(136),
    [anon_sym_INC2kr] = ACTIONS(136),
    [anon_sym_POP2kr] = ACTIONS(136),
    [anon_sym_NIP2kr] = ACTIONS(136),
    [anon_sym_SWP2kr] = ACTIONS(136),
    [anon_sym_ROT2kr] = ACTIONS(136),
    [anon_sym_DUP2kr] = ACTIONS(136),
    [anon_sym_OVR2kr] = ACTIONS(136),
    [anon_sym_EQU2kr] = ACTIONS(136),
    [anon_sym_NEQ2kr] = ACTIONS(136),
    [anon_sym_GTH2kr] = ACTIONS(136),
    [anon_sym_LTH2kr] = ACTIONS(136),
    [anon_sym_JMP2kr] = ACTIONS(136),
    [anon_sym_JCN2kr] = ACTIONS(136),
    [anon_sym_JSR2kr] = ACTIONS(136),
    [anon_sym_STH2kr] = ACTIONS(136),
    [anon_sym_LDZ2kr] = ACTIONS(136),
    [anon_sym_STZ2kr] = ACTIONS(136),
    [anon_sym_LDR2kr] = ACTIONS(136),
    [anon_sym_STR2kr] = ACTIONS(136),
    [anon_sym_LDA2kr] = ACTIONS(136),
    [anon_sym_STA2kr] = ACTIONS(136),
    [anon_sym_DEI2kr] = ACTIONS(136),
    [anon_sym_DEO2kr] = ACTIONS(136),
    [anon_sym_ADD2kr] = ACTIONS(136),
    [anon_sym_SUB2kr] = ACTIONS(136),
    [anon_sym_MUL2kr] = ACTIONS(136),
    [anon_sym_DIV2kr] = ACTIONS(136),
    [anon_sym_AND2kr] = ACTIONS(136),
    [anon_sym_ORA2kr] = ACTIONS(136),
    [anon_sym_EOR2kr] = ACTIONS(136),
    [anon_sym_SFT2kr] = ACTIONS(136),
    [anon_sym_PIPE] = ACTIONS(136),
    [anon_sym_DOLLAR] = ACTIONS(136),
    [anon_sym_POUND] = ACTIONS(136),
    [anon_sym_AT] = ACTIONS(136),
    [anon_sym_COMMA] = ACTIONS(136),
    [anon_sym__] = ACTIONS(136),
    [anon_sym_DOT] = ACTIONS(136),
    [anon_sym_DASH] = ACTIONS(136),
    [anon_sym_SEMI] = ACTIONS(136),
    [anon_sym_EQ] = ACTIONS(136),
    [anon_sym_BANG] = ACTIONS(136),
    [anon_sym_QMARK] = ACTIONS(136),
    [anon_sym_AMP] = ACTIONS(136),
    [anon_sym_LBRACK] = ACTIONS(136),
    [anon_sym_RBRACK] = ACTIONS(136),
    [anon_sym_DQUOTE] = ACTIONS(136),
    [sym_number] = ACTIONS(136),
    [sym_comment] = ACTIONS(3),
  },
  [20] = {
    [ts_builtin_sym_end] = ACTIONS(140),
    [sym_identifier] = ACTIONS(142),
    [anon_sym_PERCENT] = ACTIONS(140),
    [anon_sym_RBRACE] = ACTIONS(140),
    [anon_sym_TILDE] = ACTIONS(140),
    [anon_sym_BRK] = ACTIONS(140),
    [anon_sym_INC] = ACTIONS(142),
    [anon_sym_POP] = ACTIONS(142),
    [anon_sym_NIP] = ACTIONS(142),
    [anon_sym_SWP] = ACTIONS(142),
    [anon_sym_ROT] = ACTIONS(142),
    [anon_sym_DUP] = ACTIONS(142),
    [anon_sym_OVR] = ACTIONS(142),
    [anon_sym_EQU] = ACTIONS(142),
    [anon_sym_NEQ] = ACTIONS(142),
    [anon_sym_GTH] = ACTIONS(142),
    [anon_sym_LTH] = ACTIONS(142),
    [anon_sym_JMP] = ACTIONS(142),
    [anon_sym_JCN] = ACTIONS(142),
    [anon_sym_JSR] = ACTIONS(142),
    [anon_sym_STH] = ACTIONS(142),
    [anon_sym_LDZ] = ACTIONS(142),
    [anon_sym_STZ] = ACTIONS(142),
    [anon_sym_LDR] = ACTIONS(142),
    [anon_sym_STR] = ACTIONS(142),
    [anon_sym_LDA] = ACTIONS(142),
    [anon_sym_STA] = ACTIONS(142),
    [anon_sym_DEI] = ACTIONS(142),
    [anon_sym_DEO] = ACTIONS(142),
    [anon_sym_ADD] = ACTIONS(142),
    [anon_sym_SUB] = ACTIONS(142),
    [anon_sym_MUL] = ACTIONS(142),
    [anon_sym_DIV] = ACTIONS(142),
    [anon_sym_AND] = ACTIONS(142),
    [anon_sym_ORA] = ACTIONS(142),
    [anon_sym_EOR] = ACTIONS(142),
    [anon_sym_SFT] = ACTIONS(142),
    [anon_sym_JCI] = ACTIONS(140),
    [anon_sym_INC2] = ACTIONS(142),
    [anon_sym_POP2] = ACTIONS(142),
    [anon_sym_NIP2] = ACTIONS(142),
    [anon_sym_SWP2] = ACTIONS(142),
    [anon_sym_ROT2] = ACTIONS(142),
    [anon_sym_DUP2] = ACTIONS(142),
    [anon_sym_OVR2] = ACTIONS(142),
    [anon_sym_EQU2] = ACTIONS(142),
    [anon_sym_NEQ2] = ACTIONS(142),
    [anon_sym_GTH2] = ACTIONS(142),
    [anon_sym_LTH2] = ACTIONS(142),
    [anon_sym_JMP2] = ACTIONS(142),
    [anon_sym_JCN2] = ACTIONS(142),
    [anon_sym_JSR2] = ACTIONS(142),
    [anon_sym_STH2] = ACTIONS(142),
    [anon_sym_LDZ2] = ACTIONS(142),
    [anon_sym_STZ2] = ACTIONS(142),
    [anon_sym_LDR2] = ACTIONS(142),
    [anon_sym_STR2] = ACTIONS(142),
    [anon_sym_LDA2] = ACTIONS(142),
    [anon_sym_STA2] = ACTIONS(142),
    [anon_sym_DEI2] = ACTIONS(142),
    [anon_sym_DEO2] = ACTIONS(142),
    [anon_sym_ADD2] = ACTIONS(142),
    [anon_sym_SUB2] = ACTIONS(142),
    [anon_sym_MUL2] = ACTIONS(142),
    [anon_sym_DIV2] = ACTIONS(142),
    [anon_sym_AND2] = ACTIONS(142),
    [anon_sym_ORA2] = ACTIONS(142),
    [anon_sym_EOR2] = ACTIONS(142),
    [anon_sym_SFT2] = ACTIONS(142),
    [anon_sym_JMI] = ACTIONS(140),
    [anon_sym_INCr] = ACTIONS(140),
    [anon_sym_POPr] = ACTIONS(140),
    [anon_sym_NIPr] = ACTIONS(140),
    [anon_sym_SWPr] = ACTIONS(140),
    [anon_sym_ROTr] = ACTIONS(140),
    [anon_sym_DUPr] = ACTIONS(140),
    [anon_sym_OVRr] = ACTIONS(140),
    [anon_sym_EQUr] = ACTIONS(140),
    [anon_sym_NEQr] = ACTIONS(140),
    [anon_sym_GTHr] = ACTIONS(140),
    [anon_sym_LTHr] = ACTIONS(140),
    [anon_sym_JMPr] = ACTIONS(140),
    [anon_sym_JCNr] = ACTIONS(140),
    [anon_sym_JSRr] = ACTIONS(140),
    [anon_sym_STHr] = ACTIONS(140),
    [anon_sym_LDZr] = ACTIONS(140),
    [anon_sym_STZr] = ACTIONS(140),
    [anon_sym_LDRr] = ACTIONS(140),
    [anon_sym_STRr] = ACTIONS(140),
    [anon_sym_LDAr] = ACTIONS(140),
    [anon_sym_STAr] = ACTIONS(140),
    [anon_sym_DEIr] = ACTIONS(140),
    [anon_sym_DEOr] = ACTIONS(140),
    [anon_sym_ADDr] = ACTIONS(140),
    [anon_sym_SUBr] = ACTIONS(140),
    [anon_sym_MULr] = ACTIONS(140),
    [anon_sym_DIVr] = ACTIONS(140),
    [anon_sym_ANDr] = ACTIONS(140),
    [anon_sym_ORAr] = ACTIONS(140),
    [anon_sym_EORr] = ACTIONS(140),
    [anon_sym_SFTr] = ACTIONS(140),
    [anon_sym_JSI] = ACTIONS(140),
    [anon_sym_INC2r] = ACTIONS(140),
    [anon_sym_POP2r] = ACTIONS(140),
    [anon_sym_NIP2r] = ACTIONS(140),
    [anon_sym_SWP2r] = ACTIONS(140),
    [anon_sym_ROT2r] = ACTIONS(140),
    [anon_sym_DUP2r] = ACTIONS(140),
    [anon_sym_OVR2r] = ACTIONS(140),
    [anon_sym_EQU2r] = ACTIONS(140),
    [anon_sym_NEQ2r] = ACTIONS(140),
    [anon_sym_GTH2r] = ACTIONS(140),
    [anon_sym_LTH2r] = ACTIONS(140),
    [anon_sym_JMP2r] = ACTIONS(140),
    [anon_sym_JCN2r] = ACTIONS(140),
    [anon_sym_JSR2r] = ACTIONS(140),
    [anon_sym_STH2r] = ACTIONS(140),
    [anon_sym_LDZ2r] = ACTIONS(140),
    [anon_sym_STZ2r] = ACTIONS(140),
    [anon_sym_LDR2r] = ACTIONS(140),
    [anon_sym_STR2r] = ACTIONS(140),
    [anon_sym_LDA2r] = ACTIONS(140),
    [anon_sym_STA2r] = ACTIONS(140),
    [anon_sym_DEI2r] = ACTIONS(140),
    [anon_sym_DEO2r] = ACTIONS(140),
    [anon_sym_ADD2r] = ACTIONS(140),
    [anon_sym_SUB2r] = ACTIONS(140),
    [anon_sym_MUL2r] = ACTIONS(140),
    [anon_sym_DIV2r] = ACTIONS(140),
    [anon_sym_AND2r] = ACTIONS(140),
    [anon_sym_ORA2r] = ACTIONS(140),
    [anon_sym_EOR2r] = ACTIONS(140),
    [anon_sym_SFT2r] = ACTIONS(140),
    [anon_sym_LIT] = ACTIONS(142),
    [anon_sym_INCk] = ACTIONS(142),
    [anon_sym_POPk] = ACTIONS(142),
    [anon_sym_NIPk] = ACTIONS(142),
    [anon_sym_SWPk] = ACTIONS(142),
    [anon_sym_ROTk] = ACTIONS(142),
    [anon_sym_DUPk] = ACTIONS(142),
    [anon_sym_OVRk] = ACTIONS(142),
    [anon_sym_EQUk] = ACTIONS(142),
    [anon_sym_NEQk] = ACTIONS(142),
    [anon_sym_GTHk] = ACTIONS(142),
    [anon_sym_LTHk] = ACTIONS(142),
    [anon_sym_JMPk] = ACTIONS(142),
    [anon_sym_JCNk] = ACTIONS(142),
    [anon_sym_JSRk] = ACTIONS(142),
    [anon_sym_STHk] = ACTIONS(142),
    [anon_sym_LDZk] = ACTIONS(142),
    [anon_sym_STZk] = ACTIONS(142),
    [anon_sym_LDRk] = ACTIONS(142),
    [anon_sym_STRk] = ACTIONS(142),
    [anon_sym_LDAk] = ACTIONS(142),
    [anon_sym_STAk] = ACTIONS(142),
    [anon_sym_DEIk] = ACTIONS(142),
    [anon_sym_DEOk] = ACTIONS(142),
    [anon_sym_ADDk] = ACTIONS(142),
    [anon_sym_SUBk] = ACTIONS(142),
    [anon_sym_MULk] = ACTIONS(142),
    [anon_sym_DIVk] = ACTIONS(142),
    [anon_sym_ANDk] = ACTIONS(142),
    [anon_sym_ORAk] = ACTIONS(142),
    [anon_sym_EORk] = ACTIONS(142),
    [anon_sym_SFTk] = ACTIONS(142),
    [anon_sym_LIT2] = ACTIONS(142),
    [anon_sym_INC2k] = ACTIONS(142),
    [anon_sym_POP2k] = ACTIONS(142),
    [anon_sym_NIP2k] = ACTIONS(142),
    [anon_sym_SWP2k] = ACTIONS(142),
    [anon_sym_ROT2k] = ACTIONS(142),
    [anon_sym_DUP2k] = ACTIONS(142),
    [anon_sym_OVR2k] = ACTIONS(142),
    [anon_sym_EQU2k] = ACTIONS(142),
    [anon_sym_NEQ2k] = ACTIONS(142),
    [anon_sym_GTH2k] = ACTIONS(142),
    [anon_sym_LTH2k] = ACTIONS(142),
    [anon_sym_JMP2k] = ACTIONS(142),
    [anon_sym_JCN2k] = ACTIONS(142),
    [anon_sym_JSR2k] = ACTIONS(142),
    [anon_sym_STH2k] = ACTIONS(142),
    [anon_sym_LDZ2k] = ACTIONS(142),
    [anon_sym_STZ2k] = ACTIONS(142),
    [anon_sym_LDR2k] = ACTIONS(142),
    [anon_sym_STR2k] = ACTIONS(142),
    [anon_sym_LDA2k] = ACTIONS(142),
    [anon_sym_STA2k] = ACTIONS(142),
    [anon_sym_DEI2k] = ACTIONS(142),
    [anon_sym_DEO2k] = ACTIONS(142),
    [anon_sym_ADD2k] = ACTIONS(142),
    [anon_sym_SUB2k] = ACTIONS(142),
    [anon_sym_MUL2k] = ACTIONS(142),
    [anon_sym_DIV2k] = ACTIONS(142),
    [anon_sym_AND2k] = ACTIONS(142),
    [anon_sym_ORA2k] = ACTIONS(142),
    [anon_sym_EOR2k] = ACTIONS(142),
    [anon_sym_SFT2k] = ACTIONS(142),
    [anon_sym_LITr] = ACTIONS(140),
    [anon_sym_INCkr] = ACTIONS(140),
    [anon_sym_POPkr] = ACTIONS(140),
    [anon_sym_NIPkr] = ACTIONS(140),
    [anon_sym_SWPkr] = ACTIONS(140),
    [anon_sym_ROTkr] = ACTIONS(140),
    [anon_sym_DUPkr] = ACTIONS(140),
    [anon_sym_OVRkr] = ACTIONS(140),
    [anon_sym_EQUkr] = ACTIONS(140),
    [anon_sym_NEQkr] = ACTIONS(140),
    [anon_sym_GTHkr] = ACTIONS(140),
    [anon_sym_LTHkr] = ACTIONS(140),
    [anon_sym_JMPkr] = ACTIONS(140),
    [anon_sym_JCNkr] = ACTIONS(140),
    [anon_sym_JSRkr] = ACTIONS(140),
    [anon_sym_STHkr] = ACTIONS(140),
    [anon_sym_LDZkr] = ACTIONS(140),
    [anon_sym_STZkr] = ACTIONS(140),
    [anon_sym_LDRkr] = ACTIONS(140),
    [anon_sym_STRkr] = ACTIONS(140),
    [anon_sym_LDAkr] = ACTIONS(140),
    [anon_sym_STAkr] = ACTIONS(140),
    [anon_sym_DEIkr] = ACTIONS(140),
    [anon_sym_DEOkr] = ACTIONS(140),
    [anon_sym_ADDkr] = ACTIONS(140),
    [anon_sym_SUBkr] = ACTIONS(140),
    [anon_sym_MULkr] = ACTIONS(140),
    [anon_sym_DIVkr] = ACTIONS(140),
    [anon_sym_ANDkr] = ACTIONS(140),
    [anon_sym_ORAkr] = ACTIONS(140),
    [anon_sym_EORkr] = ACTIONS(140),
    [anon_sym_SFTkr] = ACTIONS(140),
    [anon_sym_LIT2r] = ACTIONS(140),
    [anon_sym_INC2kr] = ACTIONS(140),
    [anon_sym_POP2kr] = ACTIONS(140),
    [anon_sym_NIP2kr] = ACTIONS(140),
    [anon_sym_SWP2kr] = ACTIONS(140),
    [anon_sym_ROT2kr] = ACTIONS(140),
    [anon_sym_DUP2kr] = ACTIONS(140),
    [anon_sym_OVR2kr] = ACTIONS(140),
    [anon_sym_EQU2kr] = ACTIONS(140),
    [anon_sym_NEQ2kr] = ACTIONS(140),
    [anon_sym_GTH2kr] = ACTIONS(140),
    [anon_sym_LTH2kr] = ACTIONS(140),
    [anon_sym_JMP2kr] = ACTIONS(140),
    [anon_sym_JCN2kr] = ACTIONS(140),
    [anon_sym_JSR2kr] = ACTIONS(140),
    [anon_sym_STH2kr] = ACTIONS(140),
    [anon_sym_LDZ2kr] = ACTIONS(140),
    [anon_sym_STZ2kr] = ACTIONS(140),
    [anon_sym_LDR2kr] = ACTIONS(140),
    [anon_sym_STR2kr] = ACTIONS(140),
    [anon_sym_LDA2kr] = ACTIONS(140),
    [anon_sym_STA2kr] = ACTIONS(140),
    [anon_sym_DEI2kr] = ACTIONS(140),
    [anon_sym_DEO2kr] = ACTIONS(140),
    [anon_sym_ADD2kr] = ACTIONS(140),
    [anon_sym_SUB2kr] = ACTIONS(140),
    [anon_sym_MUL2kr] = ACTIONS(140),
    [anon_sym_DIV2kr] = ACTIONS(140),
    [anon_sym_AND2kr] = ACTIONS(140),
    [anon_sym_ORA2kr] = ACTIONS(140),
    [anon_sym_EOR2kr] = ACTIONS(140),
    [anon_sym_SFT2kr] = ACTIONS(140),
    [anon_sym_PIPE] = ACTIONS(140),
    [anon_sym_DOLLAR] = ACTIONS(140),
    [anon_sym_POUND] = ACTIONS(140),
    [anon_sym_AT] = ACTIONS(140),
    [anon_sym_COMMA] = ACTIONS(140),
    [anon_sym__] = ACTIONS(140),
    [anon_sym_DOT] = ACTIONS(140),
    [anon_sym_DASH] = ACTIONS(140),
    [anon_sym_SEMI] = ACTIONS(140),
    [anon_sym_EQ] = ACTIONS(140),
    [anon_sym_BANG] = ACTIONS(140),
    [anon_sym_QMARK] = ACTIONS(140),
    [anon_sym_AMP] = ACTIONS(140),
    [anon_sym_LBRACK] = ACTIONS(140),
    [anon_sym_RBRACK] = ACTIONS(140),
    [anon_sym_DQUOTE] = ACTIONS(140),
    [sym_number] = ACTIONS(140),
    [sym_comment] = ACTIONS(3),
  },
  [21] = {
    [ts_builtin_sym_end] = ACTIONS(144),
    [sym_identifier] = ACTIONS(146),
    [anon_sym_PERCENT] = ACTIONS(144),
    [anon_sym_RBRACE] = ACTIONS(144),
    [anon_sym_TILDE] = ACTIONS(144),
    [anon_sym_BRK] = ACTIONS(144),
    [anon_sym_INC] = ACTIONS(146),
    [anon_sym_POP] = ACTIONS(146),
    [anon_sym_NIP] = ACTIONS(146),
    [anon_sym_SWP] = ACTIONS(146),
    [anon_sym_ROT] = ACTIONS(146),
    [anon_sym_DUP] = ACTIONS(146),
    [anon_sym_OVR] = ACTIONS(146),
    [anon_sym_EQU] = ACTIONS(146),
    [anon_sym_NEQ] = ACTIONS(146),
    [anon_sym_GTH] = ACTIONS(146),
    [anon_sym_LTH] = ACTIONS(146),
    [anon_sym_JMP] = ACTIONS(146),
    [anon_sym_JCN] = ACTIONS(146),
    [anon_sym_JSR] = ACTIONS(146),
    [anon_sym_STH] = ACTIONS(146),
    [anon_sym_LDZ] = ACTIONS(146),
    [anon_sym_STZ] = ACTIONS(146),
    [anon_sym_LDR] = ACTIONS(146),
    [anon_sym_STR] = ACTIONS(146),
    [anon_sym_LDA] = ACTIONS(146),
    [anon_sym_STA] = ACTIONS(146),
    [anon_sym_DEI] = ACTIONS(146),
    [anon_sym_DEO] = ACTIONS(146),
    [anon_sym_ADD] = ACTIONS(146),
    [anon_sym_SUB] = ACTIONS(146),
    [anon_sym_MUL] = ACTIONS(146),
    [anon_sym_DIV] = ACTIONS(146),
    [anon_sym_AND] = ACTIONS(146),
    [anon_sym_ORA] = ACTIONS(146),
    [anon_sym_EOR] = ACTIONS(146),
    [anon_sym_SFT] = ACTIONS(146),
    [anon_sym_JCI] = ACTIONS(144),
    [anon_sym_INC2] = ACTIONS(146),
    [anon_sym_POP2] = ACTIONS(146),
    [anon_sym_NIP2] = ACTIONS(146),
    [anon_sym_SWP2] = ACTIONS(146),
    [anon_sym_ROT2] = ACTIONS(146),
    [anon_sym_DUP2] = ACTIONS(146),
    [anon_sym_OVR2] = ACTIONS(146),
    [anon_sym_EQU2] = ACTIONS(146),
    [anon_sym_NEQ2] = ACTIONS(146),
    [anon_sym_GTH2] = ACTIONS(146),
    [anon_sym_LTH2] = ACTIONS(146),
    [anon_sym_JMP2] = ACTIONS(146),
    [anon_sym_JCN2] = ACTIONS(146),
    [anon_sym_JSR2] = ACTIONS(146),
    [anon_sym_STH2] = ACTIONS(146),
    [anon_sym_LDZ2] = ACTIONS(146),
    [anon_sym_STZ2] = ACTIONS(146),
    [anon_sym_LDR2] = ACTIONS(146),
    [anon_sym_STR2] = ACTIONS(146),
    [anon_sym_LDA2] = ACTIONS(146),
    [anon_sym_STA2] = ACTIONS(146),
    [anon_sym_DEI2] = ACTIONS(146),
    [anon_sym_DEO2] = ACTIONS(146),
    [anon_sym_ADD2] = ACTIONS(146),
    [anon_sym_SUB2] = ACTIONS(146),
    [anon_sym_MUL2] = ACTIONS(146),
    [anon_sym_DIV2] = ACTIONS(146),
    [anon_sym_AND2] = ACTIONS(146),
    [anon_sym_ORA2] = ACTIONS(146),
    [anon_sym_EOR2] = ACTIONS(146),
    [anon_sym_SFT2] = ACTIONS(146),
    [anon_sym_JMI] = ACTIONS(144),
    [anon_sym_INCr] = ACTIONS(144),
    [anon_sym_POPr] = ACTIONS(144),
    [anon_sym_NIPr] = ACTIONS(144),
    [anon_sym_SWPr] = ACTIONS(144),
    [anon_sym_ROTr] = ACTIONS(144),
    [anon_sym_DUPr] = ACTIONS(144),
    [anon_sym_OVRr] = ACTIONS(144),
    [anon_sym_EQUr] = ACTIONS(144),
    [anon_sym_NEQr] = ACTIONS(144),
    [anon_sym_GTHr] = ACTIONS(144),
    [anon_sym_LTHr] = ACTIONS(144),
    [anon_sym_JMPr] = ACTIONS(144),
    [anon_sym_JCNr] = ACTIONS(144),
    [anon_sym_JSRr] = ACTIONS(144),
    [anon_sym_STHr] = ACTIONS(144),
    [anon_sym_LDZr] = ACTIONS(144),
    [anon_sym_STZr] = ACTIONS(144),
    [anon_sym_LDRr] = ACTIONS(144),
    [anon_sym_STRr] = ACTIONS(144),
    [anon_sym_LDAr] = ACTIONS(144),
    [anon_sym_STAr] = ACTIONS(144),
    [anon_sym_DEIr] = ACTIONS(144),
    [anon_sym_DEOr] = ACTIONS(144),
    [anon_sym_ADDr] = ACTIONS(144),
    [anon_sym_SUBr] = ACTIONS(144),
    [anon_sym_MULr] = ACTIONS(144),
    [anon_sym_DIVr] = ACTIONS(144),
    [anon_sym_ANDr] = ACTIONS(144),
    [anon_sym_ORAr] = ACTIONS(144),
    [anon_sym_EORr] = ACTIONS(144),
    [anon_sym_SFTr] = ACTIONS(144),
    [anon_sym_JSI] = ACTIONS(144),
    [anon_sym_INC2r] = ACTIONS(144),
    [anon_sym_POP2r] = ACTIONS(144),
    [anon_sym_NIP2r] = ACTIONS(144),
    [anon_sym_SWP2r] = ACTIONS(144),
    [anon_sym_ROT2r] = ACTIONS(144),
    [anon_sym_DUP2r] = ACTIONS(144),
    [anon_sym_OVR2r] = ACTIONS(144),
    [anon_sym_EQU2r] = ACTIONS(144),
    [anon_sym_NEQ2r] = ACTIONS(144),
    [anon_sym_GTH2r] = ACTIONS(144),
    [anon_sym_LTH2r] = ACTIONS(144),
    [anon_sym_JMP2r] = ACTIONS(144),
    [anon_sym_JCN2r] = ACTIONS(144),
    [anon_sym_JSR2r] = ACTIONS(144),
    [anon_sym_STH2r] = ACTIONS(144),
    [anon_sym_LDZ2r] = ACTIONS(144),
    [anon_sym_STZ2r] = ACTIONS(144),
    [anon_sym_LDR2r] = ACTIONS(144),
    [anon_sym_STR2r] = ACTIONS(144),
    [anon_sym_LDA2r] = ACTIONS(144),
    [anon_sym_STA2r] = ACTIONS(144),
    [anon_sym_DEI2r] = ACTIONS(144),
    [anon_sym_DEO2r] = ACTIONS(144),
    [anon_sym_ADD2r] = ACTIONS(144),
    [anon_sym_SUB2r] = ACTIONS(144),
    [anon_sym_MUL2r] = ACTIONS(144),
    [anon_sym_DIV2r] = ACTIONS(144),
    [anon_sym_AND2r] = ACTIONS(144),
    [anon_sym_ORA2r] = ACTIONS(144),
    [anon_sym_EOR2r] = ACTIONS(144),
    [anon_sym_SFT2r] = ACTIONS(144),
    [anon_sym_LIT] = ACTIONS(146),
    [anon_sym_INCk] = ACTIONS(146),
    [anon_sym_POPk] = ACTIONS(146),
    [anon_sym_NIPk] = ACTIONS(146),
    [anon_sym_SWPk] = ACTIONS(146),
    [anon_sym_ROTk] = ACTIONS(146),
    [anon_sym_DUPk] = ACTIONS(146),
    [anon_sym_OVRk] = ACTIONS(146),
    [anon_sym_EQUk] = ACTIONS(146),
    [anon_sym_NEQk] = ACTIONS(146),
    [anon_sym_GTHk] = ACTIONS(146),
    [anon_sym_LTHk] = ACTIONS(146),
    [anon_sym_JMPk] = ACTIONS(146),
    [anon_sym_JCNk] = ACTIONS(146),
    [anon_sym_JSRk] = ACTIONS(146),
    [anon_sym_STHk] = ACTIONS(146),
    [anon_sym_LDZk] = ACTIONS(146),
    [anon_sym_STZk] = ACTIONS(146),
    [anon_sym_LDRk] = ACTIONS(146),
    [anon_sym_STRk] = ACTIONS(146),
    [anon_sym_LDAk] = ACTIONS(146),
    [anon_sym_STAk] = ACTIONS(146),
    [anon_sym_DEIk] = ACTIONS(146),
    [anon_sym_DEOk] = ACTIONS(146),
    [anon_sym_ADDk] = ACTIONS(146),
    [anon_sym_SUBk] = ACTIONS(146),
    [anon_sym_MULk] = ACTIONS(146),
    [anon_sym_DIVk] = ACTIONS(146),
    [anon_sym_ANDk] = ACTIONS(146),
    [anon_sym_ORAk] = ACTIONS(146),
    [anon_sym_EORk] = ACTIONS(146),
    [anon_sym_SFTk] = ACTIONS(146),
    [anon_sym_LIT2] = ACTIONS(146),
    [anon_sym_INC2k] = ACTIONS(146),
    [anon_sym_POP2k] = ACTIONS(146),
    [anon_sym_NIP2k] = ACTIONS(146),
    [anon_sym_SWP2k] = ACTIONS(146),
    [anon_sym_ROT2k] = ACTIONS(146),
    [anon_sym_DUP2k] = ACTIONS(146),
    [anon_sym_OVR2k] = ACTIONS(146),
    [anon_sym_EQU2k] = ACTIONS(146),
    [anon_sym_NEQ2k] = ACTIONS(146),
    [anon_sym_GTH2k] = ACTIONS(146),
    [anon_sym_LTH2k] = ACTIONS(146),
    [anon_sym_JMP2k] = ACTIONS(146),
    [anon_sym_JCN2k] = ACTIONS(146),
    [anon_sym_JSR2k] = ACTIONS(146),
    [anon_sym_STH2k] = ACTIONS(146),
    [anon_sym_LDZ2k] = ACTIONS(146),
    [anon_sym_STZ2k] = ACTIONS(146),
    [anon_sym_LDR2k] = ACTIONS(146),
    [anon_sym_STR2k] = ACTIONS(146),
    [anon_sym_LDA2k] = ACTIONS(146),
    [anon_sym_STA2k] = ACTIONS(146),
    [anon_sym_DEI2k] = ACTIONS(146),
    [anon_sym_DEO2k] = ACTIONS(146),
    [anon_sym_ADD2k] = ACTIONS(146),
    [anon_sym_SUB2k] = ACTIONS(146),
    [anon_sym_MUL2k] = ACTIONS(146),
    [anon_sym_DIV2k] = ACTIONS(146),
    [anon_sym_AND2k] = ACTIONS(146),
    [anon_sym_ORA2k] = ACTIONS(146),
    [anon_sym_EOR2k] = ACTIONS(146),
    [anon_sym_SFT2k] = ACTIONS(146),
    [anon_sym_LITr] = ACTIONS(144),
    [anon_sym_INCkr] = ACTIONS(144),
    [anon_sym_POPkr] = ACTIONS(144),
    [anon_sym_NIPkr] = ACTIONS(144),
    [anon_sym_SWPkr] = ACTIONS(144),
    [anon_sym_ROTkr] = ACTIONS(144),
    [anon_sym_DUPkr] = ACTIONS(144),
    [anon_sym_OVRkr] = ACTIONS(144),
    [anon_sym_EQUkr] = ACTIONS(144),
    [anon_sym_NEQkr] = ACTIONS(144),
    [anon_sym_GTHkr] = ACTIONS(144),
    [anon_sym_LTHkr] = ACTIONS(144),
    [anon_sym_JMPkr] = ACTIONS(144),
    [anon_sym_JCNkr] = ACTIONS(144),
    [anon_sym_JSRkr] = ACTIONS(144),
    [anon_sym_STHkr] = ACTIONS(144),
    [anon_sym_LDZkr] = ACTIONS(144),
    [anon_sym_STZkr] = ACTIONS(144),
    [anon_sym_LDRkr] = ACTIONS(144),
    [anon_sym_STRkr] = ACTIONS(144),
    [anon_sym_LDAkr] = ACTIONS(144),
    [anon_sym_STAkr] = ACTIONS(144),
    [anon_sym_DEIkr] = ACTIONS(144),
    [anon_sym_DEOkr] = ACTIONS(144),
    [anon_sym_ADDkr] = ACTIONS(144),
    [anon_sym_SUBkr] = ACTIONS(144),
    [anon_sym_MULkr] = ACTIONS(144),
    [anon_sym_DIVkr] = ACTIONS(144),
    [anon_sym_ANDkr] = ACTIONS(144),
    [anon_sym_ORAkr] = ACTIONS(144),
    [anon_sym_EORkr] = ACTIONS(144),
    [anon_sym_SFTkr] = ACTIONS(144),
    [anon_sym_LIT2r] = ACTIONS(144),
    [anon_sym_INC2kr] = ACTIONS(144),
    [anon_sym_POP2kr] = ACTIONS(144),
    [anon_sym_NIP2kr] = ACTIONS(144),
    [anon_sym_SWP2kr] = ACTIONS(144),
    [anon_sym_ROT2kr] = ACTIONS(144),
    [anon_sym_DUP2kr] = ACTIONS(144),
    [anon_sym_OVR2kr] = ACTIONS(144),
    [anon_sym_EQU2kr] = ACTIONS(144),
    [anon_sym_NEQ2kr] = ACTIONS(144),
    [anon_sym_GTH2kr] = ACTIONS(144),
    [anon_sym_LTH2kr] = ACTIONS(144),
    [anon_sym_JMP2kr] = ACTIONS(144),
    [anon_sym_JCN2kr] = ACTIONS(144),
    [anon_sym_JSR2kr] = ACTIONS(144),
    [anon_sym_STH2kr] = ACTIONS(144),
    [anon_sym_LDZ2kr] = ACTIONS(144),
    [anon_sym_STZ2kr] = ACTIONS(144),
    [anon_sym_LDR2kr] = ACTIONS(144),
    [anon_sym_STR2kr] = ACTIONS(144),
    [anon_sym_LDA2kr] = ACTIONS(144),
    [anon_sym_STA2kr] = ACTIONS(144),
    [anon_sym_DEI2kr] = ACTIONS(144),
    [anon_sym_DEO2kr] = ACTIONS(144),
    [anon_sym_ADD2kr] = ACTIONS(144),
    [anon_sym_SUB2kr] = ACTIONS(144),
    [anon_sym_MUL2kr] = ACTIONS(144),
    [anon_sym_DIV2kr] = ACTIONS(144),
    [anon_sym_AND2kr] = ACTIONS(144),
    [anon_sym_ORA2kr] = ACTIONS(144),
    [anon_sym_EOR2kr] = ACTIONS(144),
    [anon_sym_SFT2kr] = ACTIONS(144),
    [anon_sym_PIPE] = ACTIONS(144),
    [anon_sym_DOLLAR] = ACTIONS(144),
    [anon_sym_POUND] = ACTIONS(144),
    [anon_sym_AT] = ACTIONS(144),
    [anon_sym_COMMA] = ACTIONS(144),
    [anon_sym__] = ACTIONS(144),
    [anon_sym_DOT] = ACTIONS(144),
    [anon_sym_DASH] = ACTIONS(144),
    [anon_sym_SEMI] = ACTIONS(144),
    [anon_sym_EQ] = ACTIONS(144),
    [anon_sym_BANG] = ACTIONS(144),
    [anon_sym_QMARK] = ACTIONS(144),
    [anon_sym_AMP] = ACTIONS(144),
    [anon_sym_LBRACK] = ACTIONS(144),
    [anon_sym_RBRACK] = ACTIONS(144),
    [anon_sym_DQUOTE] = ACTIONS(144),
    [sym_number] = ACTIONS(144),
    [sym_comment] = ACTIONS(3),
  },
  [22] = {
    [ts_builtin_sym_end] = ACTIONS(106),
    [sym_identifier] = ACTIONS(108),
    [anon_sym_PERCENT] = ACTIONS(106),
    [anon_sym_RBRACE] = ACTIONS(106),
    [anon_sym_TILDE] = ACTIONS(106),
    [anon_sym_BRK] = ACTIONS(106),
    [anon_sym_INC] = ACTIONS(108),
    [anon_sym_POP] = ACTIONS(108),
    [anon_sym_NIP] = ACTIONS(108),
    [anon_sym_SWP] = ACTIONS(108),
    [anon_sym_ROT] = ACTIONS(108),
    [anon_sym_DUP] = ACTIONS(108),
    [anon_sym_OVR] = ACTIONS(108),
    [anon_sym_EQU] = ACTIONS(108),
    [anon_sym_NEQ] = ACTIONS(108),
    [anon_sym_GTH] = ACTIONS(108),
    [anon_sym_LTH] = ACTIONS(108),
    [anon_sym_JMP] = ACTIONS(108),
    [anon_sym_JCN] = ACTIONS(108),
    [anon_sym_JSR] = ACTIONS(108),
    [anon_sym_STH] = ACTIONS(108),
    [anon_sym_LDZ] = ACTIONS(108),
    [anon_sym_STZ] = ACTIONS(108),
    [anon_sym_LDR] = ACTIONS(108),
    [anon_sym_STR] = ACTIONS(108),
    [anon_sym_LDA] = ACTIONS(108),
    [anon_sym_STA] = ACTIONS(108),
    [anon_sym_DEI] = ACTIONS(108),
    [anon_sym_DEO] = ACTIONS(108),
    [anon_sym_ADD] = ACTIONS(108),
    [anon_sym_SUB] = ACTIONS(108),
    [anon_sym_MUL] = ACTIONS(108),
    [anon_sym_DIV] = ACTIONS(108),
    [anon_sym_AND] = ACTIONS(108),
    [anon_sym_ORA] = ACTIONS(108),
    [anon_sym_EOR] = ACTIONS(108),
    [anon_sym_SFT] = ACTIONS(108),
    [anon_sym_JCI] = ACTIONS(106),
    [anon_sym_INC2] = ACTIONS(108),
    [anon_sym_POP2] = ACTIONS(108),
    [anon_sym_NIP2] = ACTIONS(108),
    [anon_sym_SWP2] = ACTIONS(108),
    [anon_sym_ROT2] = ACTIONS(108),
    [anon_sym_DUP2] = ACTIONS(108),
    [anon_sym_OVR2] = ACTIONS(108),
    [anon_sym_EQU2] = ACTIONS(108),
    [anon_sym_NEQ2] = ACTIONS(108),
    [anon_sym_GTH2] = ACTIONS(108),
    [anon_sym_LTH2] = ACTIONS(108),
    [anon_sym_JMP2] = ACTIONS(108),
    [anon_sym_JCN2] = ACTIONS(108),
    [anon_sym_JSR2] = ACTIONS(108),
    [anon_sym_STH2] = ACTIONS(108),
    [anon_sym_LDZ2] = ACTIONS(108),
    [anon_sym_STZ2] = ACTIONS(108),
    [anon_sym_LDR2] = ACTIONS(108),
    [anon_sym_STR2] = ACTIONS(108),
    [anon_sym_LDA2] = ACTIONS(108),
    [anon_sym_STA2] = ACTIONS(108),
    [anon_sym_DEI2] = ACTIONS(108),
    [anon_sym_DEO2] = ACTIONS(108),
    [anon_sym_ADD2] = ACTIONS(108),
    [anon_sym_SUB2] = ACTIONS(108),
    [anon_sym_MUL2] = ACTIONS(108),
    [anon_sym_DIV2] = ACTIONS(108),
    [anon_sym_AND2] = ACTIONS(108),
    [anon_sym_ORA2] = ACTIONS(108),
    [anon_sym_EOR2] = ACTIONS(108),
    [anon_sym_SFT2] = ACTIONS(108),
    [anon_sym_JMI] = ACTIONS(106),
    [anon_sym_INCr] = ACTIONS(106),
    [anon_sym_POPr] = ACTIONS(106),
    [anon_sym_NIPr] = ACTIONS(106),
    [anon_sym_SWPr] = ACTIONS(106),
    [anon_sym_ROTr] = ACTIONS(106),
    [anon_sym_DUPr] = ACTIONS(106),
    [anon_sym_OVRr] = ACTIONS(106),
    [anon_sym_EQUr] = ACTIONS(106),
    [anon_sym_NEQr] = ACTIONS(106),
    [anon_sym_GTHr] = ACTIONS(106),
    [anon_sym_LTHr] = ACTIONS(106),
    [anon_sym_JMPr] = ACTIONS(106),
    [anon_sym_JCNr] = ACTIONS(106),
    [anon_sym_JSRr] = ACTIONS(106),
    [anon_sym_STHr] = ACTIONS(106),
    [anon_sym_LDZr] = ACTIONS(106),
    [anon_sym_STZr] = ACTIONS(106),
    [anon_sym_LDRr] = ACTIONS(106),
    [anon_sym_STRr] = ACTIONS(106),
    [anon_sym_LDAr] = ACTIONS(106),
    [anon_sym_STAr] = ACTIONS(106),
    [anon_sym_DEIr] = ACTIONS(106),
    [anon_sym_DEOr] = ACTIONS(106),
    [anon_sym_ADDr] = ACTIONS(106),
    [anon_sym_SUBr] = ACTIONS(106),
    [anon_sym_MULr] = ACTIONS(106),
    [anon_sym_DIVr] = ACTIONS(106),
    [anon_sym_ANDr] = ACTIONS(106),
    [anon_sym_ORAr] = ACTIONS(106),
    [anon_sym_EORr] = ACTIONS(106),
    [anon_sym_SFTr] = ACTIONS(106),
    [anon_sym_JSI] = ACTIONS(106),
    [anon_sym_INC2r] = ACTIONS(106),
    [anon_sym_POP2r] = ACTIONS(106),
    [anon_sym_NIP2r] = ACTIONS(106),
    [anon_sym_SWP2r] = ACTIONS(106),
    [anon_sym_ROT2r] = ACTIONS(106),
    [anon_sym_DUP2r] = ACTIONS(106),
    [anon_sym_OVR2r] = ACTIONS(106),
    [anon_sym_EQU2r] = ACTIONS(106),
    [anon_sym_NEQ2r] = ACTIONS(106),
    [anon_sym_GTH2r] = ACTIONS(106),
    [anon_sym_LTH2r] = ACTIONS(106),
    [anon_sym_JMP2r] = ACTIONS(106),
    [anon_sym_JCN2r] = ACTIONS(106),
    [anon_sym_JSR2r] = ACTIONS(106),
    [anon_sym_STH2r] = ACTIONS(106),
    [anon_sym_LDZ2r] = ACTIONS(106),
    [anon_sym_STZ2r] = ACTIONS(106),
    [anon_sym_LDR2r] = ACTIONS(106),
    [anon_sym_STR2r] = ACTIONS(106),
    [anon_sym_LDA2r] = ACTIONS(106),
    [anon_sym_STA2r] = ACTIONS(106),
    [anon_sym_DEI2r] = ACTIONS(106),
    [anon_sym_DEO2r] = ACTIONS(106),
    [anon_sym_ADD2r] = ACTIONS(106),
    [anon_sym_SUB2r] = ACTIONS(106),
    [anon_sym_MUL2r] = ACTIONS(106),
    [anon_sym_DIV2r] = ACTIONS(106),
    [anon_sym_AND2r] = ACTIONS(106),
    [anon_sym_ORA2r] = ACTIONS(106),
    [anon_sym_EOR2r] = ACTIONS(106),
    [anon_sym_SFT2r] = ACTIONS(106),
    [anon_sym_LIT] = ACTIONS(108),
    [anon_sym_INCk] = ACTIONS(108),
    [anon_sym_POPk] = ACTIONS(108),
    [anon_sym_NIPk] = ACTIONS(108),
    [anon_sym_SWPk] = ACTIONS(108),
    [anon_sym_ROTk] = ACTIONS(108),
    [anon_sym_DUPk] = ACTIONS(108),
    [anon_sym_OVRk] = ACTIONS(108),
    [anon_sym_EQUk] = ACTIONS(108),
    [anon_sym_NEQk] = ACTIONS(108),
    [anon_sym_GTHk] = ACTIONS(108),
    [anon_sym_LTHk] = ACTIONS(108),
    [anon_sym_JMPk] = ACTIONS(108),
    [anon_sym_JCNk] = ACTIONS(108),
    [anon_sym_JSRk] = ACTIONS(108),
    [anon_sym_STHk] = ACTIONS(108),
    [anon_sym_LDZk] = ACTIONS(108),
    [anon_sym_STZk] = ACTIONS(108),
    [anon_sym_LDRk] = ACTIONS(108),
    [anon_sym_STRk] = ACTIONS(108),
    [anon_sym_LDAk] = ACTIONS(108),
    [anon_sym_STAk] = ACTIONS(108),
    [anon_sym_DEIk] = ACTIONS(108),
    [anon_sym_DEOk] = ACTIONS(108),
    [anon_sym_ADDk] = ACTIONS(108),
    [anon_sym_SUBk] = ACTIONS(108),
    [anon_sym_MULk] = ACTIONS(108),
    [anon_sym_DIVk] = ACTIONS(108),
    [anon_sym_ANDk] = ACTIONS(108),
    [anon_sym_ORAk] = ACTIONS(108),
    [anon_sym_EORk] = ACTIONS(108),
    [anon_sym_SFTk] = ACTIONS(108),
    [anon_sym_LIT2] = ACTIONS(108),
    [anon_sym_INC2k] = ACTIONS(108),
    [anon_sym_POP2k] = ACTIONS(108),
    [anon_sym_NIP2k] = ACTIONS(108),
    [anon_sym_SWP2k] = ACTIONS(108),
    [anon_sym_ROT2k] = ACTIONS(108),
    [anon_sym_DUP2k] = ACTIONS(108),
    [anon_sym_OVR2k] = ACTIONS(108),
    [anon_sym_EQU2k] = ACTIONS(108),
    [anon_sym_NEQ2k] = ACTIONS(108),
    [anon_sym_GTH2k] = ACTIONS(108),
    [anon_sym_LTH2k] = ACTIONS(108),
    [anon_sym_JMP2k] = ACTIONS(108),
    [anon_sym_JCN2k] = ACTIONS(108),
    [anon_sym_JSR2k] = ACTIONS(108),
    [anon_sym_STH2k] = ACTIONS(108),
    [anon_sym_LDZ2k] = ACTIONS(108),
    [anon_sym_STZ2k] = ACTIONS(108),
    [anon_sym_LDR2k] = ACTIONS(108),
    [anon_sym_STR2k] = ACTIONS(108),
    [anon_sym_LDA2k] = ACTIONS(108),
    [anon_sym_STA2k] = ACTIONS(108),
    [anon_sym_DEI2k] = ACTIONS(108),
    [anon_sym_DEO2k] = ACTIONS(108),
    [anon_sym_ADD2k] = ACTIONS(108),
    [anon_sym_SUB2k] = ACTIONS(108),
    [anon_sym_MUL2k] = ACTIONS(108),
    [anon_sym_DIV2k] = ACTIONS(108),
    [anon_sym_AND2k] = ACTIONS(108),
    [anon_sym_ORA2k] = ACTIONS(108),
    [anon_sym_EOR2k] = ACTIONS(108),
    [anon_sym_SFT2k] = ACTIONS(108),
    [anon_sym_LITr] = ACTIONS(106),
    [anon_sym_INCkr] = ACTIONS(106),
    [anon_sym_POPkr] = ACTIONS(106),
    [anon_sym_NIPkr] = ACTIONS(106),
    [anon_sym_SWPkr] = ACTIONS(106),
    [anon_sym_ROTkr] = ACTIONS(106),
    [anon_sym_DUPkr] = ACTIONS(106),
    [anon_sym_OVRkr] = ACTIONS(106),
    [anon_sym_EQUkr] = ACTIONS(106),
    [anon_sym_NEQkr] = ACTIONS(106),
    [anon_sym_GTHkr] = ACTIONS(106),
    [anon_sym_LTHkr] = ACTIONS(106),
    [anon_sym_JMPkr] = ACTIONS(106),
    [anon_sym_JCNkr] = ACTIONS(106),
    [anon_sym_JSRkr] = ACTIONS(106),
    [anon_sym_STHkr] = ACTIONS(106),
    [anon_sym_LDZkr] = ACTIONS(106),
    [anon_sym_STZkr] = ACTIONS(106),
    [anon_sym_LDRkr] = ACTIONS(106),
    [anon_sym_STRkr] = ACTIONS(106),
    [anon_sym_LDAkr] = ACTIONS(106),
    [anon_sym_STAkr] = ACTIONS(106),
    [anon_sym_DEIkr] = ACTIONS(106),
    [anon_sym_DEOkr] = ACTIONS(106),
    [anon_sym_ADDkr] = ACTIONS(106),
    [anon_sym_SUBkr] = ACTIONS(106),
    [anon_sym_MULkr] = ACTIONS(106),
    [anon_sym_DIVkr] = ACTIONS(106),
    [anon_sym_ANDkr] = ACTIONS(106),
    [anon_sym_ORAkr] = ACTIONS(106),
    [anon_sym_EORkr] = ACTIONS(106),
    [anon_sym_SFTkr] = ACTIONS(106),
    [anon_sym_LIT2r] = ACTIONS(106),
    [anon_sym_INC2kr] = ACTIONS(106),
    [anon_sym_POP2kr] = ACTIONS(106),
    [anon_sym_NIP2kr] = ACTIONS(106),
    [anon_sym_SWP2kr] = ACTIONS(106),
    [anon_sym_ROT2kr] = ACTIONS(106),
    [anon_sym_DUP2kr] = ACTIONS(106),
    [anon_sym_OVR2kr] = ACTIONS(106),
    [anon_sym_EQU2kr] = ACTIONS(106),
    [anon_sym_NEQ2kr] = ACTIONS(106),
    [anon_sym_GTH2kr] = ACTIONS(106),
    [anon_sym_LTH2kr] = ACTIONS(106),
    [anon_sym_JMP2kr] = ACTIONS(106),
    [anon_sym_JCN2kr] = ACTIONS(106),
    [anon_sym_JSR2kr] = ACTIONS(106),
    [anon_sym_STH2kr] = ACTIONS(106),
    [anon_sym_LDZ2kr] = ACTIONS(106),
    [anon_sym_STZ2kr] = ACTIONS(106),
    [anon_sym_LDR2kr] = ACTIONS(106),
    [anon_sym_STR2kr] = ACTIONS(106),
    [anon_sym_LDA2kr] = ACTIONS(106),
    [anon_sym_STA2kr] = ACTIONS(106),
    [anon_sym_DEI2kr] = ACTIONS(106),
    [anon_sym_DEO2kr] = ACTIONS(106),
    [anon_sym_ADD2kr] = ACTIONS(106),
    [anon_sym_SUB2kr] = ACTIONS(106),
    [anon_sym_MUL2kr] = ACTIONS(106),
    [anon_sym_DIV2kr] = ACTIONS(106),
    [anon_sym_AND2kr] = ACTIONS(106),
    [anon_sym_ORA2kr] = ACTIONS(106),
    [anon_sym_EOR2kr] = ACTIONS(106),
    [anon_sym_SFT2kr] = ACTIONS(106),
    [anon_sym_PIPE] = ACTIONS(106),
    [anon_sym_DOLLAR] = ACTIONS(106),
    [anon_sym_POUND] = ACTIONS(106),
    [anon_sym_AT] = ACTIONS(106),
    [anon_sym_COMMA] = ACTIONS(106),
    [anon_sym__] = ACTIONS(106),
    [anon_sym_DOT] = ACTIONS(106),
    [anon_sym_DASH] = ACTIONS(106),
    [anon_sym_SEMI] = ACTIONS(106),
    [anon_sym_EQ] = ACTIONS(106),
    [anon_sym_BANG] = ACTIONS(106),
    [anon_sym_QMARK] = ACTIONS(106),
    [anon_sym_AMP] = ACTIONS(106),
    [anon_sym_LBRACK] = ACTIONS(106),
    [anon_sym_RBRACK] = ACTIONS(106),
    [anon_sym_DQUOTE] = ACTIONS(106),
    [sym_number] = ACTIONS(106),
    [sym_comment] = ACTIONS(3),
  },
  [23] = {
    [ts_builtin_sym_end] = ACTIONS(148),
    [sym_identifier] = ACTIONS(150),
    [anon_sym_PERCENT] = ACTIONS(148),
    [anon_sym_TILDE] = ACTIONS(148),
    [anon_sym_BRK] = ACTIONS(148),
    [anon_sym_INC] = ACTIONS(150),
    [anon_sym_POP] = ACTIONS(150),
    [anon_sym_NIP] = ACTIONS(150),
    [anon_sym_SWP] = ACTIONS(150),
    [anon_sym_ROT] = ACTIONS(150),
    [anon_sym_DUP] = ACTIONS(150),
    [anon_sym_OVR] = ACTIONS(150),
    [anon_sym_EQU] = ACTIONS(150),
    [anon_sym_NEQ] = ACTIONS(150),
    [anon_sym_GTH] = ACTIONS(150),
    [anon_sym_LTH] = ACTIONS(150),
    [anon_sym_JMP] = ACTIONS(150),
    [anon_sym_JCN] = ACTIONS(150),
    [anon_sym_JSR] = ACTIONS(150),
    [anon_sym_STH] = ACTIONS(150),
    [anon_sym_LDZ] = ACTIONS(150),
    [anon_sym_STZ] = ACTIONS(150),
    [anon_sym_LDR] = ACTIONS(150),
    [anon_sym_STR] = ACTIONS(150),
    [anon_sym_LDA] = ACTIONS(150),
    [anon_sym_STA] = ACTIONS(150),
    [anon_sym_DEI] = ACTIONS(150),
    [anon_sym_DEO] = ACTIONS(150),
    [anon_sym_ADD] = ACTIONS(150),
    [anon_sym_SUB] = ACTIONS(150),
    [anon_sym_MUL] = ACTIONS(150),
    [anon_sym_DIV] = ACTIONS(150),
    [anon_sym_AND] = ACTIONS(150),
    [anon_sym_ORA] = ACTIONS(150),
    [anon_sym_EOR] = ACTIONS(150),
    [anon_sym_SFT] = ACTIONS(150),
    [anon_sym_JCI] = ACTIONS(148),
    [anon_sym_INC2] = ACTIONS(150),
    [anon_sym_POP2] = ACTIONS(150),
    [anon_sym_NIP2] = ACTIONS(150),
    [anon_sym_SWP2] = ACTIONS(150),
    [anon_sym_ROT2] = ACTIONS(150),
    [anon_sym_DUP2] = ACTIONS(150),
    [anon_sym_OVR2] = ACTIONS(150),
    [anon_sym_EQU2] = ACTIONS(150),
    [anon_sym_NEQ2] = ACTIONS(150),
    [anon_sym_GTH2] = ACTIONS(150),
    [anon_sym_LTH2] = ACTIONS(150),
    [anon_sym_JMP2] = ACTIONS(150),
    [anon_sym_JCN2] = ACTIONS(150),
    [anon_sym_JSR2] = ACTIONS(150),
    [anon_sym_STH2] = ACTIONS(150),
    [anon_sym_LDZ2] = ACTIONS(150),
    [anon_sym_STZ2] = ACTIONS(150),
    [anon_sym_LDR2] = ACTIONS(150),
    [anon_sym_STR2] = ACTIONS(150),
    [anon_sym_LDA2] = ACTIONS(150),
    [anon_sym_STA2] = ACTIONS(150),
    [anon_sym_DEI2] = ACTIONS(150),
    [anon_sym_DEO2] = ACTIONS(150),
    [anon_sym_ADD2] = ACTIONS(150),
    [anon_sym_SUB2] = ACTIONS(150),
    [anon_sym_MUL2] = ACTIONS(150),
    [anon_sym_DIV2] = ACTIONS(150),
    [anon_sym_AND2] = ACTIONS(150),
    [anon_sym_ORA2] = ACTIONS(150),
    [anon_sym_EOR2] = ACTIONS(150),
    [anon_sym_SFT2] = ACTIONS(150),
    [anon_sym_JMI] = ACTIONS(148),
    [anon_sym_INCr] = ACTIONS(148),
    [anon_sym_POPr] = ACTIONS(148),
    [anon_sym_NIPr] = ACTIONS(148),
    [anon_sym_SWPr] = ACTIONS(148),
    [anon_sym_ROTr] = ACTIONS(148),
    [anon_sym_DUPr] = ACTIONS(148),
    [anon_sym_OVRr] = ACTIONS(148),
    [anon_sym_EQUr] = ACTIONS(148),
    [anon_sym_NEQr] = ACTIONS(148),
    [anon_sym_GTHr] = ACTIONS(148),
    [anon_sym_LTHr] = ACTIONS(148),
    [anon_sym_JMPr] = ACTIONS(148),
    [anon_sym_JCNr] = ACTIONS(148),
    [anon_sym_JSRr] = ACTIONS(148),
    [anon_sym_STHr] = ACTIONS(148),
    [anon_sym_LDZr] = ACTIONS(148),
    [anon_sym_STZr] = ACTIONS(148),
    [anon_sym_LDRr] = ACTIONS(148),
    [anon_sym_STRr] = ACTIONS(148),
    [anon_sym_LDAr] = ACTIONS(148),
    [anon_sym_STAr] = ACTIONS(148),
    [anon_sym_DEIr] = ACTIONS(148),
    [anon_sym_DEOr] = ACTIONS(148),
    [anon_sym_ADDr] = ACTIONS(148),
    [anon_sym_SUBr] = ACTIONS(148),
    [anon_sym_MULr] = ACTIONS(148),
    [anon_sym_DIVr] = ACTIONS(148),
    [anon_sym_ANDr] = ACTIONS(148),
    [anon_sym_ORAr] = ACTIONS(148),
    [anon_sym_EORr] = ACTIONS(148),
    [anon_sym_SFTr] = ACTIONS(148),
    [anon_sym_JSI] = ACTIONS(148),
    [anon_sym_INC2r] = ACTIONS(148),
    [anon_sym_POP2r] = ACTIONS(148),
    [anon_sym_NIP2r] = ACTIONS(148),
    [anon_sym_SWP2r] = ACTIONS(148),
    [anon_sym_ROT2r] = ACTIONS(148),
    [anon_sym_DUP2r] = ACTIONS(148),
    [anon_sym_OVR2r] = ACTIONS(148),
    [anon_sym_EQU2r] = ACTIONS(148),
    [anon_sym_NEQ2r] = ACTIONS(148),
    [anon_sym_GTH2r] = ACTIONS(148),
    [anon_sym_LTH2r] = ACTIONS(148),
    [anon_sym_JMP2r] = ACTIONS(148),
    [anon_sym_JCN2r] = ACTIONS(148),
    [anon_sym_JSR2r] = ACTIONS(148),
    [anon_sym_STH2r] = ACTIONS(148),
    [anon_sym_LDZ2r] = ACTIONS(148),
    [anon_sym_STZ2r] = ACTIONS(148),
    [anon_sym_LDR2r] = ACTIONS(148),
    [anon_sym_STR2r] = ACTIONS(148),
    [anon_sym_LDA2r] = ACTIONS(148),
    [anon_sym_STA2r] = ACTIONS(148),
    [anon_sym_DEI2r] = ACTIONS(148),
    [anon_sym_DEO2r] = ACTIONS(148),
    [anon_sym_ADD2r] = ACTIONS(148),
    [anon_sym_SUB2r] = ACTIONS(148),
    [anon_sym_MUL2r] = ACTIONS(148),
    [anon_sym_DIV2r] = ACTIONS(148),
    [anon_sym_AND2r] = ACTIONS(148),
    [anon_sym_ORA2r] = ACTIONS(148),
    [anon_sym_EOR2r] = ACTIONS(148),
    [anon_sym_SFT2r] = ACTIONS(148),
    [anon_sym_LIT] = ACTIONS(150),
    [anon_sym_INCk] = ACTIONS(150),
    [anon_sym_POPk] = ACTIONS(150),
    [anon_sym_NIPk] = ACTIONS(150),
    [anon_sym_SWPk] = ACTIONS(150),
    [anon_sym_ROTk] = ACTIONS(150),
    [anon_sym_DUPk] = ACTIONS(150),
    [anon_sym_OVRk] = ACTIONS(150),
    [anon_sym_EQUk] = ACTIONS(150),
    [anon_sym_NEQk] = ACTIONS(150),
    [anon_sym_GTHk] = ACTIONS(150),
    [anon_sym_LTHk] = ACTIONS(150),
    [anon_sym_JMPk] = ACTIONS(150),
    [anon_sym_JCNk] = ACTIONS(150),
    [anon_sym_JSRk] = ACTIONS(150),
    [anon_sym_STHk] = ACTIONS(150),
    [anon_sym_LDZk] = ACTIONS(150),
    [anon_sym_STZk] = ACTIONS(150),
    [anon_sym_LDRk] = ACTIONS(150),
    [anon_sym_STRk] = ACTIONS(150),
    [anon_sym_LDAk] = ACTIONS(150),
    [anon_sym_STAk] = ACTIONS(150),
    [anon_sym_DEIk] = ACTIONS(150),
    [anon_sym_DEOk] = ACTIONS(150),
    [anon_sym_ADDk] = ACTIONS(150),
    [anon_sym_SUBk] = ACTIONS(150),
    [anon_sym_MULk] = ACTIONS(150),
    [anon_sym_DIVk] = ACTIONS(150),
    [anon_sym_ANDk] = ACTIONS(150),
    [anon_sym_ORAk] = ACTIONS(150),
    [anon_sym_EORk] = ACTIONS(150),
    [anon_sym_SFTk] = ACTIONS(150),
    [anon_sym_LIT2] = ACTIONS(150),
    [anon_sym_INC2k] = ACTIONS(150),
    [anon_sym_POP2k] = ACTIONS(150),
    [anon_sym_NIP2k] = ACTIONS(150),
    [anon_sym_SWP2k] = ACTIONS(150),
    [anon_sym_ROT2k] = ACTIONS(150),
    [anon_sym_DUP2k] = ACTIONS(150),
    [anon_sym_OVR2k] = ACTIONS(150),
    [anon_sym_EQU2k] = ACTIONS(150),
    [anon_sym_NEQ2k] = ACTIONS(150),
    [anon_sym_GTH2k] = ACTIONS(150),
    [anon_sym_LTH2k] = ACTIONS(150),
    [anon_sym_JMP2k] = ACTIONS(150),
    [anon_sym_JCN2k] = ACTIONS(150),
    [anon_sym_JSR2k] = ACTIONS(150),
    [anon_sym_STH2k] = ACTIONS(150),
    [anon_sym_LDZ2k] = ACTIONS(150),
    [anon_sym_STZ2k] = ACTIONS(150),
    [anon_sym_LDR2k] = ACTIONS(150),
    [anon_sym_STR2k] = ACTIONS(150),
    [anon_sym_LDA2k] = ACTIONS(150),
    [anon_sym_STA2k] = ACTIONS(150),
    [anon_sym_DEI2k] = ACTIONS(150),
    [anon_sym_DEO2k] = ACTIONS(150),
    [anon_sym_ADD2k] = ACTIONS(150),
    [anon_sym_SUB2k] = ACTIONS(150),
    [anon_sym_MUL2k] = ACTIONS(150),
    [anon_sym_DIV2k] = ACTIONS(150),
    [anon_sym_AND2k] = ACTIONS(150),
    [anon_sym_ORA2k] = ACTIONS(150),
    [anon_sym_EOR2k] = ACTIONS(150),
    [anon_sym_SFT2k] = ACTIONS(150),
    [anon_sym_LITr] = ACTIONS(148),
    [anon_sym_INCkr] = ACTIONS(148),
    [anon_sym_POPkr] = ACTIONS(148),
    [anon_sym_NIPkr] = ACTIONS(148),
    [anon_sym_SWPkr] = ACTIONS(148),
    [anon_sym_ROTkr] = ACTIONS(148),
    [anon_sym_DUPkr] = ACTIONS(148),
    [anon_sym_OVRkr] = ACTIONS(148),
    [anon_sym_EQUkr] = ACTIONS(148),
    [anon_sym_NEQkr] = ACTIONS(148),
    [anon_sym_GTHkr] = ACTIONS(148),
    [anon_sym_LTHkr] = ACTIONS(148),
    [anon_sym_JMPkr] = ACTIONS(148),
    [anon_sym_JCNkr] = ACTIONS(148),
    [anon_sym_JSRkr] = ACTIONS(148),
    [anon_sym_STHkr] = ACTIONS(148),
    [anon_sym_LDZkr] = ACTIONS(148),
    [anon_sym_STZkr] = ACTIONS(148),
    [anon_sym_LDRkr] = ACTIONS(148),
    [anon_sym_STRkr] = ACTIONS(148),
    [anon_sym_LDAkr] = ACTIONS(148),
    [anon_sym_STAkr] = ACTIONS(148),
    [anon_sym_DEIkr] = ACTIONS(148),
    [anon_sym_DEOkr] = ACTIONS(148),
    [anon_sym_ADDkr] = ACTIONS(148),
    [anon_sym_SUBkr] = ACTIONS(148),
    [anon_sym_MULkr] = ACTIONS(148),
    [anon_sym_DIVkr] = ACTIONS(148),
    [anon_sym_ANDkr] = ACTIONS(148),
    [anon_sym_ORAkr] = ACTIONS(148),
    [anon_sym_EORkr] = ACTIONS(148),
    [anon_sym_SFTkr] = ACTIONS(148),
    [anon_sym_LIT2r] = ACTIONS(148),
    [anon_sym_INC2kr] = ACTIONS(148),
    [anon_sym_POP2kr] = ACTIONS(148),
    [anon_sym_NIP2kr] = ACTIONS(148),
    [anon_sym_SWP2kr] = ACTIONS(148),
    [anon_sym_ROT2kr] = ACTIONS(148),
    [anon_sym_DUP2kr] = ACTIONS(148),
    [anon_sym_OVR2kr] = ACTIONS(148),
    [anon_sym_EQU2kr] = ACTIONS(148),
    [anon_sym_NEQ2kr] = ACTIONS(148),
    [anon_sym_GTH2kr] = ACTIONS(148),
    [anon_sym_LTH2kr] = ACTIONS(148),
    [anon_sym_JMP2kr] = ACTIONS(148),
    [anon_sym_JCN2kr] = ACTIONS(148),
    [anon_sym_JSR2kr] = ACTIONS(148),
    [anon_sym_STH2kr] = ACTIONS(148),
    [anon_sym_LDZ2kr] = ACTIONS(148),
    [anon_sym_STZ2kr] = ACTIONS(148),
    [anon_sym_LDR2kr] = ACTIONS(148),
    [anon_sym_STR2kr] = ACTIONS(148),
    [anon_sym_LDA2kr] = ACTIONS(148),
    [anon_sym_STA2kr] = ACTIONS(148),
    [anon_sym_DEI2kr] = ACTIONS(148),
    [anon_sym_DEO2kr] = ACTIONS(148),
    [anon_sym_ADD2kr] = ACTIONS(148),
    [anon_sym_SUB2kr] = ACTIONS(148),
    [anon_sym_MUL2kr] = ACTIONS(148),
    [anon_sym_DIV2kr] = ACTIONS(148),
    [anon_sym_AND2kr] = ACTIONS(148),
    [anon_sym_ORA2kr] = ACTIONS(148),
    [anon_sym_EOR2kr] = ACTIONS(148),
    [anon_sym_SFT2kr] = ACTIONS(148),
    [anon_sym_PIPE] = ACTIONS(148),
    [anon_sym_DOLLAR] = ACTIONS(148),
    [anon_sym_POUND] = ACTIONS(148),
    [anon_sym_AT] = ACTIONS(148),
    [anon_sym_COMMA] = ACTIONS(148),
    [anon_sym__] = ACTIONS(148),
    [anon_sym_DOT] = ACTIONS(148),
    [anon_sym_DASH] = ACTIONS(148),
    [anon_sym_SEMI] = ACTIONS(148),
    [anon_sym_EQ] = ACTIONS(148),
    [anon_sym_BANG] = ACTIONS(148),
    [anon_sym_QMARK] = ACTIONS(148),
    [anon_sym_AMP] = ACTIONS(148),
    [anon_sym_LBRACK] = ACTIONS(148),
    [anon_sym_DQUOTE] = ACTIONS(148),
    [sym_number] = ACTIONS(148),
    [sym_comment] = ACTIONS(3),
  },
  [24] = {
    [ts_builtin_sym_end] = ACTIONS(152),
    [sym_identifier] = ACTIONS(154),
    [anon_sym_PERCENT] = ACTIONS(152),
    [anon_sym_TILDE] = ACTIONS(152),
    [anon_sym_BRK] = ACTIONS(152),
    [anon_sym_INC] = ACTIONS(154),
    [anon_sym_POP] = ACTIONS(154),
    [anon_sym_NIP] = ACTIONS(154),
    [anon_sym_SWP] = ACTIONS(154),
    [anon_sym_ROT] = ACTIONS(154),
    [anon_sym_DUP] = ACTIONS(154),
    [anon_sym_OVR] = ACTIONS(154),
    [anon_sym_EQU] = ACTIONS(154),
    [anon_sym_NEQ] = ACTIONS(154),
    [anon_sym_GTH] = ACTIONS(154),
    [anon_sym_LTH] = ACTIONS(154),
    [anon_sym_JMP] = ACTIONS(154),
    [anon_sym_JCN] = ACTIONS(154),
    [anon_sym_JSR] = ACTIONS(154),
    [anon_sym_STH] = ACTIONS(154),
    [anon_sym_LDZ] = ACTIONS(154),
    [anon_sym_STZ] = ACTIONS(154),
    [anon_sym_LDR] = ACTIONS(154),
    [anon_sym_STR] = ACTIONS(154),
    [anon_sym_LDA] = ACTIONS(154),
    [anon_sym_STA] = ACTIONS(154),
    [anon_sym_DEI] = ACTIONS(154),
    [anon_sym_DEO] = ACTIONS(154),
    [anon_sym_ADD] = ACTIONS(154),
    [anon_sym_SUB] = ACTIONS(154),
    [anon_sym_MUL] = ACTIONS(154),
    [anon_sym_DIV] = ACTIONS(154),
    [anon_sym_AND] = ACTIONS(154),
    [anon_sym_ORA] = ACTIONS(154),
    [anon_sym_EOR] = ACTIONS(154),
    [anon_sym_SFT] = ACTIONS(154),
    [anon_sym_JCI] = ACTIONS(152),
    [anon_sym_INC2] = ACTIONS(154),
    [anon_sym_POP2] = ACTIONS(154),
    [anon_sym_NIP2] = ACTIONS(154),
    [anon_sym_SWP2] = ACTIONS(154),
    [anon_sym_ROT2] = ACTIONS(154),
    [anon_sym_DUP2] = ACTIONS(154),
    [anon_sym_OVR2] = ACTIONS(154),
    [anon_sym_EQU2] = ACTIONS(154),
    [anon_sym_NEQ2] = ACTIONS(154),
    [anon_sym_GTH2] = ACTIONS(154),
    [anon_sym_LTH2] = ACTIONS(154),
    [anon_sym_JMP2] = ACTIONS(154),
    [anon_sym_JCN2] = ACTIONS(154),
    [anon_sym_JSR2] = ACTIONS(154),
    [anon_sym_STH2] = ACTIONS(154),
    [anon_sym_LDZ2] = ACTIONS(154),
    [anon_sym_STZ2] = ACTIONS(154),
    [anon_sym_LDR2] = ACTIONS(154),
    [anon_sym_STR2] = ACTIONS(154),
    [anon_sym_LDA2] = ACTIONS(154),
    [anon_sym_STA2] = ACTIONS(154),
    [anon_sym_DEI2] = ACTIONS(154),
    [anon_sym_DEO2] = ACTIONS(154),
    [anon_sym_ADD2] = ACTIONS(154),
    [anon_sym_SUB2] = ACTIONS(154),
    [anon_sym_MUL2] = ACTIONS(154),
    [anon_sym_DIV2] = ACTIONS(154),
    [anon_sym_AND2] = ACTIONS(154),
    [anon_sym_ORA2] = ACTIONS(154),
    [anon_sym_EOR2] = ACTIONS(154),
    [anon_sym_SFT2] = ACTIONS(154),
    [anon_sym_JMI] = ACTIONS(152),
    [anon_sym_INCr] = ACTIONS(152),
    [anon_sym_POPr] = ACTIONS(152),
    [anon_sym_NIPr] = ACTIONS(152),
    [anon_sym_SWPr] = ACTIONS(152),
    [anon_sym_ROTr] = ACTIONS(152),
    [anon_sym_DUPr] = ACTIONS(152),
    [anon_sym_OVRr] = ACTIONS(152),
    [anon_sym_EQUr] = ACTIONS(152),
    [anon_sym_NEQr] = ACTIONS(152),
    [anon_sym_GTHr] = ACTIONS(152),
    [anon_sym_LTHr] = ACTIONS(152),
    [anon_sym_JMPr] = ACTIONS(152),
    [anon_sym_JCNr] = ACTIONS(152),
    [anon_sym_JSRr] = ACTIONS(152),
    [anon_sym_STHr] = ACTIONS(152),
    [anon_sym_LDZr] = ACTIONS(152),
    [anon_sym_STZr] = ACTIONS(152),
    [anon_sym_LDRr] = ACTIONS(152),
    [anon_sym_STRr] = ACTIONS(152),
    [anon_sym_LDAr] = ACTIONS(152),
    [anon_sym_STAr] = ACTIONS(152),
    [anon_sym_DEIr] = ACTIONS(152),
    [anon_sym_DEOr] = ACTIONS(152),
    [anon_sym_ADDr] = ACTIONS(152),
    [anon_sym_SUBr] = ACTIONS(152),
    [anon_sym_MULr] = ACTIONS(152),
    [anon_sym_DIVr] = ACTIONS(152),
    [anon_sym_ANDr] = ACTIONS(152),
    [anon_sym_ORAr] = ACTIONS(152),
    [anon_sym_EORr] = ACTIONS(152),
    [anon_sym_SFTr] = ACTIONS(152),
    [anon_sym_JSI] = ACTIONS(152),
    [anon_sym_INC2r] = ACTIONS(152),
    [anon_sym_POP2r] = ACTIONS(152),
    [anon_sym_NIP2r] = ACTIONS(152),
    [anon_sym_SWP2r] = ACTIONS(152),
    [anon_sym_ROT2r] = ACTIONS(152),
    [anon_sym_DUP2r] = ACTIONS(152),
    [anon_sym_OVR2r] = ACTIONS(152),
    [anon_sym_EQU2r] = ACTIONS(152),
    [anon_sym_NEQ2r] = ACTIONS(152),
    [anon_sym_GTH2r] = ACTIONS(152),
    [anon_sym_LTH2r] = ACTIONS(152),
    [anon_sym_JMP2r] = ACTIONS(152),
    [anon_sym_JCN2r] = ACTIONS(152),
    [anon_sym_JSR2r] = ACTIONS(152),
    [anon_sym_STH2r] = ACTIONS(152),
    [anon_sym_LDZ2r] = ACTIONS(152),
    [anon_sym_STZ2r] = ACTIONS(152),
    [anon_sym_LDR2r] = ACTIONS(152),
    [anon_sym_STR2r] = ACTIONS(152),
    [anon_sym_LDA2r] = ACTIONS(152),
    [anon_sym_STA2r] = ACTIONS(152),
    [anon_sym_DEI2r] = ACTIONS(152),
    [anon_sym_DEO2r] = ACTIONS(152),
    [anon_sym_ADD2r] = ACTIONS(152),
    [anon_sym_SUB2r] = ACTIONS(152),
    [anon_sym_MUL2r] = ACTIONS(152),
    [anon_sym_DIV2r] = ACTIONS(152),
    [anon_sym_AND2r] = ACTIONS(152),
    [anon_sym_ORA2r] = ACTIONS(152),
    [anon_sym_EOR2r] = ACTIONS(152),
    [anon_sym_SFT2r] = ACTIONS(152),
    [anon_sym_LIT] = ACTIONS(154),
    [anon_sym_INCk] = ACTIONS(154),
    [anon_sym_POPk] = ACTIONS(154),
    [anon_sym_NIPk] = ACTIONS(154),
    [anon_sym_SWPk] = ACTIONS(154),
    [anon_sym_ROTk] = ACTIONS(154),
    [anon_sym_DUPk] = ACTIONS(154),
    [anon_sym_OVRk] = ACTIONS(154),
    [anon_sym_EQUk] = ACTIONS(154),
    [anon_sym_NEQk] = ACTIONS(154),
    [anon_sym_GTHk] = ACTIONS(154),
    [anon_sym_LTHk] = ACTIONS(154),
    [anon_sym_JMPk] = ACTIONS(154),
    [anon_sym_JCNk] = ACTIONS(154),
    [anon_sym_JSRk] = ACTIONS(154),
    [anon_sym_STHk] = ACTIONS(154),
    [anon_sym_LDZk] = ACTIONS(154),
    [anon_sym_STZk] = ACTIONS(154),
    [anon_sym_LDRk] = ACTIONS(154),
    [anon_sym_STRk] = ACTIONS(154),
    [anon_sym_LDAk] = ACTIONS(154),
    [anon_sym_STAk] = ACTIONS(154),
    [anon_sym_DEIk] = ACTIONS(154),
    [anon_sym_DEOk] = ACTIONS(154),
    [anon_sym_ADDk] = ACTIONS(154),
    [anon_sym_SUBk] = ACTIONS(154),
    [anon_sym_MULk] = ACTIONS(154),
    [anon_sym_DIVk] = ACTIONS(154),
    [anon_sym_ANDk] = ACTIONS(154),
    [anon_sym_ORAk] = ACTIONS(154),
    [anon_sym_EORk] = ACTIONS(154),
    [anon_sym_SFTk] = ACTIONS(154),
    [anon_sym_LIT2] = ACTIONS(154),
    [anon_sym_INC2k] = ACTIONS(154),
    [anon_sym_POP2k] = ACTIONS(154),
    [anon_sym_NIP2k] = ACTIONS(154),
    [anon_sym_SWP2k] = ACTIONS(154),
    [anon_sym_ROT2k] = ACTIONS(154),
    [anon_sym_DUP2k] = ACTIONS(154),
    [anon_sym_OVR2k] = ACTIONS(154),
    [anon_sym_EQU2k] = ACTIONS(154),
    [anon_sym_NEQ2k] = ACTIONS(154),
    [anon_sym_GTH2k] = ACTIONS(154),
    [anon_sym_LTH2k] = ACTIONS(154),
    [anon_sym_JMP2k] = ACTIONS(154),
    [anon_sym_JCN2k] = ACTIONS(154),
    [anon_sym_JSR2k] = ACTIONS(154),
    [anon_sym_STH2k] = ACTIONS(154),
    [anon_sym_LDZ2k] = ACTIONS(154),
    [anon_sym_STZ2k] = ACTIONS(154),
    [anon_sym_LDR2k] = ACTIONS(154),
    [anon_sym_STR2k] = ACTIONS(154),
    [anon_sym_LDA2k] = ACTIONS(154),
    [anon_sym_STA2k] = ACTIONS(154),
    [anon_sym_DEI2k] = ACTIONS(154),
    [anon_sym_DEO2k] = ACTIONS(154),
    [anon_sym_ADD2k] = ACTIONS(154),
    [anon_sym_SUB2k] = ACTIONS(154),
    [anon_sym_MUL2k] = ACTIONS(154),
    [anon_sym_DIV2k] = ACTIONS(154),
    [anon_sym_AND2k] = ACTIONS(154),
    [anon_sym_ORA2k] = ACTIONS(154),
    [anon_sym_EOR2k] = ACTIONS(154),
    [anon_sym_SFT2k] = ACTIONS(154),
    [anon_sym_LITr] = ACTIONS(152),
    [anon_sym_INCkr] = ACTIONS(152),
    [anon_sym_POPkr] = ACTIONS(152),
    [anon_sym_NIPkr] = ACTIONS(152),
    [anon_sym_SWPkr] = ACTIONS(152),
    [anon_sym_ROTkr] = ACTIONS(152),
    [anon_sym_DUPkr] = ACTIONS(152),
    [anon_sym_OVRkr] = ACTIONS(152),
    [anon_sym_EQUkr] = ACTIONS(152),
    [anon_sym_NEQkr] = ACTIONS(152),
    [anon_sym_GTHkr] = ACTIONS(152),
    [anon_sym_LTHkr] = ACTIONS(152),
    [anon_sym_JMPkr] = ACTIONS(152),
    [anon_sym_JCNkr] = ACTIONS(152),
    [anon_sym_JSRkr] = ACTIONS(152),
    [anon_sym_STHkr] = ACTIONS(152),
    [anon_sym_LDZkr] = ACTIONS(152),
    [anon_sym_STZkr] = ACTIONS(152),
    [anon_sym_LDRkr] = ACTIONS(152),
    [anon_sym_STRkr] = ACTIONS(152),
    [anon_sym_LDAkr] = ACTIONS(152),
    [anon_sym_STAkr] = ACTIONS(152),
    [anon_sym_DEIkr] = ACTIONS(152),
    [anon_sym_DEOkr] = ACTIONS(152),
    [anon_sym_ADDkr] = ACTIONS(152),
    [anon_sym_SUBkr] = ACTIONS(152),
    [anon_sym_MULkr] = ACTIONS(152),
    [anon_sym_DIVkr] = ACTIONS(152),
    [anon_sym_ANDkr] = ACTIONS(152),
    [anon_sym_ORAkr] = ACTIONS(152),
    [anon_sym_EORkr] = ACTIONS(152),
    [anon_sym_SFTkr] = ACTIONS(152),
    [anon_sym_LIT2r] = ACTIONS(152),
    [anon_sym_INC2kr] = ACTIONS(152),
    [anon_sym_POP2kr] = ACTIONS(152),
    [anon_sym_NIP2kr] = ACTIONS(152),
    [anon_sym_SWP2kr] = ACTIONS(152),
    [anon_sym_ROT2kr] = ACTIONS(152),
    [anon_sym_DUP2kr] = ACTIONS(152),
    [anon_sym_OVR2kr] = ACTIONS(152),
    [anon_sym_EQU2kr] = ACTIONS(152),
    [anon_sym_NEQ2kr] = ACTIONS(152),
    [anon_sym_GTH2kr] = ACTIONS(152),
    [anon_sym_LTH2kr] = ACTIONS(152),
    [anon_sym_JMP2kr] = ACTIONS(152),
    [anon_sym_JCN2kr] = ACTIONS(152),
    [anon_sym_JSR2kr] = ACTIONS(152),
    [anon_sym_STH2kr] = ACTIONS(152),
    [anon_sym_LDZ2kr] = ACTIONS(152),
    [anon_sym_STZ2kr] = ACTIONS(152),
    [anon_sym_LDR2kr] = ACTIONS(152),
    [anon_sym_STR2kr] = ACTIONS(152),
    [anon_sym_LDA2kr] = ACTIONS(152),
    [anon_sym_STA2kr] = ACTIONS(152),
    [anon_sym_DEI2kr] = ACTIONS(152),
    [anon_sym_DEO2kr] = ACTIONS(152),
    [anon_sym_ADD2kr] = ACTIONS(152),
    [anon_sym_SUB2kr] = ACTIONS(152),
    [anon_sym_MUL2kr] = ACTIONS(152),
    [anon_sym_DIV2kr] = ACTIONS(152),
    [anon_sym_AND2kr] = ACTIONS(152),
    [anon_sym_ORA2kr] = ACTIONS(152),
    [anon_sym_EOR2kr] = ACTIONS(152),
    [anon_sym_SFT2kr] = ACTIONS(152),
    [anon_sym_PIPE] = ACTIONS(152),
    [anon_sym_DOLLAR] = ACTIONS(152),
    [anon_sym_POUND] = ACTIONS(152),
    [anon_sym_AT] = ACTIONS(152),
    [anon_sym_COMMA] = ACTIONS(152),
    [anon_sym__] = ACTIONS(152),
    [anon_sym_DOT] = ACTIONS(152),
    [anon_sym_DASH] = ACTIONS(152),
    [anon_sym_SEMI] = ACTIONS(152),
    [anon_sym_EQ] = ACTIONS(152),
    [anon_sym_BANG] = ACTIONS(152),
    [anon_sym_QMARK] = ACTIONS(152),
    [anon_sym_AMP] = ACTIONS(152),
    [anon_sym_LBRACK] = ACTIONS(152),
    [anon_sym_DQUOTE] = ACTIONS(152),
    [sym_number] = ACTIONS(152),
    [sym_comment] = ACTIONS(3),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(76), 1,
      anon_sym_AMP,
    ACTIONS(156), 1,
      sym_identifier,
    ACTIONS(158), 1,
      aux_sym_rune_token1,
    STATE(14), 1,
      sym_sublabel_reference,
    ACTIONS(68), 2,
      anon_sym_COMMA,
      anon_sym__,
    ACTIONS(70), 2,
      anon_sym_DOT,
      anon_sym_DASH,
    ACTIONS(72), 2,
      anon_sym_SEMI,
      anon_sym_EQ,
    ACTIONS(74), 2,
      anon_sym_BANG,
      anon_sym_QMARK,
    STATE(26), 2,
      sym_rune_char,
      aux_sym_rune_repeat1,
  [36] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(76), 1,
      anon_sym_AMP,
    ACTIONS(160), 1,
      sym_identifier,
    ACTIONS(162), 1,
      aux_sym_rune_token1,
    STATE(22), 1,
      sym_sublabel_reference,
    ACTIONS(68), 2,
      anon_sym_COMMA,
      anon_sym__,
    ACTIONS(70), 2,
      anon_sym_DOT,
      anon_sym_DASH,
    ACTIONS(72), 2,
      anon_sym_SEMI,
      anon_sym_EQ,
    ACTIONS(74), 2,
      anon_sym_BANG,
      anon_sym_QMARK,
    STATE(27), 2,
      sym_rune_char,
      aux_sym_rune_repeat1,
  [72] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(164), 1,
      sym_identifier,
    ACTIONS(166), 1,
      aux_sym_rune_token1,
    ACTIONS(180), 1,
      anon_sym_AMP,
    ACTIONS(168), 2,
      anon_sym_COMMA,
      anon_sym__,
    ACTIONS(171), 2,
      anon_sym_DOT,
      anon_sym_DASH,
    ACTIONS(174), 2,
      anon_sym_SEMI,
      anon_sym_EQ,
    ACTIONS(177), 2,
      anon_sym_BANG,
      anon_sym_QMARK,
    STATE(27), 2,
      sym_rune_char,
      aux_sym_rune_repeat1,
  [105] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(183), 1,
      ts_builtin_sym_end,
    ACTIONS(185), 1,
      anon_sym_PERCENT,
    ACTIONS(188), 1,
      anon_sym_TILDE,
    ACTIONS(191), 1,
      anon_sym_PIPE,
    ACTIONS(194), 1,
      anon_sym_AT,
    STATE(3), 1,
      sym_absolute_pad_operation,
    STATE(4), 1,
      sym_label,
    STATE(28), 5,
      sym_memory_execution,
      sym_subroutine,
      sym_macro,
      sym_include,
      aux_sym_program_repeat1,
  [137] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(7), 1,
      anon_sym_PERCENT,
    ACTIONS(9), 1,
      anon_sym_TILDE,
    ACTIONS(11), 1,
      anon_sym_PIPE,
    ACTIONS(13), 1,
      anon_sym_AT,
    ACTIONS(197), 1,
      ts_builtin_sym_end,
    STATE(3), 1,
      sym_absolute_pad_operation,
    STATE(4), 1,
      sym_label,
    STATE(28), 5,
      sym_memory_execution,
      sym_subroutine,
      sym_macro,
      sym_include,
      aux_sym_program_repeat1,
  [169] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(199), 1,
      sym_identifier,
    ACTIONS(201), 10,
      aux_sym_rune_token1,
      anon_sym_COMMA,
      anon_sym__,
      anon_sym_DOT,
      anon_sym_DASH,
      anon_sym_SEMI,
      anon_sym_EQ,
      anon_sym_BANG,
      anon_sym_QMARK,
      anon_sym_AMP,
  [188] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(203), 1,
      sym_identifier,
    ACTIONS(205), 10,
      aux_sym_rune_token1,
      anon_sym_COMMA,
      anon_sym__,
      anon_sym_DOT,
      anon_sym_DASH,
      anon_sym_SEMI,
      anon_sym_EQ,
      anon_sym_BANG,
      anon_sym_QMARK,
      anon_sym_AMP,
  [207] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(207), 1,
      sym_identifier,
    ACTIONS(209), 10,
      aux_sym_rune_token1,
      anon_sym_COMMA,
      anon_sym__,
      anon_sym_DOT,
      anon_sym_DASH,
      anon_sym_SEMI,
      anon_sym_EQ,
      anon_sym_BANG,
      anon_sym_QMARK,
      anon_sym_AMP,
  [226] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(211), 1,
      sym_identifier,
    ACTIONS(213), 10,
      aux_sym_rune_token1,
      anon_sym_COMMA,
      anon_sym__,
      anon_sym_DOT,
      anon_sym_DASH,
      anon_sym_SEMI,
      anon_sym_EQ,
      anon_sym_BANG,
      anon_sym_QMARK,
      anon_sym_AMP,
  [245] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(215), 1,
      sym_identifier,
    ACTIONS(217), 10,
      aux_sym_rune_token1,
      anon_sym_COMMA,
      anon_sym__,
      anon_sym_DOT,
      anon_sym_DASH,
      anon_sym_SEMI,
      anon_sym_EQ,
      anon_sym_BANG,
      anon_sym_QMARK,
      anon_sym_AMP,
  [264] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(219), 5,
      ts_builtin_sym_end,
      anon_sym_PERCENT,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_AT,
  [275] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(221), 5,
      ts_builtin_sym_end,
      anon_sym_PERCENT,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_AT,
  [286] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(223), 5,
      ts_builtin_sym_end,
      anon_sym_PERCENT,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_AT,
  [297] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(225), 1,
      sym_identifier,
    ACTIONS(227), 1,
      aux_sym_macro_token1,
  [307] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(229), 1,
      aux_sym_raw_ascii_token1,
  [314] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(231), 1,
      anon_sym_LBRACE,
  [321] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(233), 1,
      sym_number,
  [328] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(235), 1,
      ts_builtin_sym_end,
  [335] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(237), 1,
      sym_identifier,
  [342] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(239), 1,
      sym_identifier,
  [349] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(241), 1,
      aux_sym_include_token1,
  [356] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(243), 1,
      sym_number,
  [363] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(245), 1,
      aux_sym_hex_literal_token1,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(25)] = 0,
  [SMALL_STATE(26)] = 36,
  [SMALL_STATE(27)] = 72,
  [SMALL_STATE(28)] = 105,
  [SMALL_STATE(29)] = 137,
  [SMALL_STATE(30)] = 169,
  [SMALL_STATE(31)] = 188,
  [SMALL_STATE(32)] = 207,
  [SMALL_STATE(33)] = 226,
  [SMALL_STATE(34)] = 245,
  [SMALL_STATE(35)] = 264,
  [SMALL_STATE(36)] = 275,
  [SMALL_STATE(37)] = 286,
  [SMALL_STATE(38)] = 297,
  [SMALL_STATE(39)] = 307,
  [SMALL_STATE(40)] = 314,
  [SMALL_STATE(41)] = 321,
  [SMALL_STATE(42)] = 328,
  [SMALL_STATE(43)] = 335,
  [SMALL_STATE(44)] = 342,
  [SMALL_STATE(45)] = 349,
  [SMALL_STATE(46)] = 356,
  [SMALL_STATE(47)] = 363,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 0, 0, 0),
  [7] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [9] = {.entry = {.count = 1, .reusable = true}}, SHIFT(45),
  [11] = {.entry = {.count = 1, .reusable = true}}, SHIFT(41),
  [13] = {.entry = {.count = 1, .reusable = true}}, SHIFT(44),
  [15] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0),
  [17] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(13),
  [20] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(15),
  [23] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(15),
  [26] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(46),
  [29] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(47),
  [32] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(34),
  [35] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(33),
  [38] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(32),
  [41] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(31),
  [44] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(30),
  [47] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(7),
  [50] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(39),
  [53] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_memory_execution_repeat1, 2, 0, 0), SHIFT_REPEAT(2),
  [56] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_memory_execution, 1, 0, 0),
  [58] = {.entry = {.count = 1, .reusable = false}}, SHIFT(13),
  [60] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [62] = {.entry = {.count = 1, .reusable = false}}, SHIFT(15),
  [64] = {.entry = {.count = 1, .reusable = true}}, SHIFT(46),
  [66] = {.entry = {.count = 1, .reusable = true}}, SHIFT(47),
  [68] = {.entry = {.count = 1, .reusable = true}}, SHIFT(34),
  [70] = {.entry = {.count = 1, .reusable = true}}, SHIFT(33),
  [72] = {.entry = {.count = 1, .reusable = true}}, SHIFT(32),
  [74] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [76] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [78] = {.entry = {.count = 1, .reusable = true}}, SHIFT(7),
  [80] = {.entry = {.count = 1, .reusable = true}}, SHIFT(39),
  [82] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [84] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine, 1, 0, 0),
  [86] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [88] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine, 2, 0, 0),
  [90] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2),
  [92] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_memory_execution, 2, 0, 0),
  [94] = {.entry = {.count = 1, .reusable = true}}, SHIFT(19),
  [96] = {.entry = {.count = 1, .reusable = true}}, SHIFT(9),
  [98] = {.entry = {.count = 1, .reusable = true}}, SHIFT(35),
  [100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(17),
  [102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(37),
  [104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [106] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rune, 3, 0, 6),
  [108] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rune, 3, 0, 6),
  [110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(43),
  [112] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rune, 2, 0, 6),
  [114] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rune, 2, 0, 6),
  [116] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__non_toplevel_statement, 1, 0, 0),
  [118] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__non_toplevel_statement, 1, 0, 0),
  [120] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_opcode, 1, 0, 0),
  [122] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_opcode, 1, 0, 0),
  [124] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sublabel_reference, 3, 0, 0),
  [126] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sublabel_reference, 3, 0, 0),
  [128] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_brackets, 3, 0, 0),
  [130] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_brackets, 3, 0, 0),
  [132] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raw_ascii, 2, 0, 0),
  [134] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_raw_ascii, 2, 0, 0),
  [136] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_brackets, 2, 0, 0),
  [138] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_brackets, 2, 0, 0),
  [140] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hex_literal, 2, 0, 0),
  [142] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hex_literal, 2, 0, 0),
  [144] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_relative_pad_operation, 2, 0, 0),
  [146] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_relative_pad_operation, 2, 0, 0),
  [148] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_absolute_pad_operation, 2, 0, 0),
  [150] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_absolute_pad_operation, 2, 0, 0),
  [152] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_label, 2, 0, 0),
  [154] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_label, 2, 0, 0),
  [156] = {.entry = {.count = 1, .reusable = false}}, SHIFT(12),
  [158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(14),
  [160] = {.entry = {.count = 1, .reusable = false}}, SHIFT(11),
  [162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [164] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_rune_repeat1, 2, 0, 0),
  [166] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_rune_repeat1, 2, 0, 0),
  [168] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_rune_repeat1, 2, 0, 0), SHIFT_REPEAT(34),
  [171] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_rune_repeat1, 2, 0, 0), SHIFT_REPEAT(33),
  [174] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_rune_repeat1, 2, 0, 0), SHIFT_REPEAT(32),
  [177] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_rune_repeat1, 2, 0, 0), SHIFT_REPEAT(31),
  [180] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_rune_repeat1, 2, 0, 0), SHIFT_REPEAT(30),
  [183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2, 0, 0),
  [185] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2, 0, 0), SHIFT_REPEAT(38),
  [188] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2, 0, 0), SHIFT_REPEAT(45),
  [191] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2, 0, 0), SHIFT_REPEAT(41),
  [194] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2, 0, 0), SHIFT_REPEAT(44),
  [197] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 1, 0, 0),
  [199] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rune_char, 1, 0, 5),
  [201] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rune_char, 1, 0, 5),
  [203] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rune_char, 1, 0, 4),
  [205] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rune_char, 1, 0, 4),
  [207] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rune_char, 1, 0, 3),
  [209] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rune_char, 1, 0, 3),
  [211] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rune_char, 1, 0, 2),
  [213] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rune_char, 1, 0, 2),
  [215] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rune_char, 1, 0, 1),
  [217] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rune_char, 1, 0, 1),
  [219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro, 5, 0, 0),
  [221] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_include, 2, 0, 0),
  [223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro, 4, 0, 0),
  [225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(40),
  [227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [235] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(16),
  [239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(24),
  [241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(36),
  [243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(21),
  [245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
};

enum ts_external_scanner_symbol_identifiers {
  ts_external_token_comment = 0,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token_comment] = sym_comment,
};

static const bool ts_external_scanner_states[2][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token_comment] = true,
  },
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_uxntal_external_scanner_create(void);
void tree_sitter_uxntal_external_scanner_destroy(void *);
bool tree_sitter_uxntal_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_uxntal_external_scanner_serialize(void *, char *);
void tree_sitter_uxntal_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#else
#define TS_PUBLIC __attribute__((visibility("default")))
#endif

TS_PUBLIC const TSLanguage *tree_sitter_uxntal(void) {
  static const TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_uxntal_external_scanner_create,
      tree_sitter_uxntal_external_scanner_destroy,
      tree_sitter_uxntal_external_scanner_scan,
      tree_sitter_uxntal_external_scanner_serialize,
      tree_sitter_uxntal_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
